/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.base.internal.gdc.AbstractNode"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.gdc.GfxWidget"); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$

dojo.declare(
	"com.ibm.relm.explorer.web.base.internal.gdc.AbstractNode", //$NON-NLS-1$
	com.ibm.relm.explorer.web.base.internal.gdc.GfxWidget,
{
	left: 0, // Integer
	top: 0, // Integer
	width: 200, // Integer
	height: 150, // Integer
	fill: "", // String //$NON-NLS-1$
	stroke: "", // String //$NON-NLS-1$

	isGfxNodeWidget: true,
	doRefresh: true,

	postCreate: function(){
		this.domNode.isGfxNodeWidget = true; // [custom property]
		this.domNode.style.position = "absolute"; //$NON-NLS-1$
		try {
			this.domNode.style.top = this.top + "px"; //$NON-NLS-1$
			this.domNode.style.left = this.left + "px"; //$NON-NLS-1$
		}catch(e){
			// IE8 causes problem form Image node // TODO
		}
		this.inherited(arguments);
	},

	// subclass must implement
	getShapeInfo: function(){
		return null;
	},

	// subclass may override
	editModeChanged: function( /* boolean */ editMode){
		// subclass should override if necessary
	},
	
	// subclass may override
	update: function(props){
		this.inherited(arguments);
		this.domNode._offsetWidth = this.width; // [custom property]
		this.domNode._offsetHeight = this.height; // [custom property]
		this.setShape();
		this.setFill(this.fill);
		this.setStroke(this.stroke);
		this.setPosition();
		var editor = this.hlp_getEditor ? this.hlp_getEditor() : null;
		if(editor){
			this.editModeChanged(editor._editMode);
		}
	},

	// subclass may override
	updatePosition: function(){
		this.setPosition();
	},
	
	_layoutReady: true,
	
	setLayoutReady: function(ready){
		this._layoutReady = ready;
	},

	setPosition: function(){
		if(!this.group){ return; }
		var old = this.group.getTransform();
		if(!old || old.dx != this.left || old.dy != this.top){
			this.group.setTransform({ dx:this.left, dy:this.top });
			if(this.nodeMoved && this._layoutReady){
				this.nodeMoved(this);
			}
		}
	},

	setShape: function(){
		if(!this.shape){ return; }
		var shapeInfo = this.getShapeInfo(); // js object
		if(!shapeInfo){ return; }
		var current = this.shape.getShape(); // js object
		if(current == null){ return null; }	// Group
		
		for(var prop in shapeInfo){
			if(shapeInfo[prop] != current[prop]){
				// something has been changed
				return this.shape.setShape(shapeInfo);
			}
		}
		return this.shape;
	},

	nodeMoved: function(/*Widget*/widget){
	},

	getBB: function(/*String*/text, /*String|Object*/font){
		var span = dojo.create("span", {innerHTML:jazz.util.html.escape(text)}, dojo.body()); //$NON-NLS-1$
		var props = dojo.mixin({}, dojox.gfx.defaultFont);
		font = font || {};
		if(typeof(font) == "string"){ //$NON-NLS-1$
			font = {style:font};
		}
		dojo.mixin(props, font);
		for(var p in props){
			var cap = p.charAt(0).toUpperCase() + p.substring(1);
			span.style["font"+cap] = props[p]; //$NON-NLS-1$
		}
		var dim = {w:span.offsetWidth, h:span.offsetHeight};
		dojo.body().removeChild(span);
		return dim;
	},

// ======================= Setters/Getters =================================
	getFill: function(){
		return this.fill;
	},
	
	_strToG: function(str){
		var g = str;
		if(dojo.isString(str) && (str != "")){ //$NON-NLS-1$
			str = dojo.trim(str);
			if(str[0]=="{" || str[0]=="["){//$NON-NLS-1$//$NON-NLS-2$
				g = dojo.fromJson(str);
			}else{
				g = new dojo.Color(str);
			}
		}
		return g;
	},

	setFill: function(/*String|Array|dojo.Color*/fill){
		this.fill = fill;
		if(!this.shape){ return; }

		try {
			var g = this._strToG(fill);
			this.shape.setFill(g);
		}catch(e){
			console.error("Failed to setFill"); //$NON-NLS-1$ 
		}
	},

	getStroke: function(){
		return this.stroke;
	},

	setStroke: function(/*String|Array|dojo.Color*/stroke){
		this.stroke = stroke; // see dojox.gfx.defaultStroke
		if(!this.shape){ return; }

		try {
			var g = this._strToG(stroke);
			this.shape.setStroke(g);
		}catch(e){
			console.error("Failed to setStroke"); //$NON-NLS-1$ 
		}
	},
	
	setTopLeft: function(top, left){
		var newTop = Number(top);
		var newLeft = Number(left);
		if(newTop != this.top || newLeft != this.left){
			this.doRefresh = false;
			this.setTop(top);
			this.doRefresh = true;
			this.setLeft(left);
		}
	},


	getTop: function(){
		return this.top;
	},

	setTop: function(/*Integer*/top){
		this.top = Number(top);
		this.domNode.style.top = top + "px"; //$NON-NLS-1$
		if(this.doRefresh){
			this.setPosition();
		}
	},


	getLeft: function(){
		return this.left;
	},

	setLeft: function(/*Integer*/left){
		this.left = Number(left);
		this.domNode.style.left = left + "px"; //$NON-NLS-1$
		if(this.doRefresh){
			this.setPosition();
		}
	},


	getWidth: function(){
		return this.width;
	},
	
	setWidth: function(/*Integer*/width){
		this.width = Number(width);
		if(this.doRefresh && this.shape && this.shape.getShape().width != width){
			this.shape.setShape({width:width});
		}
	},


	getHeight: function(){
		return this.height;
	},
	
	setHeight: function(/*Integer*/height){
		this.height = Number(height);
		if(this.doRefresh && this.shape && this.shape.getShape().height != height){
			this.shape.setShape({height:height});
		}
	}	
});
