/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.explorer.queryparams.paramprompt.DataPrompt"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.explorer.queryparams.paramprompt", "DataPromptMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.util"); //$NON-NLS-1$

(function(){

var escapeHtml = com.ibm.relm.common.web.ui.util.escapeHtml;

dojo.declare(
	"com.ibm.relm.explorer.web.explorer.queryparams.paramprompt.DataPrompt", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ],
	{
		templatePath: "", //$NON-NLS-1$
		widgetsInTemplate: false,
		messages: dojo.i18n.getLocalization('com.ibm.relm.explorer.web.explorer.queryparams.paramprompt', 'DataPromptMessages'), //$NON-NLS-1$ //$NON-NLS-2$

		name: "", //$NON-NLS-1$

		/**
		 * If this is passed in, the prompt will be pre-filled with a value
		 */
		value: null, 

		constructor: function(args, node) {
			this.inputId = dijit.getUniqueId('com.ibm.relm.explorer.web.explorer.queryparams.paramprompt.Input'); //$NON-NLS-1$
		},

		postMixInProperties: function() {
			this.prettyLabel = dojo.replace(this.messages.labelX, [escapeHtml(this.label)]);
		},

		postCreate: function() {
			dojo.addClass(this.domNode, "relm-explorer-paramPrompt-row"); //$NON-NLS-1$
			this.connect(this.input, "onChange", this._onChange); //$NON-NLS-1$
			this.connect(this.input, "onchange", this._onChange); //$NON-NLS-1$
			this.connect(this.input, "onkeyup", this._onChange); //$NON-NLS-1$
			this.connect(this.input, "onKeyUp", this._onChange); //$NON-NLS-1$
		},

		startup: function() {
			this.inherited(arguments);

			if (this.value) {
				this.fillValue(this.value);
			}
		},

		/**
		* This should be overridden by subclasses. Subclasses should also call this.inherited at the end
		* or alternatively call this.onChange(this, this.getValue()).  
		* 
		*/
		_onChange: function() {
			this.onChange();
		},

		/**
		* This is the onChange handler that can be used to monitor whether 
		* all prompts have a value or not by a subscriber (generally ParameterPromptDialog). 
		* 
		*/
		onChange: function() {
			//No-op
		},

		fillValue: function(value) {
			throw new Error('subclass must override'); //$NON-NLS-1$
		},
		
		/**
		 * Subclass must override this. Should return the value expected for the
		 * appropriate data type.
		 * 
		 * @return {Object{type: String, value: Sting|boolean|Object}} value
		 */
		getValue: function() {
			throw new Error('subclass must override'); //$NON-NLS-1$
		},

		/**
		 * Returns the name value pair associated with this prompt
		 *
		 * @param existingValues An object on which to attach the name and value.
		 * @param passThroughIfNoValue If the value is unset, use the parameter
		 * 		string itself ($param$) as the value (prevents a StringTemplate
		 * 		error).
		 * @returns {parameterName: parameterValue}
		 */
		 getNameAndValue: function(existingValues, passThroughIfNoValue) {
		 	var value = this.getValue();
		 	
		 	// a value is not set
		 	if (!value.value && passThroughIfNoValue) {
		 		value.value = '$' + this.name + '$'; //$NON-NLS-1$ //$NON-NLS-2$
		 	}
		 	
		 	existingValues[this.name] = value;
		 	return existingValues;
		 }
	});

	})();