/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit.form.ComboBox"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-2$ //$NON-NLS-1$

(function() { 

var util = com.ibm.relm.explorer.web.ui.internal.util;

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent", [dijit._Widget, dijit._Templated] , { //$NON-NLS-1$
//
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/templates/AddHoverPropertyContent.html"),  //$NON-NLS-1$  //$NON-NLS-2$
	
	messages: null,
	queries: [],
	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);

		// add "query" field
		var self = this;
		dojo.forEach(this.queries, function(queryTemplate) {
			var optionChild = dojo.doc.createElement("option"); //$NON-NLS-1$
			optionChild.innerHTML =  util.escapeHtml(queryTemplate.id);
			optionChild.value= queryTemplate.id;
			self.queryField.appendChild(optionChild);
		});

	},
	
	getValues: function(){
		var typeProperty = this.typeProperty.checked;

		return {
			label: this.labelField.value,
			layout: this.layoutField.value,
			query: (typeProperty || this.queryField.value == this.messages.configEditorResourceHoverDefaultQuery) ? "" : this.queryField.value, //$NON-NLS-1$
			name: (typeProperty && this.nameField.value) || ""//$NON-NLS-1$
		};
	},
	
	valueTypeChanged: function(){
		var typeProperty = this.typeProperty.checked;
		this.valueTypeQueryRow.style.display = typeProperty ? "none" : "";  //$NON-NLS-1$ //$NON-NLS-2$
		this.valueTypePropertyRow.style.display = !typeProperty ? "none" : "";  //$NON-NLS-1$ //$NON-NLS-2$
	}
});


com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent.openDialog = function(resource, queries, callback2){

	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent(
			{queries: queries});

	// show dialog
	new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: contentWidget.messages.AddHoverProperty,
		onOpen: function(){
			dijit.focus(contentWidget.labelField);
		},
		validate: function(values, callback){
			var label = values["label"]; //$NON-NLS-1$
			if(label && label.length>0){
				var prop = null;
				var hoverProps = resource.hoverProps;
				if(hoverProps) {
					for(var i = 0; i < hoverProps.length; i++) {
						if(hoverProps[i].label == label) {
							prop = hoverProps[i];
							break;
						}
					}
				}
				if(prop) {
					alert(contentWidget.messages.errPropertyNameExists);
				} else {
					callback();
				}
			}else{
				alert(contentWidget.messages.errPropertyNameEmpty);
			}
		},
		onOK: function(values){
			if(resource.hoverProps == null) {
				resource.hoverProps = [];
			}

			resource.hoverProps.push({label: values["label"] , //$NON-NLS-1$
							layout: values["layout"] , //$NON-NLS-1$
							baseQueryId: values["query"] , //$NON-NLS-1$
							name: values["name"] }); //$NON-NLS-1$
			if(callback2) {
				callback2();
			}
		}
	});
};

})();