/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.CascadingComboBox"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "PropertyDialogUtilsMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.explorer.queryparams.paramprompt", "DataPromptMessages"); //$NON-NLS-1$ //$NON-NLS-2$

// for tentative message mixin
dojo.requireLocalization("com.ibm.relm.explorer.web.explorer.queryparams", "ParameterTableMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "FillPropertyDialogContentMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { 

var util = com.ibm.relm.explorer.web.ui.internal.util;
var enumUtils = com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils;
var escapeHtml = jazz.util.html.escape;

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent", [dijit._Widget, dijit._Templated] , { //$NON-NLS-1$

	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/templates/ParameterEditDialog.html"),  //$NON-NLS-1$  //$NON-NLS-2$
	
	messages:null,
	nameOptions:null,
	valueOptions:null,
	paramName: null,
	param: null,
	fExtended:false, // if true, show type and mandatory fields
	allowNameEdit:true,

	constructor: function(args){
		this._valueComboId = dijit.getUniqueId('valueCombo');//$NON-NLS-1$ 
		this._valueSelectId = dijit.getUniqueId('valueSelect');//$NON-NLS-1$ 
		this._nameFieldId = dijit.getUniqueId('nameField');//$NON-NLS-1$
		this._enumId = dijit.getUniqueId('enumId');//$NON-NLS-1$
	},
	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		
		var dlgMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "PropertyDialogUtilsMessages"); //$NON-NLS-1$  //$NON-NLS-2$
		var paramMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.explorer.queryparams", "ParameterTableMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		var fillMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "FillPropertyDialogContentMessages"); //$NON-NLS-1$  //$NON-NLS-2$
		var dataMessages = dojo.i18n.getLocalization('com.ibm.relm.explorer.web.explorer.queryparams.paramprompt', 'DataPromptMessages'); //$NON-NLS-1$ //$NON-NLS-2$

		dojo.mixin(this.messages, dlgMessages);
		dojo.mixin(this.messages, paramMessages);
		dojo.mixin(this.messages, fillMessages);
		dojo.mixin(this.messages, dataMessages);
	},
	
	postCreate: function(){
		this.inherited(arguments);

		// add "name" field
		this.paramNameField = dojo.create(this.nameOptions && this.allowNameEdit ? "select" : "input", { //$NON-NLS-1$ //$NON-NLS-2$
			id: this._nameFieldId,
			readonly: !this.allowNameEdit,
			style: {
				width: "100%" //$NON-NLS-1$
			},
			'aria-required':'true' //$NON-NLS-1$ //$NON-NLS-2$
		}, this.nameSection);
		if(this.nameOptions && this.allowNameEdit){
			this.paramNameField.options = null;
			var i=0;
			dojo.forEach(this.nameOptions, function(option){
				this.paramNameField.options[i++] = new Option(option.name, option.value);
			}, this);
		}

		// remove required mark (*) and disable name field as it is not editable
		if (!this.allowNameEdit) {
			this.nameRequired.style.display = "none"; //$NON-NLS-1$
		}

		// fill "value"'s combo
		var typeItems = {items: this.valueOptions || []};
		var valueStore = new dojo.data.ItemFileReadStore({data: typeItems});
		this.paramValueField.set("store",  valueStore); //$NON-NLS-1$
		this.paramValueField.set("style", {width: "100%"}); //$NON-NLS-1$ //$NON-NLS-2$
		this.editButton.set("style", {display: "none"});; //$NON-NLS-1$ //$NON-NLS-2$
		if(!this.fExtended) {
			// hide type and mandatory fields
			this.typeRow.style.display = "none"; //$NON-NLS-1$
			this.mandatoryRow.style.display = "none"; //$NON-NLS-1$
		}
		
		enumUtils.loadEnum(dojo.hitch(this, function(enums) {
			this.enums = enums;

			// fill param values if exists
			var param = this.param;
			if(param){
				this.paramNameField.value = this.paramName || "";//$NON-NLS-1$
				this.paramTypeField.value = param.type || "";//$NON-NLS-1$
				this.paramDescriptionField.value = param.description || "";//$NON-NLS-1$
				this.paramMandatoryField.checked = param.mandatory;
				if (enumUtils.isEnumType(param.type, this.enums)) {
					this.paramTypeField.value = "enum";//$NON-NLS-1$
					var typeValue = enumUtils.getEnumCategory(param.type, this.enums);
					this._resetEnumFields(true, typeValue, param.value);
				} else {
					this._resetEnumFields(false);
					this.paramValueField.set("value", param.value || "");//$NON-NLS-1$//$NON-NLS-2$
				}
				this._paramTypeChanged();
			} else {
				this._hideSecondarySelect();
			}
		}));
	},
	
	_resetEnumFields: function(enable, initType, initValue) {
		if (enable) { //$NON-NLS-1$
			this._showSecondarySelect();
			this._intEnumTypes(initType, initValue);
		} else {
			this._hideSecondarySelect();
			this._clearEnumParamValues();
		}
	},
	
	_paramTypeChanged: function() {
		
		var type = this.paramTypeField.value;
		
		var isEnum = this._isEnumType();
		var isEnumVisible = this._isEnumVisible();
		if ((isEnum && !isEnumVisible) || (!isEnum && isEnumVisible)) {
			this._resetEnumFields(isEnum, this._getEnumTypeValue(), this.paramSelectField.value);
		}

		if(type == "product" || type == "artifact" || type == "color" || type == "fill" || type == "stroke") { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
			this.paramValueField.set("style", {width: "92%"}); //$NON-NLS-1$ //$NON-NLS-2$
			this.editButton.set("style", {display: ""}); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			this.paramValueField.set("style", {width: "100%"}); //$NON-NLS-1$ //$NON-NLS-2$
			this.editButton.set("style", {display: "none"}); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(type.substr(0, 7) == "system:") { //$NON-NLS-1$
			this.paramValueField.set("value", ""); //$NON-NLS-1$ //$NON-NLS-2$
			this.paramValueField.set("readOnly", true); //$NON-NLS-1$
			
		} else {
			this.paramValueField.set("readOnly", false); //$NON-NLS-1$
		}
	},

	_getEnumTypeValue: function() {
		return this.enumTypeField.value;
	},

	_isEnumType: function() {
		return this.paramTypeField.value == "enum"; //$NON-NLS-1$
	},

	_isEnumVisible: function() {
		return this.enumTypeRow.style.display != "none"; //$NON-NLS-1$
	},
	
	_showSecondarySelect: function() {
		this.paramValueField.set("readOnly", false); //$NON-NLS-1$
		this.paramSelectField.set("style", {display: ""}); //$NON-NLS-1$ //$NON-NLS-2$
		this.paramValueField.set("style", {display: "none"}); //$NON-NLS-1$ //$NON-NLS-2$
		this.enumTypeRow.style.display = "table-row"; //$NON-NLS-1$
	},
	
	_hideSecondarySelect: function() {
		this.paramValueField.set("style", {display: ""}); //$NON-NLS-1$ //$NON-NLS-2$
		this.paramSelectField.set("style", {display: "none"}); //$NON-NLS-1$ //$NON-NLS-2$
		this.enumTypeRow.style.display = "none"; //$NON-NLS-1$
	},
	
	_intEnumTypes: function(initType, initValue) {
		this._setEnumsToSecondarySelect(this.enums, initType, initValue);
	},

	_setEnumsToSecondarySelect: function(enums, initType, initValue) {
		var options = this.enumTypeField.options;
		var length = options.length;
		for (var i = 0; i < length - 1; i++) {
			options.remove(0);
		}

		var values = enumUtils.getEnumTypes(enums);
		var selectedIndex = 0;
		dojo.forEach(values, function(v, idx) {
			dojo.create("option", {value: v.value, innerHTML: escapeHtml(v.label)}, this.enumMoreOption, "before"); //$NON-NLS-1$ //$NON-NLS-2$
			if (v.value === initType) {
				selectedIndex = idx;
			}
		}, this);
		
		this._resetEnumsToSecondarySelect(selectedIndex, initValue);
	},

	_resetEnumsToSecondarySelect: function(initSelection, initValue) {
		this.enumTypeField.selectedIndex = initSelection != undefined ? initSelection : 0;
		this._setEnumParamValues(initValue);
	},
	
	_enumTypeChanged: function() {
		if (this.enumTypeField.selectedIndex < 0) {
			return;
		}
		var option = this.enumTypeField.options[this.enumTypeField.selectedIndex];
		
		if (option.value == "more") { //$NON-NLS-1$
			enumUtils.openEnumerationManagerDialog(dojo.hitch(this, this._onEnumManagerSave), 
					dojo.hitch(this, this._onEnumManagerCancel));
		} else {
			this._setEnumParamValues();
		}
	},
	
	_setEnumParamValues: function(initValue) {
		this._clearEnumParamValues();
		
		var initItem = null;
		var e = this.enums[this.enumTypeField.selectedIndex];
		var items = enumUtils.makeEnumValueOptions(enumUtils.getEnumValues(e), this.messages.notSpecified);
		items = dojo.map(items, function(item, idx) {
			item = dojo.clone(item);
			if (item.label) item.label = escapeHtml(item.label);
			if (initValue) {
				if (item.value == initValue) {
					initItem = item;
				}
			} else if (idx == 0) {
				initItem = item;
			}
			return item;
		});
			
		this.paramSelectField.addOption(items);//$NON-NLS-1$
		if (initItem) {
			this.paramSelectField.set("value", initItem.value);//$NON-NLS-1$
		}
	},
	
	_clearEnumParamValues: function() {
		this.paramSelectField.options = [];
	},
	
	_onEnumManagerSave: function(enums) {
		if (enums) {
			this.enums = enums;
			this._setEnumsToSecondarySelect(this.enums);
		} else {
			this._clearEnumParamValues();
		}
		this._resetEnumsToSecondarySelect();
	},
	
	_onEnumManagerCancel: function() {
		this._resetEnumsToSecondarySelect();
	},
	
	/**
	 * Callback indicating that changes in the Enumeration Manager dialog mean that
	 * the table should be updated.
	 * This function is available to be overridden by callers or used with dojo.connect.
	 */
	onEnumRefresh: function(updatedStore) {
		//No-op
	},

	_onInputAssist: function() {
		var type = this.paramTypeField.value;
		if(util.hasPicker(type)){
			var self = this;
			util.openPicker(type, this.paramValueField.value, this.messages.paramSelectParameterValue, function(value){
				self.paramValueField.set("value", value.uri || value); //$NON-NLS-1$
				this.close();			
			});
		}
	},
	
	getValues: function(){
		var value;
		var type;
		if (this._isEnumType()) {
			value = this.paramSelectField.value;
			type = this.enumTypeField.value;
		} else {
			value = this.paramValueField.value;
			type = this.paramTypeField.value;
		}	

		return {
			name: this.paramNameField.value,
			value: value,
			description: this.paramDescriptionField.value,
			type: type,
			mandatory: this.paramMandatoryField.checked
		};
	}


});

com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent.openDialog = function(params, callback2, contentParams, editParamName, isNameReadonly){
	// isArray is true  :  params = [{name, value, description}]
	// isArray is false :  params = { name: {value, description} }
	var isArray = dojo.isArray(params);
	if(isArray){
		// if array, ignore editParamName
		editParamName = null;
	}
	// create edit icon dialog content
	var cp = dojo.mixin({}, contentParams);
	if(editParamName){
		cp.paramName = editParamName;
		cp.param = params[editParamName];
		cp.allowNameEdit = !isNameReadonly;
	}
	var contentWidget = new com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent(cp);
	
	var dialogTitle = editParamName ? contentWidget.messages.configEditorEditParams : contentWidget.messages.AddParameter;
	
	// show dialog
	new com.ibm.relm.explorer.web.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: dialogTitle,
		onOpen: function(){
			dijit.focus(contentWidget.paramNameField);
		},
		validate: function(values, callback){
			var paramName = values["name"]; //$NON-NLS-1$
			if(paramName && paramName.length>0){
				if(params[paramName] && !editParamName) {
					alert(contentWidget.messages.errParamNameExists);
				} else {
					callback();
				}
			}else{
				alert(contentWidget.messages.errParamNameEmpty);
			}
		},
		onOK: function(values){
			var name = values["name"]; //$NON-NLS-1$
			var value = values["value"]; //$NON-NLS-1$
			var description = values["description"]; //$NON-NLS-1$

			if(editParamName && editParamName != name){
				// delete existing one
				delete params[editParamName];
			}
			
			if(contentParams && contentParams.fExtended) {
				var type = values["type"]; //$NON-NLS-1$
				var mandatory = values["mandatory"]; //$NON-NLS-1$
				if(isArray){
					params.push({name: name, value: value, description: description, type: type, mandatory: mandatory});
				}else{
					params[name] = {value: value, description: description, type: type, mandatory: mandatory};				
				}
			} else {
				if(isArray){
					params.push({name: name, value: value, description: description});
				}else{
					params[name] = {value: value, description: description};				
				}
			}
			callback2();
		}
	});
};

})();