/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$

(function(){

dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.ConfigUtils"); //$NON-NLS-1$
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$
dojo.require("jazz.core.loader");//$NON-NLS-1$


// check analysis menu is ready or not
var analysisConfigClient = null;
var filterCacheManager = null;
var toolURI = null;

var linkTypeValues = [
     // OSLC CM domain (see http://open-services.net/bin/view/Main/CmVocabulary# )
     "oslc_cm:affectedByDefect", //$NON-NLS-1$
     "oslc_cm:affectsPlanItem", //$NON-NLS-1$
     "oslc_cm:affectsRequirement", //$NON-NLS-1$
     "oslc_cm:affectsTestResult", //$NON-NLS-1$
     "oslc_cm:blocksTestExecutionRecord", //$NON-NLS-1$
     "oslc_cm:implementsRequirement", //$NON-NLS-1$
     "oslc_cm:relatedChangeRequest", //$NON-NLS-1$
     "oslc_cm:relatedTestCase", //$NON-NLS-1$
     "oslc_cm:relatedTestExecutionRecord", //$NON-NLS-1$
     "oslc_cm:relatedTestPlan", //$NON-NLS-1$
     "oslc_cm:relatedTestScript", //$NON-NLS-1$
     "oslc_cm:requirementsChangeRequest", //$NON-NLS-1$
     "oslc_cm:testedByTestCase", //$NON-NLS-1$
     "oslc_cm:tracksChangeSet", //$NON-NLS-1$
     "oslc_cm:tracksRequirement", //$NON-NLS-1$
     		             
     // OSLC QM domain (see http://open-services.net/bin/view/Main/QmVocabulary# )
     "oslc_qm:affectedByChangeRequest", //$NON-NLS-1$
     "oslc_qm:blockedByChangeRequest", //$NON-NLS-1$
     "oslc_qm:executesTestScript", //$NON-NLS-1$
     "oslc_qm:executionInstructions", //$NON-NLS-1$
     "oslc_qm:producedByTestExecutionRecord", //$NON-NLS-1$
     "oslc_qm:relatedChangeRequest", //$NON-NLS-1$
     "oslc_qm:reportsOnTestCase", //$NON-NLS-1$
     "oslc_qm:reportsOnTestPlan", //$NON-NLS-1$
     "oslc_qm:runsOnTestEnvironment", //$NON-NLS-1$
     "oslc_qm:runsTestCase", //$NON-NLS-1$
     "oslc_qm:testsChangeRequest", //$NON-NLS-1$
     "oslc_qm:usesTestCase", //$NON-NLS-1$
     "oslc_qm:usesTestScript", //$NON-NLS-1$
     "oslc_qm:validatesRequirement", //$NON-NLS-1$
     "oslc_qm:validatesRequirementCollection", //$NON-NLS-1$
     
     // OSLC RM domain (see http://open-services.net/bin/view/Main/RmVocabulary# )
     "oslc_rm:affectedBy", //$NON-NLS-1$
     "oslc_rm:elaboratedBy", //$NON-NLS-1$
     "oslc_rm:implementedBy", //$NON-NLS-1$
     "oslc_rm:specifiedBy", //$NON-NLS-1$
     "oslc_rm:trackedBy", //$NON-NLS-1$
     "oslc_rm:uses", //$NON-NLS-1$
     "oslc_rm:validatedBy", //$NON-NLS-1$
     
     // RTC defined
     "rtc_cm:com.ibm.team.workitem.linktype.blocksworkitem.blockedby", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.blocksworkitem.blocks", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.copiedworkitem.copiedFrom", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.copiedworkitem.copies", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.duplicateworkitem.duplicateOf", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.duplicateworkitem.duplicates", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.parentworkitem.children", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.parentworkitem.parent", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.relatedworkitem.related", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.relatedartifact.relatedArtifact", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.workitem.linktype.textualReference.textuallyReferenced", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.build.linktype.reportedWorkItems.com.ibm.team.build.common.link.reportedAgainstBuilds", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.build.linktype.includedWorkItems.com.ibm.team.build.common.link.includedInBuilds", //$NON-NLS-1$
     "rtc_cm:com.ibm.team.filesystem.workitems.change_set.com.ibm.team.scm.ChangeSet", //$NON-NLS-1$
     "rtc_cm:modifiedBy", //$NON-NLS-1$
     "rtc_cm:resolvedBy", //$NON-NLS-1$
     "rtc_cm:subscribers", //$NON-NLS-1$
     
     // DM Link Types (see https://jazz.net/wiki/bin/view/Main/DmLinkTypesVocabulary# )
     "dm_linktypes:derives", //$NON-NLS-1$
     "dm_linktypes:elaborates", //$NON-NLS-1$
     "dm_linktypes:external", //$NON-NLS-1$
     "dm_linktypes:refine", //$NON-NLS-1$
     "dm_linktypes:satisfy", //$NON-NLS-1$
     "dm_linktypes:trace", //$NON-NLS-1$
     
     // Product Definition (see https://jazz.net/wiki/bin/view/Main/ProductAndPartDefinitionVocabulary# )
     "dcterms:references" //$NON-NLS-1$
];

var linkTypes = dojo.map(linkTypeValues, function(l){
	return {name: l, value: l};
});

jazz.core.loader.load_async("dm.analysis.app.ui.internal.impactanalysis.config.FilterCacheManager", function() {  //$NON-NLS-1$
	analysisConfigClient = dojo.getObject("dm.common.app.ui.internal.rhlink.AnalysisConfigClient"); //$NON-NLS-1$
	filterCacheManager = dojo.getObject("dm.analysis.app.ui.internal.impactanalysis.config.FilterCacheManager"); //$NON-NLS-1$
});

var util = com.ibm.relm.explorer.web.ui.internal.util;
var configUtils = com.ibm.relm.explorer.web.ui.internal.ConfigUtils;
var nodeUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;

com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils =
{
	isEmptyObj: function(obj) {
		for(var prop in obj) return false;
		return true;		
	},

	_getQueryVariables: function(config, queryId){
		var variables = [];
		var queries = config.queryTemplates;
		dojo.forEach(queries, function(query) {
			var sparql = query.sparql;
			if (queryId && query.id != queryId) {
				return;
			}
			var res = sparql.match( new RegExp("\\?\\w*", "g") );//$NON-NLS-1$ //$NON-NLS-2$
			dojo.forEach(res, function(r){
				if(dojo.indexOf(variables, r) < 0){ 
					variables.push(r);
				}
			});
		});
		return variables;
		
	},
	
	getQueryVariables: function(config, queryId){
		var variables = this._getQueryVariables(config, queryId);
		if(variables.length == 0) return null;
		return dojo.map(variables, function(v){
			return {name: v, value: configUtils.generateVariableString(v.substring(1))};
		});
	},

	getQueryVariablesForTarget: function(config){
		var variables = this._getQueryVariables(config);
		if(variables.length == 0) return null;
		return dojo.map(variables, function(v){
			return {label: v, name: v.substring(1)};
		});
	},
	
	getResourceInfo: function(widget, config){
		var resInfo = widget.resourceInfo;
		if(resInfo && config){
			// config is not same as resInfo's config, so need reget resInfo with the id
			resInfo = config.getResourceInfoById(resInfo.id);
		}
		if(!resInfo && widget.resource){
			resInfo = (config || widget._viewer.config).getResourceInfo(widget.resource);
		}
		return resInfo;
	},

	getClassTypeItems: function() {
		var items = [
		             {value: "com.ibm.relm.explorer.web.ui.view.nodes.ResourceNode"} //$NON-NLS-1$
		             ];
		return items;
	}, 
	
	getConnectionTypeItems: function(callback, usePrefixedUri) {
		if(!callback){
			callback = function(){}; // dummy
		}
		if(analysisConfigClient && filterCacheManager){
			var projectId = jazz.app.context.get().itemId;
			var deferred = analysisConfigClient.getToolsTypes(projectId);
			deferred.then(function(toolsObj) {
				if(!toolsObj || !toolsObj.tools) return; // something wrong
				dojo.forEach(toolsObj.tools, function(tool){
					if(!toolURI || !tool.ToolPublisher){
						toolURI = tool.ToolURI;
					}
				});
				if(!toolURI) return;// something wrong
				var deferred2 = filterCacheManager.getLinkTypesForTool(projectId, toolURI);
				deferred2.then(function(links) {
					if(!links || !links.types) return;// something wrong
					linkTypes = dojo.map(links.types, function(l){
						var uri = (usePrefixedUri && nodeUtils.makeUriPrefixed(l.key)) || ("<" + l.key + ">"); //$NON-NLS-1$//$NON-NLS-2$
						return {
							value: uri, 
							name: l.value + " (" + uri + ")"};//$NON-NLS-1$//$NON-NLS-2$
					});
					callback(linkTypes);
				}, function(){
					// error
					callback(linkTypes);
				});
			}, function(){
				// error
				callback(linkTypes);
			}); 
		}else{
			callback(linkTypes);
		}
		return linkTypes;
	},
	
	loadComponentTypeToDropdown: function(/* dijit.form.Select */dropdownListWidget, config, editor, defaultValue, suffixForSystemWidget) {
		// clear drop-down list
		var options = dropdownListWidget.options;
		options.length = 0;

		//
		var componentInfoIds = [];
		if(defaultValue){
			componentInfoIds.push(defaultValue.value);
			dropdownListWidget.addOption(defaultValue);
		}
		// componentInfo in this config
		var items = config.componentInfo.items;
		for(var i = 0; i < items.length; i++) {
			var componentInfoId = items[i].id;
			dropdownListWidget.addOption({value: componentInfoId, label: util.escapeHtml(componentInfoId)});
			componentInfoIds.push(componentInfoId);
		}
		// componentInfo in system
		items = editor._componentInfo;
		for(var itemName in items){
			var item = items[itemName];
			var componentInfoId = item.id;
			// must have 'uri' attribute for resource node... need more explicit condition
			if(editor.getAttributeInfo(componentInfoId, "uri")){ //$NON-NLS-1$
				// check the component is system's one or not
				if(dojo.indexOf(componentInfoIds, componentInfoId) < 0){
					var label = componentInfoId;
					if(suffixForSystemWidget) label += " " + suffixForSystemWidget; //$NON-NLS-1$
					dropdownListWidget.addOption({value: componentInfoId, label: util.escapeHtml(label)});
					componentInfoIds.push(componentInfoId);
				}
			}
		}
		return componentInfoIds;
	},
	
	getComponentInfoById: function(id, config, editor) {
		// in this config
		var items = config.componentInfo.items;
		if(items) {
			for(var i = 0; i < items.length; i++) {
				if(items[i].id == id) {
					return {componentInfo: items[i], isSystem: false};
				}
			}
		}

		// in system
		items = editor._componentInfo;
		for(var itemName in items){
			if(items[itemName].id == id){
				return {componentInfo: items[itemName], isSystem: true};
			}
		}

		return {componentInfo: null};
	}
};
})();
