/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.General"); //$NON-NLS-1$

dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.SimpleTextarea"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dijit.form.CheckBox"); //$NON-NLS-1$

dojo.require("jazz.ui.Tree"); //$NON-NLS-1$
dojo.require("jazz.ui.tree.IconItem"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.ViewClient"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var JazzUIWorkbench = PlatformUI.getWorkbench();

var messages = {};

var util = com.ibm.relm.explorer.web.ui.internal.util;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;
var ViewClient = com.ibm.relm.explorer.web.ui.internal.view.ViewClient;
var ServiceResponseHandler = com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler;
var AddParameterContent = com.ibm.relm.explorer.web.ui.internal.editor.AddParameterContent;
var enumUtils = com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils;

var CLASS_ADD_COMMAND = "add-command"; //$NON-NLS-1$
var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$
var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.General", [dijit._Widget, dijit._Templated, com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin], { //$NON-NLS-1$
	
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/sections/templates/ConfigFormGeneral.html"),  //$NON-NLS-1$ //$NON-NLS-2$

	editMode: null,
	
	postMixInProperties: function(){
		this.viewMessages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ViewMessages"); //$NON-NLS-1$  //$NON-NLS-2$
		messages = this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$  //$NON-NLS-2$
	},
	
	postCreate: function() {
		this._responseCount = 0;
		
		var titleNode = dojo.create("span"); //$NON-NLS-1$ 
		titleNode.appendChild(dojo.doc.createTextNode(messages.LabelGeneral));
	
		this.inherited(arguments);

		// create parameters table
		this.paramToolbar.appendChild(this._createAddButton(messages.AddParameter, this._handleAddParameter));
		var disableNameEdit = this.editMode && this.editMode.common.disableNameEdit;
		var columns = this._createParamTableColumnsEx(this.updateParamName, this.updateParamValue, disableNameEdit, this.inSidebar);
		this.paramTable.setupColumns(columns);
		
		// update UI
		this.updateUI();
		
		// event handlers
		this.connect(this.idTextBox, "onChange", function(newVal) { //$NON-NLS-1$
			this._updateProperty("diagramId", newVal, this.idTextBox);//$NON-NLS-1$
		});

		this.connect(this.nameTextBox, "onChange", function(newVal) { //$NON-NLS-1$
			this._updateProperty("diagramName", newVal, this.nameTextBox);//$NON-NLS-1$
			JazzUIWorkbench.setTitle(newVal || "");   //$NON-NLS-1$
		});

		this.connect(this.descriptionTextArea, "onChange", function(newVal) { //$NON-NLS-1$
			this._updateProperty("description", newVal, this.descriptionTextArea);//$NON-NLS-1$
		});
		
	},
	
	updateUI: function(){
		var config = this.getConfig();
		// update general properties
		this.idTextBox.set('value',config.diagramId); //$NON-NLS-1$
		this.nameTextBox.set('value',config.diagramName); //$NON-NLS-1$
		this.descriptionTextArea.set('value',config.description); //$NON-NLS-1$

		// folder tree
		this._updateFolderTree();
		
		enumUtils.loadEnum(dojo.hitch(this, function(enums) {
			this.enums = enums;
			// parameter table
			this.setParamData(config.params);
		}));
	},
	
	_updateFolderTree: function(){
		if(dojo.style(this.folderRow, "display") !== "none"){//$NON-NLS-1$//$NON-NLS-2$
			this._allFolders = [];
			// A map mapping folder nodes to the underlying folder
			this._folderMap = {};
			
			this._createFolderTree();
	
			var responseHandler = new ServiceResponseHandler(
				this,
				"_addFolders", //$NON-NLS-1$
				"_remoteError"); //$NON-NLS-1$
			ViewClient.getFolderDescriptors(responseHandler, {scope: "personal"}); //$NON-NLS-1$
			ViewClient.getFolderDescriptors(responseHandler, {scope: "shared"}); //$NON-NLS-1$
		}
	},
	
	_updateProperty: function(propName, newValue, target){
		var config = this.getConfig();
		if(config[propName] != newValue){
			var self = this;
			var cmd = new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.SimpleUpdateConfigCommand(config, propName, newValue, function(){
				self._updateDirty(config[propName], newValue, propName); 
			} );
			this.getDiagramEditor().executeCommand(cmd);
		}
	},
	
	startup: function() {
		
	},
	
	setParamData: function(params) {
		for(var name in params) {
			var param = params[name];
			if (enumUtils.isEnumType(param.type, this.enums)) {
				this._setParamData(params, this.enums);
				return;
			}
		}
		this._setParamData(params);
	},
	
	_setParamData: function(params, enums) {

		var propTable = this.paramTable;
		propTable.clearItems();

		var rowCallback = function(rowData, propName, newName, dirtyKey) {
			if (propName == "name") { //$NON-NLS-1$
				this.updateParamName(rowData, propName, newName, dirtyKey);
			} else if (propName == "value") { //$NON-NLS-1$
				this.updateParamValue(rowData, propName, newName, dirtyKey);
			}
		};

		var self = this;
		for(var name in params) {
			var param = params[name];
			var value = param["value"]; //$NON-NLS-1$
			var description = param["description"]; //$NON-NLS-1$
			var type = param["type"]; //$NON-NLS-1$
			var mandatory = (param["mandatory"] == "true" || param["mandatory"])? true: false; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			var isSysParam = self.isSystemParameter(type);
			if(isSysParam) { 
				value = ""; //$NON-NLS-1$
			}
		
			// create options for enumeration. this makes inline editor selection(combobox)
			var options = null;
			var callback = null;
			if (enumUtils.isEnumType(type, this.enums)) {
				options = enumUtils.getEnumValueOptions(enums, enumUtils.getEnumCategory(type, this.enums), this.messages.notSpecified); //$NON-NLS-1$
				callback = rowCallback;
			}
			
			propTable.addItem({
				name : name, 
				value: value,
				readonly: isSysParam,
				editButton: self._getEditButton(type, value),
				buttonTarget: "value",//$NON-NLS-1$
				type : type,
				mandatory: mandatory,
				description: description,
				param: param,
				actions: [
					{
						iconClass: CLASS_EDIT_COMMAND,
						title: messages.Edit,
						onClick: function(e){
							var name = this.data.name;
							self._handleAddParameter(name);
						}
					}, 
					{
						iconClass: CLASS_DELETE_COMMAND,
						title: messages.configEditorRemoveParams,
						onClick: function(e){
							var name = this.data.name;
							if(name){
								if(confirm(dojo.string.substitute(messages.confirmRemoveParameter, [ name ]))){
									// remove parameter
									delete params[name];
									self.refreshParamData(self.getConfig().params);
								}
							}
						}
					}
				]
			}, {value: options}, callback, self); 
		}
	},
		
	isSystemParameter: function(type) {
		return (dojo.isString(type) && (type.substr(0, 7) == "system:")); //$NON-NLS-1$
	},
	
	_getEditButton: function(type, value){
		var button = null;
		if(util.hasPicker(type)){
			button = dojo.create("a", { //$NON-NLS-1$
				href: "#", //$NON-NLS-1$
				title: messages.configEditorOpenSelectDialog,
				className: "dijitInline actionIcon" //$NON-NLS-1$
			}); 
			this.connect(button, "onclick", function(ev){//$NON-NLS-1$
				var initialValue = button.targetColumn ? button.targetColumn.get("value") : value;  //$NON-NLS-1$
				util.openPicker(type, initialValue, messages.paramSelectParameterValue, function(value){
					if(value.uri){
						value = value.uri; // artifact picker and product picker
					}
					button.targetColumn.set("value", value); //$NON-NLS-1$
					this.close();
				});
				dojo.stopEvent(ev);
			});
			dojo.create("span", { //$NON-NLS-1$
				className: "dijitInline selectButton" //$NON-NLS-1$
			}, button);
			
			button.editInPlace = true;
		}
		return button;
	},

	updateParamName: function(rowData, propName, newName, dirtyKey) {
		var oldName = rowData.name;	
		if(oldName != newName) {
			var params = this.getConfig().params;
			if(params) {
				params[newName] = params[oldName];
				delete params[oldName];
				this.refreshParamData();//will cause updateDirty
			}
		}
	},
	
//	updateParamValue: function(rowData, propName, newValue, dirtyKey) {
//		var queryId = this.queryDropdown.value;
//		var query = this.getQueryById(queryId);
//		if(query) {
//			var param = query.params[rowData.name];
//			if(param) {
//				var oldValue = param[propName];
//				if(oldValue != newValue){
//					param[propName] = newValue;
//					this.setParamData(query.params);
//					this._updateDirty(oldValue, newValue, dirtyKey);
//				}
//			}
//		}
//	},
//
	updateParamValue: function(rowData, propName, newValue, dirtyKey) {
		var params = this.getConfig().params;
		var paramName = rowData.name;
		if(params && params[paramName]) {
			var param = params[paramName];
			if(param) {
				var oldValue = param[propName];
				if(propName=="mandatory") newValue = (newValue=="true") ? true: false; //$NON-NLS-1$ //$NON-NLS-2$
				if(oldValue != newValue){
					param[propName] = newValue; 
					this.refreshParamData();//will cause updateDirty
				}
			}
		}
	},
	
	refreshParamData: function() {
		this.setParamData(this.getConfig().params);
		this._updateDirty(/* this.orgConfig.params, this.uiConfig.params, this.paramTable */);
		this.setEditMode(this.editMode);

	},

	_handleAddParameter: function(paramName) {
		AddParameterContent.openDialog(this.getConfig().params, dojo.hitch(this, this.refreshParamData), {fExtended: true}, paramName);
	},

	showSection: function(bShow) {
		
	},
	
	_createFolderTree: function() {
		if(!this._folderTreeWidget){
			this._folderTreeWidget = new jazz.ui.Tree();
			this._folderTree.appendChild(this._folderTreeWidget.domNode);
		}
	},

	_getFoldersFromResult: function(response){

		return dojo.map(response.results.bindings, function(r){
			return { 
				"name": {  //$NON-NLS-1$
					label: r.name.value, 
					type: "uri",  //$NON-NLS-1$
					value: r.uri.value
				}, 
				"description": r.description.value, //$NON-NLS-1$
				"creator": r.creator.value, //$NON-NLS-1$
				"modified": r.modified.value,            			 //$NON-NLS-1$
				"ownedBy": r.ownedBy.value, //$NON-NLS-1$
				"shared" : (r.scope.value == "shared") ? true: false, //$NON-NLS-1$ //$NON-NLS-2$
				"parentUUID": r.parentUuid.value, //$NON-NLS-1$
				"folderUUID": r.folderUUID.value //$NON-NLS-1$
			};
		});
	},

	_addFolders: function(response) {
		var folders = this._getFoldersFromResult(response);
		this._responseCount++;
		if (!folders) folders = [];
		for (var i = 0; i < folders.length; i++) {
			this._allFolders.push(folders[i]);
		}
		if (this._responseCount == 1) {
			return;
		}
		this._addFoldersToTree();
		
	},
	
	_addFoldersToTree: function() {
		if(!this.domNode){
			// maybe this widget was already destroyed during loading folder information
			return;
		}
		// First, find the two root folders
		var sharedRoot = null;
		var privateRoot = null;
		for (var i = 0; i < this._allFolders.length; i++) {
			var folder = this._allFolders[i];
			if (folder.parentUUID == "") { //$NON-NLS-1$
				if (folder.shared) {
					sharedRoot = folder;
				} else {
					privateRoot = folder;
				}
			}
		}

		if (privateRoot) {
			var rootItem = new jazz.ui.tree.IconItem({label: this.viewMessages.myQueries, iconClass: "iconFolder"}); //$NON-NLS-1$
			privateRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
			this._folderMap[rootItem] = privateRoot;
			this._folderTreeWidget.addChild(rootItem);
			this._addChildrenForFolder(rootItem, privateRoot, this._allFolders, false, false);
		}
		if (sharedRoot) {
			var rootItem = new jazz.ui.tree.IconItem({label: this.viewMessages.sharedQueries, iconClass: "iconFolder"}); //$NON-NLS-1$
			sharedRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
			this._folderMap[rootItem] = sharedRoot;
			this._folderTreeWidget.addChild(rootItem);
			this._addChildrenForFolder(rootItem, sharedRoot, this._allFolders, false, true);
			this._sharedRootItem = rootItem;
		}
		this._selectFolder(this.getConfig().parentUUID) ;
		this._folderTreeWidget.onSelectionChanged = dojo.hitch(this,this._handleFolderSelectionChange);

	},

	/**
	 * From the list of folders, find any that are children of parentFolder, and
	 * add them to the tree. Then recurse for each of the children.
	 */
	_addChildrenForFolder: function(parentFolderNode, parentFolder, folders, parentIsRoot, parentIsShared) {
		var foldersToAdd = [];
		for (var i = 0; i < folders.length; i++) {
			// Special case where the parent is a root system folder
			if (parentIsRoot) {
				if (!folders[i].parentUUID && (folders[i].shared == parentIsShared) && (folders[i].name.label != parentFolder.name.label)) {
					foldersToAdd.push(folders[i]);
				}
			} else {
				if (folders[i].parentUUID == parentFolder.folderUUID) {
					foldersToAdd.push(folders[i]);
				}
			}
		}
		foldersToAdd = foldersToAdd.sort(function(a, b) {return a.name.label > b.name.label;});
		for (var i = 0; i < foldersToAdd.length; i++) {
			var childNode = new jazz.ui.tree.IconItem({label: com.ibm.relm.explorer.web.ui.internal.util.escapeHtml(foldersToAdd[i].name.label), iconClass: "iconFolder"}); //$NON-NLS-1$
			foldersToAdd[i].treeItemID = childNode.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
			this._folderMap[childNode] = foldersToAdd[i];
			parentFolderNode.addChild(childNode);
			this._addChildrenForFolder(childNode, foldersToAdd[i], folders, false, parentIsShared);
		}
	},

	_selectFolder: function(folderUUID)
	{
		if(!folderUUID){
			// fallback
			this._folderTreeWidget.setSelection(this._sharedRootItem);
			return;
		}
		//Get the itemID from the folder map based on the folder UUID
		//the select that item in the tree
		for (var key in this._folderMap)
		{
			//If the folder in the map is the same as the supplied folder UUID then get the item
			if (this._folderMap[key].folderUUID == folderUUID)
			{
				// for to be returned view lsit (shared/private)
				this.getConfig().shared = this._folderMap[key].shared;
				//Get the item from the tree
				var folderToSelect = this._folderTreeWidget.getItem(this._folderMap[key].treeItemID);
				//Expand the tree down to the folder to select
				var parentItem = folderToSelect.getParent();
				while (parentItem != null) 
				{
					parentItem.setExpanded(true);
					parentItem = parentItem.getParent();
				}
				//select it
				this._folderTreeWidget.setSelection(folderToSelect);
				break;
			}
		}
		
//		this._folderTreeWidget.onSelectionChanged = dojo.hitch(this,this._handleFolderSelectionChange);
	},
	
	_handleFolderSelectionChange: function(selection)
	{
		var selectedFolder = this._folderMap[selection];
		
		this._updateDirty(/* this.orgConfig.parentUUID, selectedFolder.folderUUID, this._folderTreeWidget */);
		var config = this.getConfig(); 
		if(config.parentUUID != selectedFolder.folderUUID) {  
			config.parentUUID =  selectedFolder.folderUUID;   
			config.shared = selectedFolder.shared;
		}

	},

	getSelectedFolder: function() {
		var currentFolderNode = this._folderTreeWidget.getSelection();
		if(!currentFolderNode || currentFolderNode == null){
			return null;
		}
		var currentFolder = this._folderMap[currentFolderNode];
		if (!currentFolder || currentFolder == null) {
			return null;
		}

		return currentFolder;
	},

	_remoteError: function(error) {
		PlatformUI.getWorkbench().showMessage(error.message);
	},

	setEditMode: function(editMode) {
		if(!editMode) return;
		
		this.editMode = editMode;

		var mode = editMode.general;
		
		this.generalSection.style.display = mode.generalSection ? "" : "none";   //$NON-NLS-1$ //$NON-NLS-2$
		this.idTextBox.set('disabled', !mode.idRow); //$NON-NLS-1$ 
		this.nameTextBox.set('disabled', !mode.nameRow); //$NON-NLS-1$ 
		this.descriptionRow.style.display = mode.descriptionRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		var folderDisplay = mode.folderRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		if(this.folderRow.style.display != folderDisplay){
			this.folderRow.style.display = folderDisplay;
			this._updateFolderTree();
		}
		
		var editModeCommon = editMode.common;
		this.paramRow.style.display = !mode.paramRow || (editModeCommon && editModeCommon.hideEmptyTable && editorUtils.isEmptyObj(this.getConfig().params)) ? "none" : "";  //$NON-NLS-1$ //$NON-NLS-2$
		
		var displayType = editModeCommon && editModeCommon.disableActionButton ? "none" : ""; //$NON-NLS-1$ //$NON-NLS-2$ 
		dojo.query("."+CLASS_ADD_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_EDIT_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_DELETE_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	destroy: function() {
		this.paramTable.destroyRecursive();
		this.inherited(arguments);
		if(this._folderTreeWidget){
			this._folderTreeWidget.destroyRecursive();
		}
	}
	
});

})();

