/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.editor.sections.Resource"); //$NON-NLS-1$

dojo.require("jazz.core.loader"); //$NON-NLS-1$
dojo.require("dijit._Templated");//$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dijit.form.Select"); //$NON-NLS-1$

dojo.require("dojo.data.ItemFileReadStore");//$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Select"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.ActionEditTable"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.sections.ComponentParamsEditTable"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.DiagramEditor"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var messages = {};

var DEFAULT_COMPONENT_TYPE = "baseResourceNode"; //$NON-NLS-1$ 

var load_async = jazz.core.loader.load_async;
var util = com.ibm.relm.explorer.web.ui.internal.util;
var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;
var SectionDialog = com.ibm.relm.explorer.web.ui.internal.editor.SectionDialog;

// must be sync with AbstractLayout
var CLASS_ADD_COMMAND = "add-command"; //$NON-NLS-1$
var CLASS_EDIT_COMMAND = "edit-command"; //$NON-NLS-1$
var CLASS_DELETE_COMMAND = "delete-command"; //$NON-NLS-1$

var prepareQueryForDropdownMenu = function (queries, defaultId){
	var defaultQueryFound = false;
	for(var i = 0; i < queries.length ; i++) {
		if(queries[i].id == "") { //$NON-NLS-1$
			queries[i].id = defaultId;
			defaultQueryFound = true;
		}
	}
	if(!defaultQueryFound) {
		// add default query on the list
		queries.unshift({id: defaultId});
	}
};

dojo.declare("com.ibm.relm.explorer.web.ui.internal.editor.sections.Resource", [dijit._Widget, dijit._Templated, com.ibm.relm.explorer.web.ui.internal.editor.sections._SectionMixin], { //$NON-NLS-1$
	
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/editor/sections/templates/ConfigFormResource.html"),  //$NON-NLS-1$  //$NON-NLS-2$

	editMode: null,
	
	_uiPreviewComponent: null,
	
	constructor: function(){
		this._resourceNumId = dijit.getUniqueId('resourceNumId');//$NON-NLS-1$
		this._resourceIdId = dijit.getUniqueId('resourceIdId');//$NON-NLS-1$
		this._resourceNameId = dijit.getUniqueId('resourceNameId');//$NON-NLS-1$
		this._typeDropDownId = dijit.getUniqueId('typeDropDownId');//$NON-NLS-1$
		
	},

	postMixInProperties: function(){
		messages = this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$  //$NON-NLS-2$
	},
	
	postCreate: function() {
	
		var titleNode = dojo.create("span");  //$NON-NLS-1$
		titleNode.appendChild(dojo.doc.createTextNode(messages.LabelResource));
	
		this.inherited(arguments);

		this.resourceInfo = this.getConfig().resourceInfo;

		this.loadComponentTypeToDropdown();

		var self = this;
		// tool bars
		this.targetToolbar.appendChild(this._createAddButton(messages.AddTargetProperty, this._handleAddTargetProperty));
		this.statusLinkToolbar.appendChild(this._createAddButton(messages.AddStatusLinkProperty, this._handleAddStatusLinkProperty));
//		this.componentParamToolbar.appendChild(this._createAddButton(messages.AddComponentParameter, this._handleAddComponentParameter));
		this.hoverToolbar.appendChild(this._createAddButton(messages.AddHoverProperty, this._handleAddHoverProperty));
		this.actionToolbar.appendChild(this._createAddButton(messages.AddAction, this._handleAddAction));
		
		this.targetTable.setupColumns([
		    { id: "value", name: messages.configEditorResourceTargetConditions}, //$NON-NLS-1$
		    { id: "actions", name: messages.configEditorAction, isAction: true} //$NON-NLS-1$
		]);

		this.statusLinkTable.setupColumns([
		    { id: "type", name: messages.configEditorResourceStatusLinkType}, //$NON-NLS-1$
		    { id: "actions", name: messages.configEditorAction,	isAction: true} //$NON-NLS-1$
		]);

		this.hoverTable.setupColumns(this._createTableColumns(
				[
				 {
       		    	id: "label",   //$NON-NLS-1$
       		    	name: messages.configEditorResourceHoverLabel,
       		    	editable: true, 
       		    	callback: this.updateHoverData,
       		    	preventEmpty: true,
       		    	width: "120px"  //$NON-NLS-1$
				 },{
    		    	id: "layout",   //$NON-NLS-1$
       		    	name: messages.configEditorResourceHoverLayout,
       		    	editable: true, 
       		    	callback: this.updateHoverData,
       		    	preventEmpty: false,
       		    	options: [
   		               {label: messages.configEditorResourceHoverLayout_Horizontal, value: "horizontal"}, //$NON-NLS-1$ 
		               {label: messages.configEditorResourceHoverLayout_Vertical, value: "vertical"} //$NON-NLS-1$
       		        ],
       		    	width: "100px"//$NON-NLS-1$
				 },{
    		    	id: "baseQueryId",   //$NON-NLS-1$
       		    	name: messages.configEditorResourceHoverQueryId,
       		    	editable: true, 
       		    	callback: this.updateHoverData,
       		    	preventEmpty: false,
       		    	options: function(){
	       		 		// query in hover
	       		 		var orgQueries = self.getConfig().queryTemplates;
	       		 		var queries = dojo.clone(orgQueries);
	       		 		prepareQueryForDropdownMenu(queries, messages.configEditorResourceHoverDefaultQuery);
	       		 		return dojo.map(queries, function(query){
	       		 			var ret = { label: query.id, value: query.id };
	       		 			return ret;
	       		 		});
       		    	},
       		    	width: "100px"//$NON-NLS-1$
				 },{
    		    	id: "name",   //$NON-NLS-1$
       		    	name: messages.configEditorResourceHoverName,
       		    	editable: true, 
       		    	callback: this.updateHoverData,
       		    	preventEmpty: true,
       		    	width: "360px"  //$NON-NLS-1$
				 },{
     		    	id: "actions",  //$NON-NLS-1$
    		    	name: messages.configEditorAction,
    		    	isAction: true					 
				 }
		]));
		
		this.componentParamTable.setConfigEditor(this.getConfigEditor());
		this.componentParamTable.defaultComponentType = DEFAULT_COMPONENT_TYPE;
		this.actionTable.setConfigEditor(this.getConfigEditor());

		this.setResourceInfo(this.getConfig().resourceInfo);
		if(this.getConfig().resourceInfo[0]) {
			this.resourceDropdown.set('value', String(0));  //$NON-NLS-1$
		}
	
		this.connect(this.resourceDropdown,"onChange", function(value) {  //$NON-NLS-1$
			var index = Number(value);
			this.setResourceDataToPage(index);
		});

		this.connect(this.resourceDropdown,"onClick", function() {  //$NON-NLS-1$
			// avoid to change selected Node if specified node id is duplicated or empty.
			if(!this._checkNodeId()) {
				dojo.stopEvent();
			}
		});

		this.connect(this.componentTypeDropdown, "onChange", function() {  //$NON-NLS-1$
			var resource = this.getCurrentResource();
			if(resource ) {
				if(!resource.component) resource.component = {}; 
				var newValue = this.componentTypeDropdown.get("value");  //$NON-NLS-1$
				if(newValue == messages.configEditorResourceDefaultComponent) {
					if(resource.component.type == null ||resource.component.type == "") { //$NON-NLS-1$
						return;
					}
					resource.component.type = ""; //$NON-NLS-1$
					this._updateDirty();
					
				} else {
					if(resource.component.type != newValue) { 
						resource.component.type = newValue;  
						this._updateDirty();
					}
				}
				this.componentParamTable.setResourceInfo(resource);
			}
		});
		
		this.connect(this.idTextBox,"onChange", function(newVal) { //$NON-NLS-1$
			// reflect the name change to resource dropdown list entry
			var resource = this.getCurrentResource();

			if(resource.id != newVal) { 
				if(resource) {
					resource.id = newVal;   
					this._updateDirty();
				}
			}
		});
		
		this.connect(this.nameTextBox,"onChange", function(newVal) { //$NON-NLS-1$
			// reflect the name change to resource dropdown list entry
			var resource = this.getCurrentResource();

			if(resource.name != newVal) { 
				if(resource) {
					resource.name = newVal;   
					this._updateDirty();
				}
				this.setResourceInfo(this.getConfig().resourceInfo);
				var index = dojo.indexOf(this.getConfig().resourceInfo, resource);
				if(this.getConfig().resourceInfo[index]) {
					this.resourceDropdown.set('value', String(index));  //$NON-NLS-1$
				}
			}
		});
		
		this.connect(this.componentParamTable, "setResourceInfo", function(){//$NON-NLS-1$
			this._updatePreview();
		});


		this.subscribe("/queryListChanged", "setupHoverData");  //$NON-NLS-1$  //$NON-NLS-2$
	}, // postCreate

	_updatePreview: function(){
		if(this._uiPreviewComponent){
			this._uiPreviewComponent.destroy();
			this._uiPreviewComponent = null;
		}
		var uiPreview = this.uiPreview;
		var resource = this.getCurrentResource();
		if(!resource) return;
		var params = {};
		var componentInfo = resource.component || {};
		uiPreview._componentInfo = this.getDiagramEditor()._componentInfo;
		// find target componentInfo
		if(componentInfo.type){
			var compInfo = editorUtils.getComponentInfoById(componentInfo.type, this.getConfig(), this.getDiagramEditor());
			uiPreview._componentInfo = {};
			uiPreview._componentInfo[componentInfo.type] = compInfo.componentInfo; 
		}
		for(var p in componentInfo.params || []){
			params[p] = componentInfo.params[p].value;
		};
		var component = this._uiPreviewComponent = uiPreview.createComponent(componentInfo.type || DEFAULT_COMPONENT_TYPE, params);
		var h = component.getHeight();
		uiPreview.domNode.style.height = h + "px"; //$NON-NLS-1$
		var w = component.getWidth();
		uiPreview.domNode.style.width = w + "px"; //$NON-NLS-1$
		uiPreview.addChild(this._uiPreviewComponent);
		uiPreview.adjustCanvasSize();
	},

	loadComponentTypeToDropdown: function() {
		editorUtils.loadComponentTypeToDropdown(this.componentTypeDropdown, this.getConfig(), this.getDiagramEditor(), 
				{value: messages.configEditorResourceDefaultComponent, label: messages.configEditorResourceDefaultComponent});
	},

	setResourceDataToPage: function(value) {
		var index = value;
		var resource = null;

		if(index >= 0) {
			// clear
			resource = this.getConfig().resourceInfo[index];
		} 
		this.setResourceInfoDetail(resource);
	},
	
	setSelected: function(params){
		if(!params.resourceInfo)
			return;
		var resInfos = this.getConfig().resourceInfo;
		for(var i=0; i<resInfos.length; i++){
			if(params.resourceInfo == resInfos[i]){
				this.resourceDropdown.set('value', String(i));  //$NON-NLS-1$
				break;
			}
		}
	},

	setResourceInfo: function(resourceInfo) {
		var options = this.resourceDropdown.options;
		options.length = 0;
		
		for(var i = 0; i < resourceInfo.length; i++) {
			var resourceName = resourceInfo[i].name || messages.noName;
			this.resourceDropdown.addOption({value: String(i), label: util.escapeHtml(resourceName)});
		}
	},
	
	setResourceInfoDetail: function(resource) {
		this.idTextBox.set('value', (resource && resource.id) || ""); //$NON-NLS-1$ //$NON-NLS-2$
		this.nameTextBox.set('value', (resource && resource.name) || ""); //$NON-NLS-1$ //$NON-NLS-2$
		this.setTargetProperty(resource);
		this.setStatusData(resource);
		this.setComponentData(resource);
		this.setHoverData(resource);
		this.actionTable.setResourceInfo(resource);
		
		if(this.editMode) {
			this.setEditMode(this.editMode);
		}
	},

	setTargetProperty: function(resource) {
		// 
		var propTable = this.targetTable;
		propTable.clearItems();
		
		if(!resource) return;

		var self = this;
		var targets = resource.targetProps;
		for(var i = 0; i < targets.length; i++) {
			var targetName = dojo.string.substitute(messages.targetConditionLabel, [i+1]);
			var idx = i;
			propTable.addItem({name : targetName, 
								value: self.getPropertyString(targets[idx]),
								index: i,
								actions: [
								          {
									iconClass: CLASS_EDIT_COMMAND,
									title: messages.configEditorEditParams,
									onClick: function(e){
										self._handleEditTargetProperty(this.data.index);										
									}
								}, 
								{
									iconClass: CLASS_DELETE_COMMAND,
									title: messages.configEditorRemoveParams,
									onClick: function(e){
										if(this.data.name){
											if(confirm(dojo.string.substitute(messages.confirmRemoveTargetProperty, [ this.data.name ]))){
												// remove parameter
												targets.splice(this.data.index, 1);
												self.refreshTargetProperty(true);
											}
										}
									}
								}]
								
			});
		}

	},
	
	getPropertyString: function(props) {
		var ret = ""; //$NON-NLS-1$
		
		for(var i = 0; i < props.length; i++) {
			ret += "[ '"+ props[i].name + "' : '" + props[i].value + "'] "; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		
		return ret;
	},
	
	refreshTargetProperty: function(fModified) {
		var resource = this.getCurrentResource();
		if(resource) {
			this.setTargetProperty(resource);
			if(fModified) {
				this._updateDirty();
			}
		}
		this.setEditMode(this.editMode);
	},
	
	getCurrentResource: function() {
		var resource = null;
		var index = Number(this.resourceDropdown.get('value'));  //$NON-NLS-1$
		if(index >= 0 && index < this.getConfig().resourceInfo.length) {
			resource = this.getConfig().resourceInfo[index];
		}
		return resource;
	},

	setStatusData: function(resource) {
		var propTable = this.statusLinkTable;
		propTable.clearItems();
		
		if(!resource) return;

		var self = this;
		var linkTypes = resource.statusLinks;
		for(var i = 0; i < linkTypes.length; i++) {
			var linkType = linkTypes[i];
			propTable.addItem({type : linkType.type, 
								index: i,
								actions: [
								          {
									iconClass: CLASS_EDIT_COMMAND,
									title: messages.configEditorEditLinkType,
									onClick: function(e){
										self._handleEditStatusLinkProperty(this.data.index);										
									}
								}, 
								{
									iconClass: CLASS_DELETE_COMMAND,
									title: messages.configEditorRemoveLinkType,
									onClick: function(e){
										if(this.data.type){
											if(confirm(dojo.string.substitute(messages.confirmRemoveLinkType, [ this.data.type ]))){
												// remove link type
												linkTypes.splice(this.data.index, 1);
												self.refreshStatusLinkProperty(true);
											}
										}
									}
								}]
								
			});
		}


	},
	
	refreshStatusLinkProperty: function(fModified) {
		var resource = this.getCurrentResource();
		if(resource) {
			this.setStatusData(resource);
			if(fModified) {
				this._updateDirty();
			}
		}
		this.setEditMode(this.editMode);
	},
	
	setComponentData: function(resource) {
		if(!resource || !resource.component || resource.component.type == null || resource.component.type == "") { //$NON-NLS-1$
			this.componentTypeDropdown.set("value", messages.configEditorResourceDefaultComponent);  //$NON-NLS-1$
		} else {
			this.componentTypeDropdown.set("value", resource.component.type);  //$NON-NLS-1$
		}
		this.componentParamTable.setResourceInfo(resource);
	},
	
	setHoverData: function(resource) {
		var propTable = this.hoverTable;
		propTable.clearItems();

		if(!resource) return;

		var self = this;
		if(resource.hoverProps) {
			dojo.forEach(resource.hoverProps, function(hover) {
				var queryId = hover.baseQueryId;
				if(queryId == null || queryId == "") { //$NON-NLS-1$
					queryId = messages.configEditorResourceHoverDefaultQuery;
				}
				propTable.addItem({
					label: hover.label,
					layout: hover.layout,
					name: hover.name || "",//$NON-NLS-1$
					baseQueryId: queryId,
					actions: [{
						iconClass: CLASS_DELETE_COMMAND,
						title: messages.Remove,
						onClick: function(e){
							if(hover.label){
								if(confirm(dojo.string.substitute(messages.confirmRemoveHoverProperty, [ hover.label ]))){
									// remove parameter
									self.removeHoverProp(hover);
								}
							}
						}
					}]
				});
//			}
			});
		}
	},
	
	removeHoverProp: function(hover) {
		var resource = this.getCurrentResource();
		if(resource) {
			for(var i = 0; i < resource.hoverProps.length; i++) {
				if(resource.hoverProps[i].label == hover.label) {
					delete resource.hoverProps.splice(i,1);
					this._updateDirty();

					break;
				}
			}
			this.setHoverData(resource);
		}
		
	},
	
	updateHoverData: function(rowData, propName, newValue, dirtyKey) {
		var label = rowData.label;
		if(label) {
			var orgProp = this.getHoverDataByLabel(label);
			var oldValue = orgProp[propName];
			orgProp[propName] = newValue;
			if(propName == "baseQueryId") { //$NON-NLS-1$
				if(newValue == messages.configEditorResourceHoverDefaultQuery) {
					orgProp.baseQueryId = ""; //$NON-NLS-1$
				}else{
					// if custom query is specified, property name is no meaning 
					orgProp.name = "";  //$NON-NLS-1$
				}
			}else if(propName == "name") { //$NON-NLS-1$
				// if property name is specified, query should be default query 
				orgProp.baseQueryId = ""; //$NON-NLS-1$
			}
			this._updateDirty(oldValue, newValue, dirtyKey);
		}
		this.setupHoverData();
	},
	
	getHoverDataByLabel: function(label) {
		var resource = this.getCurrentResource(); 
		if(resource) {
			if(resource.hoverProps && resource.hoverProps.length > 0) {
				for(var i = 0; i < resource.hoverProps.length; i++) {
					if(resource.hoverProps[i].label == label) {
						return resource.hoverProps[i];
					}
				}
			}
		}
		
	},

	
	setupHoverData: function() {
		var resource = this.getCurrentResource();
		if(resource) {
			this.setHoverData(resource);
		}
	},
	
	refreshHoverData: function() {
		this._updateDirty();
		this.setupHoverData();
		this.setEditMode(this.editMode);
	},

	createResourceDropdownItem: function (resource, index) {
		
		var optionChild = dojo.doc.createElement("option");  //$NON-NLS-1$
		optionChild.innerHTML = index;
		optionChild.value=resource;
		optionChild.selected = (index == 0);
		
		return optionChild;
	},
	
	_handleAddTargetProperty: function() {
		var resource = this.getCurrentResource();
		if(resource) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent", dojo.hitch(this, function() { //$NON-NLS-1$
				var queryVariables = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils.getQueryVariablesForTarget(this.getConfig());
				com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent.openDialog(resource, -1, dojo.hitch(this, this.refreshTargetProperty), {nameOptions: queryVariables});
			}));
		}
		
	},
	
	_handleEditTargetProperty: function(index) {
		var resource = this.getCurrentResource();
		if(resource) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent", dojo.hitch(this, function() { //$NON-NLS-1$
				var queryVariables = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils.getQueryVariablesForTarget(this.getConfig());
				com.ibm.relm.explorer.web.ui.internal.editor.AddTargetPropertyContent.openDialog(resource, index, dojo.hitch(this, this.refreshTargetProperty), {nameOptions: queryVariables});
			}));
		}
		
	},

	_handleAddStatusLinkProperty: function() {
		var resource = this.getCurrentResource();
		if(resource) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddStatusLinkContent", dojo.hitch(this, function() { //$NON-NLS-1$
				com.ibm.relm.explorer.web.ui.internal.editor.AddStatusLinkContent.openDialog(resource, -1, dojo.hitch(this, this.refreshStatusLinkProperty));
			}));
		}
		
	},
	
	_handleEditStatusLinkProperty: function(index) {
		var resource = this.getCurrentResource();
		if(resource) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddStatusLinkContent", dojo.hitch(this, function() { //$NON-NLS-1$
				com.ibm.relm.explorer.web.ui.internal.editor.AddStatusLinkContent.openDialog(resource, index, dojo.hitch(this, this.refreshStatusLinkProperty));
			}));
		}
		
	},
	
	_handleAddComponentParameter: function() {
		this.componentParamTable.addParameter();
	},
	
	_handleAddHoverProperty: function() {
		var resource = this.getCurrentResource();
		var queries = dojo.clone(this.getConfig().queryTemplates);
		prepareQueryForDropdownMenu(queries, messages.configEditorResourceHoverDefaultQuery);
		if(resource) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent", dojo.hitch(this, function() { //$NON-NLS-1$
				com.ibm.relm.explorer.web.ui.internal.editor.AddHoverPropertyContent.openDialog(resource, queries, dojo.hitch(this, this.refreshHoverData));
			}));
		}
	},

	_handleAddAction: function() {
		this.actionTable.addAction();
	},
	
	_duplicateResource: function(skipConfirm) {
		if((skipConfirm !== true) && !this._checkNodeId()) return;
		
		if(skipConfirm===true || confirm(messages.confirmAddResource)) {
			var duplicatedResource = dojo.clone(this.getCurrentResource());
			duplicatedResource.name = dojo.string.substitute(messages.CopyOf, [ duplicatedResource.name ]); 
			duplicatedResource.id = this._createNewResourceId();
			this.getConfig().resourceInfo.unshift(duplicatedResource);
			
			this._updateDirty();
			
			this.setResourceInfo(this.getConfig().resourceInfo);
			this.resourceDropdown.set('value', String(0));  //$NON-NLS-1$
			this.setResourceDataToPage(0);
		}
	},
	
	_addResource: function(skipConfirm) {

		if((skipConfirm!==true) && !this._checkNodeId()) return;

		if(skipConfirm===true || confirm(messages.confirmAddResource)) {
			// add the new resource definition at the first of
			this.getConfig().resourceInfo.unshift({
				name: this._createNewResourceName(),
				id: this._createNewResourceId(),
				targetProps: [],
				hoverProps: [],
				actions: [],
				statusLinks: [],
				component : {
					type: "", //$NON-NLS-1$
					params: {}
				}
			});
			
			this._updateDirty();
			
			this.setResourceInfo(this.getConfig().resourceInfo);
			this.resourceDropdown.set('value', String(0));  //$NON-NLS-1$
			this.setResourceDataToPage(0);
		}
	},
	
	_createNewResourceName: function() {
		for(var i = 0; i < this.getConfig().resourceInfo.length + 1; i++) {
			var tempName = dojo.string.substitute( this.messages.DefautResourceName, [i]);
			var isUsed = dojo.some(this.getConfig().resourceInfo, function(resource){
				return resource.name == tempName;
			});
			if(!isUsed) return tempName;
		}
		
	},
	
	_createNewResourceId: function() {
		for(var i = 0; i < this.getConfig().resourceInfo.length + 1; i++) {
			var tempId = dojo.string.substitute( "node${0}", [i]); //$NON-NLS-1$  
			var isUsed = dojo.some(this.getConfig().resourceInfo, function(resource){
				return resource.id == tempId;
			});
			if(!isUsed) return tempId;
		}
	},
	
	_removeResource: function() {
		var index = Number(this.resourceDropdown.get('value'));  //$NON-NLS-1$

		var config = this.getConfig();
		var label = config.resourceInfo[index].name || messages.noName;
		if(confirm(dojo.string.substitute(messages.confirmRemoveResource, [label]))) {
			config.resourceInfo.splice(index, 1);
			this._updateDirty();
			this.setResourceInfo(config.resourceInfo);
			
			if(config.resourceInfo[0]) {
				this.resourceDropdown.set('value', String(0));  //$NON-NLS-1$
				if(index == 0) {
					// onChange not happen
					this.setResourceDataToPage(0);
				}
			} else {
				// clear fields
				this.setResourceDataToPage(-1);
				this.resourceDropdown.reset();
			}

		}
	},
	
	_openReorderResourceDialog: function() {
		// open dialog for reorder of reources
		var resourceList = this.getConfig().resourceInfo;
		var resource = this.getCurrentResource();
		if(resourceList && resourceList.length > 0) {
			load_async("com.ibm.relm.explorer.web.ui.internal.editor.ReorderResourceDialogContent", dojo.hitch(this, function() { //$NON-NLS-1$
				com.ibm.relm.explorer.web.ui.internal.editor.ReorderResourceDialogContent.openDialog(resourceList, resource,  dojo.hitch(this, this.setResourceInfoAfterReorder));
			}));

		} else {
			alert(messages.noResourceDefined);
		}
		
		
		
	},
	
	_checkNodeId: function() {
		var id = this.idTextBox.get("value"); //$NON-NLS-1$
		if(id == "") { //$NON-NLS-1$
			// warn empty id
			alert(messages.nodeIdEmpty);
			dijit.focus(this.idTextBox.domNode);
			return false;
//			dojo.stopEvent(e);
		} else {
			var currentResource = this.getCurrentResource();
			for(var i = 0; i < this.getConfig().resourceInfo.length; i++) {
				var isExist = dojo.some(this.getConfig().resourceInfo, function(resource){
					return ((resource.id == id) && (resource != currentResource));
				});
				if(isExist) {
					// warn duplicated id
					alert(dojo.string.substitute(messages.nodeIdDuplicated, [id]));
					dijit.focus(this.idTextBox.domNode);
					return false;
//					dojo.stopEvent(e);
				}
			}

		}
		
		return true;
	},
	
	setResourceInfoAfterReorder: function(resourceInfo) {
		var selectedResource = this.getCurrentResource();
		var config = this.getConfig();
		config.resourceInfo = resourceInfo;
		this.setResourceInfo(config.resourceInfo);
		this._updateDirty();

		var index = dojo.indexOf(config.resourceInfo, selectedResource);
		if(config.resourceInfo[index]) {
			this.resourceDropdown.set('value', String(index));  //$NON-NLS-1$
			this.setResourceDataToPage(0);
		}
	},

	setEditMode: function(editMode) {
		if(!editMode) return;

		this.editMode = editMode;
		var mode = editMode.resource;

		this.resourceSection.style.display = mode.resourceSection ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.resourceSelectRow.style.display = mode.resourceSelectRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.targetRow.style.display = mode.targetRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.statusLinkRow.style.display = mode.statusLinkRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.componentTypeRow.style.display = mode.componentTypeRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.componentParameterRow.style.display = mode.componentParameterRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.hoverRow.style.display = mode.hoverRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$
		this.actionRow.style.display = mode.actionRow ? "" : "none";  //$NON-NLS-1$ //$NON-NLS-2$

		if(editMode.common.hideEmptyTable) {
			var resource = this.getCurrentResource();
			if(resource) {
				var targets = resource.targetProps;
				if (!targets || (targets.length == 0)) {
					this.targetRow.style.display = "none"; //$NON-NLS-1$
				}
				
				if(!resource.component || editorUtils.isEmptyObj(resource.component.params)) {
					this.componentParameterRow.style.display = "none"; //$NON-NLS-1$
				}

				var hoverProps = resource.hoverProps;
				if (!hoverProps || (hoverProps.length == 0)) {
					this.hoverRow.style.display = "none"; //$NON-NLS-1$
				}

				var actions = resource.actions;
				if (!actions || (actions.length == 0)) {
					this.actionRow.style.display = "none"; //$NON-NLS-1$
				}
			}
		}

		var readonly = editMode.common.disableActionButton ? true : false;
		this.componentTypeDropdown.set("readOnly", readonly); //$NON-NLS-1$

		var displayType = editMode.common.disableActionButton ? "none" : ""; //$NON-NLS-1$ //$NON-NLS-2$ 
		dojo.query("."+CLASS_ADD_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_EDIT_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("."+CLASS_DELETE_COMMAND).style("display", displayType); //$NON-NLS-1$ //$NON-NLS-2$

		if(this.componentTypeRow.style.display === ""){//$NON-NLS-1$ 
			this.uiPreview.adjustCanvasSize();			
		}
	},
	
	_editComponentInfo: function(e) {
		var config = dojo.clone(this.getConfig());
		var uiType = this.componentTypeDropdown.get("value"); //$NON-NLS-1$
		var self = this;
		SectionDialog.openComponentInfoDialog({
			getConfig: function(){ return config; },
			getDiagramEditor: function(){ return self.getDiagramEditor(); },
			getSelectedType: function(){ return uiType; },
			_updateDirty: function() {
				// do nothing during editing. edit result will be applied to original config at onOK callback
			}
		}, 
		null, 
		function(){
			dojo.mixin(self.getConfig(), config);
			self.loadComponentTypeToDropdown();
			// select the UI type selected before opened UI type dialog
			// it is better to show a message dialog if user changed the UI type in the UI type dialog 
			self.setComponentData(self.getCurrentResource());
		});
		dojo.stopEvent(e);
		
	},

	validate: function(values) {
		return this._checkNodeId();
	},
	
	startup: function() {
		
	},
	
	destroy: function() {
		if(this.uiPreview)
			this.uiPreview.destroy();
		this.inherited(arguments);
	},
	
	setSelectedIndex: function(index){
		if(index < 0){
			// create a new link
			this._addResource(true);
		}else{
			this.resourceDropdown.set('value', String(index));  //$NON-NLS-1$
		}
	}
	
});

})();

