/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.list.ListRow"); //$NON-NLS-1$

dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

(function(){

	dojo.declare(
		"com.ibm.relm.explorer.web.ui.internal.list.ListRow", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{

			//Both of these should be set by the subclass
			templatePath: "", //$NON-NLS-1$
			widgetsInTemplate: false,

			item: null, //can be folder or row
			
			constructor: function(args, node) {
				this.item = args.item;
				this.padding = args.padding;
				this.formattedTime = this._formatDateTime(this.item.modified);
				
				//Even though this isn't used anywhere, it is referenced in the code
				this.item._explorerTemp_ = {};
			},

			postCreate: function() {
				this.inherited(arguments);
				this._formatResource(this.item.creator, this.createdBy, null);
				var domNode = this.domNode;
				domNode._indent = this.padding;
				domNode._rowType = this.rowType;
				domNode._name = this.item.name.label;
				if(this.fShow !== true) {
					domNode.style.display="none"; //$NON-NLS-1$
				}
			},
			
			onFocus: function(){
				this.inherited(arguments);
				dojo.toggleClass(this.domNode, "hoverRow", true); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
			},
			
			onBlur: function(){
				this.inherited(arguments);
				dojo.toggleClass(this.domNode, "hoverRow", false); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
			},
			
			_formatResource: function (value, td, resource) {
				var self = this;
				var displayValue = value.value;
				if(displayValue){
					displayValue = this._getDisplayValue(value);
					var link = dojo.create("a", {href: "#edit", innerHTML: displayValue, title: value.value}, td); //$NON-NLS-1$ //$NON-NLS-2$
					link.resource = resource;
					link._uri_ = value.value;
					link._name_ = displayValue;
					new jazz.ui.ResourceLink( { 
						uri: value.value, 
						lazyFetch: false, 
						retainLinkText: false, 
						useAbbreviation: false }, link);
					if(resource) {
						self.connect(link, "onclick", function(evt){ //$NON-NLS-1$
							self.onResourceSelected(evt.target);
							dojo.stopEvent(evt);
						});
						if(!resource._explorerTemp_._uri_){
							// used in onDelete
							// assumes the first uri is the resource uri
							resource._explorerTemp_._uri_ = value.value;
							resource._explorerTemp_._name_ = displayValue;
						}
					}

					sortValue = displayValue; // sort by displayed string
					displayValue = null;
				}
				if(td && displayValue){
					td.innerHTML = displayValue;
				}

			}, 

			_getDisplayValue: function(valueObject) {
				if (!valueObject.label) {
					var value = valueObject.value;
					var displayValue = decodeURIComponent(value.substring(value.lastIndexOf('/') + 1)); //$NON-NLS-1$
					//Try to cache the label so it doesn't need to computed again when sorting
					valueObject.label = displayValue;
				}
				return com.ibm.relm.explorer.web.ui.internal.util.escapeHtml(valueObject.label);
			},

			_formatDateTime: function (value) {
				sortValue = dojo.date.stamp.fromISOString(value); // convert created to Date type
				if(sortValue){
					displayValue = dojo.date.locale.format(sortValue, {formatLength: "medium"}); //$NON-NLS-1$
				}
				
				return displayValue;
			}
		});
	})();