/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.reports.ReportSelectionStep"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool"); //$NON-NLS-1$
dojo.require("com.ibm.team.reports.web.ui.ReportsConstants"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("jazz.ui.FilterBox"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("jazz.ui.SimpleToolbar"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ReportMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var HTML = jazz.util.html;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;
var ResourceLink = jazz.ui.ResourceLink;

var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ReportMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var BASE_PADDING = 9;
var INDENT = 38;

var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var REPORTIMG = ImagePool.REPORT_OBJ;
var FOLDERIMG = ImagePool.REPORTS_OBJ;

var FOLDERTYPE = "folder"; //$NON-NLS-1$
var QUERYTYPE = "query"; //$NON-NLS-1$

var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;

var AllReportsTab = dojo.declare("com.ibm.relm.explorer.web.ui.internal.reports.ReportSelectionStep", //$NON-NLS-1$
	[WizardStep, dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/reports/templates/ReportSelectionStep.html"), //$NON-NLS-1$ //$NON-NLS-2$
	_childMap: {},
	heading: null,
	shared: true, // default 
	
	constructor: function(args) {
		dojo.mixin(this, args);
		this.messages = messages;
		this.heading = messages.sharedReports;
		if(args.shared == "false") this.heading = messages.myReports; //$NON-NLS-1$
		this.stepTitle = this.messages.selectReportStep;
	},
	
	postCreate: function() {
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter)});
		this._filterArea.appendChild(this._filterBox.domNode);
		this._table = new StyledTable({
			columns: [
			          {label: messages.report, width: "40%", sortable: false},  //$NON-NLS-1$
			          {label: messages.descriptioncolumnheader, width: "60%", sortable: false} //$NON-NLS-1$
			], 
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS, 
			showHover: true
		});
		this._resultsArea.appendChild(this._table.domNode);
		this._runNewQuery();
		this.connect(this._folderFilterSelect, "onchange", this._runNewQueryForView); //$NON-NLS-1$
	},
	
	_runNewQueryForView: function(event){
		var select = this._folderFilterSelect;
		var folderUUID = select.options[select.selectedIndex].value;
		if(folderUUID){
			if(folderUUID == "_allFolders"){ //$NON-NLS-1$
				this._runNewQuery();
			}else{
				this._renderChildFoldersForParent(folderUUID);
			}
		}
	},
	
	_runNewQuery: function() {
//		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		//Signal the loading to clear the folder filter
		//We have to clear it or refreshing will add duplicates to the filter combo box
		this._clearFilter = true;
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var responseHandler = new ServiceResponseHandler(
				this,
				"_renderRootFolders", //$NON-NLS-1$
				"_remoteError"); //$NON-NLS-1$
		ReportsClient.getSystemRootFolderDescriptors(responseHandler);
	},	
	
	_renderRootFolders: function(results) {
		// Don't actually create table nodes for the root "My Reports" or "Shared Reports".
		// Just do the recursive call.
		if (results.length != 2) {
			// Something went wrong
			return;
		}
		var index = (this.shared) ? 1 : 0;
		this._rootFolder = results[index];
		this._renderChildFoldersForParent(this._rootFolder.folderUUID);
	},
	
	_renderChildFoldersForParent: function(folderUUID){
		//Clear the filter box if the clear flag is set
		if (this._clearFilter)
		{
			var len = this._folderFilterSelect.options.length;
			for (var i = len-1; i > 0; i--)
				this._folderFilterSelect.remove(i);
			this._clearFilter = false;
		}
		
//		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		this._table.clearBody();
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var args = {
			// Project Area will be added by ReportsClient
			// Owner is intentionally empty
    		parentUUID: folderUUID,
    		flags: ReportsClient.ALL,
    		sortByName: true
    	};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_addChildFolders", //$NON-NLS-1$
				"_remoteError"); //$NON-NLS-1$
		ReportsClient.getFolderDescriptors(responseHandler, args);
		var args2 = {
			// Project Area will be added by ReportsClient
			// Owner is intentionally empty
    		folderUUID: folderUUID,
    		flags: ReportsClient.EXCLUDE_MICRO,
    		sortByName: true
    	};
    	if (this._currentFilter) {
    		args2.filter = this._currentFilter;
    	}
		var responseHandler2 = new ServiceResponseHandler(
				this,
				"_addChildQueries", //$NON-NLS-1$
				"_remoteError"); //$NON-NLS-1$
		ReportsClient.getQueryDescriptors(responseHandler2, args2);
	},
	
	_addChildFolders: function(results) {
//		PlatformUI.getWorkbench().hideMessageArea();
		//How many filters are already in the list
		var filterEntryCount = this._folderFilterSelect.options.length;
		if (results != null) {
			for (var i = 0; i < results.length; i++) {
				var folder = results[i];
				this._folderFilterSelect.options[filterEntryCount + i] = new Option(HTML.escape(folder.name), folder.folderUUID);
				this._childMap[folder.folderUUID] = new Array();
				// Locate the parent to insert it under
				if (folder.parentUUID == this._rootFolder.folderUUID || folder.parentUUID == null) {
					this._table.tbody.appendChild(this._createFolderRow(folder, 9));
				} else {
					var parent = document.getElementById(this.widgetId + folder.parentUUID);
					if (parent == null) {
						// Do nothing, shouldn't happen.
					} else {
						this._childMap[folder.parentUUID].push(folder.folderUUID);	//MFV added folders to the child map to handle click on parent folder checkbox
						this._insertAfter(this._createFolderRow(folder, parent._indent + INDENT), parent);
					}
				}
			}
			// Recurse
			for (var i = 0; i < results.length; i++) {
				var folder = results[i];
				var args = {
					// Project Area will be added by ReportsClient
					// Owner is intentionally empty
	        		parentUUID: folder.folderUUID,
	        		flags: ReportsClient.ALL,
	        		sortByName: true
	        	};
				var responseHandler = new ServiceResponseHandler(
						this,
						"_addChildFolders", //$NON-NLS-1$
						"_remoteError"); //$NON-NLS-1$
				ReportsClient.getFolderDescriptors(responseHandler, args);
				var args2 = {
					// Project Area will be added by ReportsClient
					// Owner is intentionally empty
	        		folderUUID: folder.folderUUID,
	        		flags: ReportsClient.EXCLUDE_MICRO,
		    		sortByName: true
	        	};
		    	if (this._currentFilter) {
		    		args2.filter = this._currentFilter;
		    	}
				var responseHandler2 = new ServiceResponseHandler(
						this,
						"_addChildQueries", //$NON-NLS-1$
						"_remoteError"); //$NON-NLS-1$
				ReportsClient.getQueryDescriptors(responseHandler2, args2);
			}
		}
	},
	
	_addChildQueries: function(results) {
//		PlatformUI.getWorkbench().hideMessageArea();
		if (results != null) {
			for (var i = 0; i < results.length; i++) {
				var query = results[i];
				if (this._childMap[query.folderUUID] == null)
					this._childMap[query.folderUUID] = new Array();
				this._childMap[query.folderUUID].push(query.queryUUID);
				// Locate the parent to insert it under
				if (query.folderUUID == this._rootFolder.folderUUID) {
					this._table.tbody.appendChild(this._createQueryRow(query, 9));
				} else {
					var parentFolder = document.getElementById(this.widgetId + query.folderUUID);
					if (parentFolder == null) {
						// Couldn't find the folder, so must be a root folder
						this._table.tbody.appendChild(this._createQueryRow(query, 9));
					} else {
						//this._insertAfter(this._createQueryRow(query, parentFolder._indent + INDENT), parentFolder);
						this._insertAfter(this._createQueryRow(query, parentFolder._indent + INDENT), this._getQueryInsertionPoint(query.folderUUID));
					}
				}
			}
		}
	},
	
	_getQueryInsertionPoint: function(folderUUID) {
		var folderTRNode = document.getElementById(this.widgetId + folderUUID);
		var candidate = folderTRNode;
		var nextCandidate = candidate.nextSibling;
		while (true) {
			if (nextCandidate == null) {
				return candidate;
			}
			if (nextCandidate._indent <= folderTRNode._indent || nextCandidate._rowType == FOLDERTYPE){   
				return candidate;
			}
			candidate = nextCandidate;
			nextCandidate = nextCandidate.nextSibling;
		} 
	},
	
	_renderResults: function(results) {
		this._table.clearBody();
		if (results && results.length > 0) {
			for (var i = 0; i < results.length; i++) {
				this._table.tbody.appendChild(this._createQueryRow(results[i]));
			}
		} else {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
	},
	
	validate: function() {
		var ret = [];
		if(!this.wizard._selectedReport){
			ret.push(this.messages.noReportSelected);
		}
		return ret;
	},
	
	_selectReport: function(report){
		this.wizard._selectedReport = report;
		this.triggerButtonStateUpdate();
	},
	
	_createQueryRow: function(report, padding) {
		var self = this;
		
		var row = document.createElement("tr"); //$NON-NLS-1$
		row._indent = padding;
		row._rowType = QUERYTYPE;
		row._name = report.name;
		row.id = this.widgetId + report.queryUUID;
		row.folderID = report.folderUUID;   //Save this so we can find the folder row easily

		var td = document.createElement("td"); //$NON-NLS-1$

		//Create the checkbox
		var cb = document.createElement("input"); //$NON-NLS-1$
		cb.style.paddingRight = "5px"; //$NON-NLS-1$
		cb.type = "radio"; //$NON-NLS-1$
		cb.name = "selectedReport"; //$NON-NLS-1$
		cb.value = report.queryUUID;
		cb.checked = false;
		this.connect(cb, "onclick", function(){ //$NON-NLS-1$
			this._selectReport(report);
		});

		//The report image
		var img = document.createElement("img"); //$NON-NLS-1$
		img.src = REPORTIMG;
		img.classname = "row-image"; //$NON-NLS-1$
		img.style.paddingLeft = "5px"; //$NON-NLS-1$
		img.style.paddingRight= "5px";		 //$NON-NLS-1$

		//Create the anchor element for the report
		var anchor = document.createElement("span"); //$NON-NLS-1$
		anchor.appendChild(document.createTextNode(" " + HTML.escape(report.name))); //$NON-NLS-1$
		td.style.paddingLeft = padding + "px"; //$NON-NLS-1$
		td.className="reportName"; //$NON-NLS-1$
		
		//append the checkbox and anchor to the table data
		td.appendChild(cb);
		td.appendChild(img);
		td.appendChild(anchor);
		
		//append the td to the row
		row.appendChild(td);
		row.id = this.widgetId + report.queryUUID;
		row.folderID = report.folderUUID;   //Save this so we can find the folder row easily
		
		//Create the report description
		td = document.createElement("td"); //$NON-NLS-1$
		td.classname = "reportName"; //$NON-NLS-1$
		td.innerHTML = HTML.escape(report.description);
		row.appendChild(td);
		
		return row;
	},
	
	_createFolderRow: function(folder, padding) {
		var row = document.createElement("tr"); //$NON-NLS-1$
		row.setAttribute("id", this.widgetId + folder.folderUUID); //$NON-NLS-1$
		row._indent = padding;
		row._rowType = FOLDERTYPE;
		row._name = folder.name;
		row.id = this.widgetId + folder.folderUUID;

		td = document.createElement("td"); //$NON-NLS-1$
		td.style.paddingLeft = padding + "px"; //$NON-NLS-1$
		td.className="folderName"; //$NON-NLS-1$
		
		//Folder image
		var img = document.createElement("img"); //$NON-NLS-1$
		img.src = FOLDERIMG;
		img.classname = "row-image"; //$NON-NLS-1$
		img.style.paddingLeft = "5px"; //$NON-NLS-1$
		img.style.paddingRight= "5px";		 //$NON-NLS-1$

		//Create the text node to hold the folder name
		var textNode = document.createTextNode(" " + HTML.escape(folder.name)); //$NON-NLS-1$
		td.appendChild(img);
		td.appendChild(textNode);
		row.appendChild(td);

		//Create the description
		td = document.createElement("td"); //$NON-NLS-1$
		td.innerHTML = HTML.escape(folder.description);
		td.classname = "folderName"; //$NON-NLS-1$
		row.appendChild(td);
		
		return row;
	},
	
	_okDialogCallback: function(args)
	{
		this._dialog.close();
		this._dialog.destroy();
		this._runNewQuery();
		
	},
	
	_cancelDialogCallback: function(args)
	{
		this._dialog.close();
		this._dialog.destroy();
	},

	
	_createNoneMessage: function() {
		var row = document.createElement("tr"); //$NON-NLS-1$
		var td = document.createElement("td"); //$NON-NLS-1$
		td.innerHTML = HTML.escape(messages.allReportsNone);
		row.appendChild(td);
		return row;
	},
	
	_doFilter: function(filter) {
		this._currentFilter = filter;
		this._isRelatedFilterStale = true;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to _runQuery or setTimeout will pass some other value
		if (!this._runQueryTimeout) {
			this._runQueryTimeout = setTimeout(dojo.hitch(this,this._runNewQueryForView,0,true),500);
		}
	},
	
	refreshCallback: function()
	{
		this._runNewQuery();		
	},
	
	_remoteError: function(error) {
		PlatformUI.getWorkbench().showMessage(error.message);		
	},
	
	_insertAfter: function(newNode, referenceNode) {
	    referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
	},
	
	_trim: function(string) {
		return this._trimLeft(this._trimRight(string));
	},
	_trimLeft: function(string) {
		return string.replace(new RegExp("^[\\s]+", "g"), ""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},
	_trimRight: function(string) {
		return string.replace(new RegExp("[\\s]+$", "g"), ""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

});

})();
