/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.reports.ReportTemplateSelectionStep"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("jazz.app.context"); //$NON-NLS-1$
dojo.require("jazz.ui.FilterBox"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStepHeader"); //$NON-NLS-1$
dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$


dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ReportMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var HTML = jazz.util.html;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;
var ResourceLink = jazz.ui.ResourceLink;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ReportMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;

var AllReportTemplatesTab = dojo.declare("com.ibm.relm.explorer.web.ui.internal.reports.ReportTemplateSelectionStep", //$NON-NLS-1$
	[WizardStep, dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/reports/templates/ReportTemplateSelectionStep.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	constructor: function(args) {
		dojo.mixin(this, args);
		this.messages = messages;
		this.templatesHeading = messages.reportTemplates;
		this.stepTitle = this.messages.selectReportStep;
	},
	
	postCreate: function() {
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter)});
		this._filterArea.appendChild(this._filterBox.domNode);
		this._table = new StyledTable({
			columns: [
			          {label: "", width: "1", sortable: false},  //$NON-NLS-1$ //$NON-NLS-2$
			          {label: messages.reportTemplate, width: "35%", sortable: false},  //$NON-NLS-1$
			          {label: messages.descriptioncolumnheader, width: "60%", sortable: false}],  //$NON-NLS-1$
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS, 
			showHover: true});
		this._resultsArea.appendChild(this._table.domNode);
		
		this._runQuery();
		
		this.messageBox = new jazz.ui.MessageBox();
		this._messageBoxContainer.appendChild(this.messageBox.domNode);
	},
	
	_runQuery: function() {
//		PlatformUI.getWorkbench().showMessage(messages.allReportsSearching);
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		var serviceObject = {
			self: this,
			id: this._nextQueryId,
			success: function(results) {
				if (this.self._lastQueryId && this.self._lastQueryId > this.id)
					return;
				this.self._lastQueryId = this.id;
				this.self._renderResults(results);
				PlatformUI.getWorkbench().hideMessageArea();
			},
			failure: function(error) {
				// TODO
				//dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure"); //$NON-NLS-1$ //$NON-NLS-2$
		var args = {
			sortByName: true
		};
		if (this._currentFilter) {
			args.filter = this._currentFilter;
		}
		ReportsClient.getReportDescriptors(srh, args);
		this._nextQueryId++;
	},
	
	_renderResults: function(results) {
		this._table.clearBody();
		if (results && results.length > 0) {
			for (var i = 0; i < results.length; i++) {
				this._table.tbody.appendChild(this._createTemplateRow(results[i]));
			}
		} else {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
	},
	
	validate: function() {
		var ret = [];
		if(!this.wizard._selectedReport){
			ret.push(this.messages.noReportSelected);
		}
		return ret;
	},
	
	_selectReport: function(report){
		this.wizard._selectedReport = report;
		this.triggerButtonStateUpdate();
	},
	
	_createTemplateRow: function(template) {
		var row = document.createElement("tr"); //$NON-NLS-1$
		
		//Create the checkbox

		var td = document.createElement("td"); //$NON-NLS-1$
		td.className = "checkBox"; //$NON-NLS-1$
		var cb = document.createElement("input"); //$NON-NLS-1$
		cb.type = "radio"; //$NON-NLS-1$
		cb.name = "selectedReport"; //$NON-NLS-1$
		cb.value = template.reportUUID;
		cb.checked = false;
		if(this.wizard._selectedReport){
			if(this.wizard._selectedReport.reportUUID == template.reportUUID){
				cb.checked = true;				
			}
		}
		this.connect(cb, "onclick", function(){ //$NON-NLS-1$
			this._selectReport(template);
		});
		td.appendChild(cb);
		row.appendChild(td);
		
		td = document.createElement("td"); //$NON-NLS-1$
		var innerHTML = "<span>" + HTML.escape(template.name) + "</span>"; //$NON-NLS-1$ //$NON-NLS-2$
		td.innerHTML = innerHTML;
		row.id = template.reportUUID;
		row.appendChild(td);

		td = document.createElement("td"); //$NON-NLS-1$
		td.classname = "reportName"; //$NON-NLS-1$
		td.innerHTML = HTML.escape(template.description);
		row.appendChild(td);
		
		return row;
	},
	
	
	_createNoneMessage: function() {
		var row = document.createElement("tr"); //$NON-NLS-1$
		var td = document.createElement("td"); //$NON-NLS-1$
		td.colSpan = 3;
		td.innerHTML = HTML.escape(messages.allTemplatesNone);
		row.appendChild(td);
		return row;
	},
	
	_doFilter: function(filter) {
		this._currentFilter = filter;
		this._isRelatedFilterStale = true;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to _runQuery or setTimeout will pass some other value
		if (!this._runQueryTimeout) {
			this._runQueryTimeout = setTimeout(dojo.hitch(this,this._runQuery,0,true),500);
		}
	},
	
	_remoteError: function(error) {
		//alert(error.message);
	},
	
	_showStatusMessage: function(message) {
		this._clearStatusMessage();
		var status = {
			type : jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary : message,
			closable : false
		};
		dojo.addClass(this._messageBoxContainer, "messageBoxVisible"); //$NON-NLS-1$
		this.messageBox.showMessage(status);
	},
	
	_clearStatusMessage: function() {
		dojo.removeClass(this._messageBoxContainer, "messageBoxVisible"); //$NON-NLS-1$
		this.messageBox.hideMessage();
	}	
});

})();
