/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2016.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

( function(){

  dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.Layout"); //$NON-NLS-1$

  dojo.require("dijit.Dialog"); //$NON-NLS-1$
  dojo.require("dijit.Menu"); //$NON-NLS-1$
  dojo.require("dijit.form.Button"); //$NON-NLS-1$
  dojo.require("dijit.form.DropDownButton"); //$NON-NLS-1$
  dojo.require("dijit.form.ComboBox"); //$NON-NLS-1$
  dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
  dojo.require("dijit.form.HorizontalSlider"); //$NON-NLS-1$
  dojo.require("dijit._Templated"); //$NON-NLS-1$

  dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
  dojo.require("jazz.ui.toolbar.Separator"); //$NON-NLS-1$
  dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
  dojo.require("jazz.ui.toolbar.MenuButton"); //$NON-NLS-1$
  dojo.require("jazz.ui.SearchBox"); //$NON-NLS-1$
  dojo.require("jazz.util.html"); //$NON-NLS-1$
  dojo.require("rmpw.jazz.ui.Controls"); //$NON-NLS-1$
  dojo.require("com.ibm.relm.explorer.web.ui.internal.view.DiagramWidget"); //$NON-NLS-1$
  dojo.require("com.ibm.relm.explorer.web.ui.internal.view.RelmEditConfiguration"); //$NON-NLS-1$
  dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.Palette"); //$NON-NLS-1$
  dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

  dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "LayoutMessages"); //$NON-NLS-1$ //$NON-NLS-2$

  var controls = rmpw.jazz.ui.Controls.items;

  var HIGHLIGHT = "Highlight Button"; //$NON-NLS-1$
  var FITTOWINDOW = "Fit To Window Button"; //$NON-NLS-1$
  var WHEELSCROLL = "Scroll with Mouse Wheel"; //$NON-NLS-1$
  /*
   * Layout is created in DiagramView
   * 
   * DiagramView 
   *    <-(diagramView)- Layout 
   *                 -(viewer)-> DiagramWidget 
   *                               -(diagramLayer)-> DiagramLayer 
   *                                                   -(viewer)-> DiagramEditRoot 
   *                                                                -(editor)-> DiagramEditor
   */
  dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.Layout", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$

  	 templateString: "<div dojoAttachPoint='containerNode' class='relm-view-layout'><div dojoAttachPoint='toolsNode' style=\"width:100%;\"/></div><div dojoAttachPoint='paletteNode'></div><div dojoAttachPoint='borderNode'></div><div dojoAttachPoint='canvas'/><div dojoAttachPoint='right'/></div>", //$NON-NLS-1$

     initialZoomLevel: 1.0, 
     outlineState: true,
     
     size: null,
     toolbar: null,
     toolbarHeight: 33,

 	 prefix: null,
     diagramView: null, // DiagramView
     viewer: null, // DiagramWidget
     selection: null,
     highlightState: true,
     wheelState: true, // true: wheel scroll, false: wheel zoom

     highlight_button: null,
     wheel_button: null,
     viewUri: null,

 	tools: [{ name: controls.ZOOM, display: true, enabled: true }, 
 	        { name: FITTOWINDOW, display: true, enabled: true },
 	        { name: controls.OUTLINE, display: true, enabled: true },
 	        { name: HIGHLIGHT, display: true, enabled: true },
	        { name: WHEELSCROLL, display: true, enabled: false }],
	        
	prefix: "com.ibm.relm.explorer.web.editor.domain", //$NON-NLS-1$

	postMixInProperties: function() {
	    this.inherited(arguments);
	    this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "LayoutMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.configuration = new com.ibm.relm.explorer.web.ui.internal.view.RelmEditConfiguration();
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
	    this.subscribe('btp:SelectionChanged', this._updateToolbarForSelection); //$NON-NLS-1$
	    this.subscribe('relmZoomChanged', this.setSliderValue); //$NON-NLS-1$
		this.buildToolBar();
		this.initializeDiagram();
	},
	 
	destroy: function(){
		if(this.viewer){
			this.viewer.destroy();
			this.viewer = null;
		}
		if(this._palettePane){
			this._palettePane.destroy();
			this._palettePane = null;
		}
		if(this.toolbar){
			this.toolbar.destroyRecursive();
			this.toolbar = null;
		}
		this.inherited(arguments);
	},
     
	_getDiagramLayer: function(){
		return dojo.getObject("viewer.diagramLayer", false, this); //$NON-NLS-1$
	},
	
   	_getDiagramEditRoot: function(){
		return dojo.getObject("viewer.diagramLayer.viewer", false, this); //$NON-NLS-1$
	},
	
  	_getDiagramEditor: function(){
		return dojo.getObject("viewer.diagramLayer.viewer.editor", false, this); //$NON-NLS-1$
	},
	
	_getSurface: function(){
		return this._getDiagramEditor().surface;
	},
	
    _updateToolbarForSelection: function(message) {
        this.showQueryButton.set('enabled', false); //$NON-NLS-1$
        this.startAnalysisButton.set('enabled', false); //$NON-NLS-1$
        this.customActionsButton.set('enabled', false); //$NON-NLS-1$

        var customActions = [];

        if (message && message.selection) {
            var node = message.selection;
            if(node._viewer && node._viewer === this._getDiagramEditor()){
	            if(dojo.isFunction(node._getAllActions)){
		            var actions = node._getAllActions();
		            
		            for (var i = 0; i < actions.length; i++) {
		                var action = actions[i];
		
		                if (action.actionType === 'showQuery') { //$NON-NLS-1$
		                    this.showQueryButton.set('enabled', true); //$NON-NLS-1$
		                    this.showQueryButton.set('onClick', action.action); //$NON-NLS-1$
		                }
		                else if (action.actionType === 'startAnalysis') { //$NON-NLS-1$
		                    this.startAnalysisButton.set('enabled', true); //$NON-NLS-1$
		                    this.startAnalysisButton.set('onClick', action.action); //$NON-NLS-1$
		                }
		                else if (action.actionType === 'custom') { //$NON-NLS-1$
		                customActions.push(action);
		                }
		            }
	            }
            }
        }

        if (customActions.length) {
            this.customActionsButton.set('enabled', true); //$NON-NLS-1$

            function menuProvider() {
            var actionsMenu = new jazz.ui.Menu();

            dojo.forEach(customActions, function(action){
				var label = action._isEscaped ? action.label : jazz.util.html.escape(action.label); 
				action._isEscaped = true;
                var menuItem = new dijit.MenuItem({
                    label: label, 
                    onClick: function(e){
                        action.action(action.nodeWidget, e);
                    }
                });
                actionsMenu.addChild(menuItem);
            }, this);

            return actionsMenu;
            }

            this.customActionsButton.set('menuProvider', menuProvider); //$NON-NLS-1$
        }
    },

     _startAnalysis: function() {
     	
     },

     _showQuery: function() {
     	
     },

     addButton: function( label, call, icon, show ){
    	 var visibleLabel = show ? label : null;

    	 var button = new jazz.ui.toolbar.Button({
     		title: label,
     		label: visibleLabel,
     		onClick: call,
     		imgClass: icon });

    	 this.addTool( button );

    	 return button;
     },

     buildToolBar: function() {
        this.toolbar = new jazz.ui.Toolbar({}, this.toolsNode);
        if(this.hideToolbar){
        	dojo.style(this.toolbar.domNode, "display", "none");//$NON-NLS-1$ //$NON-NLS-2$
        }
        dojo.addClass(this.toolbar.domNode, "rmpsToolbar"); //$NON-NLS-1$


		this.toolbar.domNode.style.height = "25px"; //$NON-NLS-1$
		this.toolbar.domNode.style.background = "none repeat scroll 0 0 white"; //$NON-NLS-1$
		this.toolbar.domNode.style.marginTop = "2px"; //$NON-NLS-1$
		this.toolbar.domNode.style.borderBottom = "none"; //$NON-NLS-1$
		
		this.borderNode.style.width = "100%"; //$NON-NLS-1$
		this.borderNode.style.borderTop = "1px solid #E8E8E8"; //$NON-NLS-1$
		this.borderNode.style.marginTop = "1px"; //$NON-NLS-1$
		this.borderNode.style.paddingTop = "1px"; //$NON-NLS-1$


        for(var item = 0; item < this.tools.length; item++ ) {
        	if(this.tools[item] && this.tools[item].display) {
       			 switch( this.tools[item].name ){
	 		 		case controls.OUTLINE:
			 			this.addButton(this._messages.outline, dojo.hitch(this, "toggleOutline"), "toggleOutline"); //$NON-NLS-1$ //$NON-NLS-2$
			 			break;
	
	 		 		case WHEELSCROLL:
			 			this.wheel_button = this.addButton(this._messages.wheelzoom, dojo.hitch(this, "toggleMouseWheel"), "wheelScrollOn"); //$NON-NLS-1$ //$NON-NLS-2$
		 			break;

	 		 		case controls.SAVE:
	       			 	this.saveBtn = this.addButton(this._messages.save, dojo.hitch( this, "saveDiagram" ), "saveIcon" ); //$NON-NLS-1$ //$NON-NLS-2$
	       			    this.saveBtn.attr('disabled', true); //$NON-NLS-1$
	       			 	break;

       			 	case controls.SETTINGS:
       			    	this.addSettingsDropDown( this.tools[item].subcontrol );
       			    	break;

       			 	case controls.SEARCHBOX:
       			        this.addSearchBox( this.tools[item].scopes );
       			    	break;

       			 	case controls.RELOAD_DIAGRAM_CODE:
       			    	this.addButton( this._messages.reloadDiagramCode, dojo.hitch( this, "loadDiagramCode" ), "resetIcon" ); //$NON-NLS-1$ //$NON-NLS-2$
       			    	break;

       			 	case controls.UNDO:
       			    	this.addButton( this._messages.undo, dojo.hitch( this, "undo" ), "undoIcon" ); //$NON-NLS-1$ //$NON-NLS-2$
       			    	break;

       			 	case controls.REDO:
       			    	this.addButton( this._messages.redo, dojo.hitch( this, "redo" ), "redoIcon" ); //$NON-NLS-1$ //$NON-NLS-2$
       			    	break;

       			 	case controls.DELETE:
       			    	this.addButton( this._messages.deleteLabel, dojo.hitch( this, "deleteDiagram" ), "deleteIcon" ); //$NON-NLS-1$ //$NON-NLS-2$
       			    	break;

       			 	case controls.EDIT:
       			 		this.addButton( this._messages.edit, dojo.hitch( this, "editConfiguration"), "editButton" ); //$NON-NLS-1$ //$NON-NLS-2$
       			 		break;
       			 		
       			 	case controls.RELOAD_DIAGRAM_CONTENT:
       			 		this.addButton( this._messages.reload, dojo.hitch( this, "reloadView"), "reloadButton" ); //$NON-NLS-1$ //$NON-NLS-2$
       			 		break;
       			 		
       			 	case controls.ZOOM:
       			 		this.addZoom();
       			 		break;
       			 		
       			 	case HIGHLIGHT:
       			 		this.highlight_button = this.addButton(this._messages.highlight, dojo.hitch( this, "toggleHighlight"), "highlightOn");//$NON-NLS-1$ //$NON-NLS-2$
       			 		break;

       			 	case FITTOWINDOW:
       			 		this.addButton( this._messages.fitToWindow, dojo.hitch( this, "fitToWindow" ), "fitIcon"); //$NON-NLS-1$ //$NON-NLS-2$
						break;
						
       			 	default:
       			 		break;
       			 }
       		 }
       	 }

       	 this.addTool(new jazz.ui.toolbar.Separator());

         this.showQueryButton = this.addButton(this._messages.showQuery, function() {}, 'query-icon'); //$NON-NLS-1$
         this.showQueryButton.set('enabled', false); //$NON-NLS-1$

        this.startAnalysisButton = this.addButton(this._messages.startAnalysis, function() {}, 'analysis-icon'); //$NON-NLS-1$
        this.startAnalysisButton.set('enabled', false); //$NON-NLS-1$

       	this.customActionsButton = new jazz.ui.toolbar.MenuButton({
       		label: this._messages.actions,
            labelPressed: this._messages.actions,   // Work around display bug in MenuButton if there is no labelPressed
       		destroyOnClose: true,
       		imgClass: 'relm-views-icon-customAction', //$NON-NLS-1$
       		enabled: false
       	});

       	this.addTool(new jazz.ui.toolbar.Separator());
       	this.addTool(this.customActionsButton);
     },
     
     setEditMode: function(editMode){
    	 if(this.viewer.setEditMode(editMode)){
    		 this.resetZoom();
    	 }
    	 if(editMode){
    		 this.buildPalette();
    	 }
    	 if(this._palettePane){
    		 dojo.style(this._palettePane.domNode, "display", editMode ? "" : "none");//$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
    	 }
    	 if(!this.wheelState) {
    		 this.wheelState = true; // initial state is scroll
    		 this.toggleMouseWheel();
    	 }
    	 if(!this.highlightState) {
    		 this.highlightState = true; // initial state
    		 this.toggleHighlight();
    	 }
     },

     buildPalette: function() {
		if(!this._palettePane){
	 		this._palettePane = new com.ibm.relm.explorer.web.ui.internal.view.edit.Palette({
	 			layout: "horizontal",//$NON-NLS-1$
	 			style: "width:100%"//$NON-NLS-1$
			}, this.paletteNode);
	 		this._palettePane.setEditorProvider(this);
	 		this._palettePane.startup();
		}
     },
     
	addSearchBox: function(scopes) {
		this.searchBox = new jazz.ui.SearchBox({
			scopes: scopes,
			tooltip: this._messages.search,
			onExecute: dojo.hitch(this, "loadDiagram") //$NON-NLS-1$
		});

		dojo.style(this.searchBox.domNode, "width", "250px"); //$NON-NLS-1$ //$NON-NLS-2$
		if(!dojo.isIE) {
			dojo.style(this.searchBox.domNode, "display", "inline-block"); //$NON-NLS-1$ //$NON-NLS-2$
		}

		dojo.style(this.searchBox.domNode, "margin", "0px"); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.style(this.searchBox.domNode, "position", "relative"); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.style(this.searchBox.domNode, "verticalAlign", "bottom"); //$NON-NLS-1$ //$NON-NLS-2$

		this.addTool( this.searchBox );
	},

     addSettingsDropDown: function(){
     },

     fitToWindow: function() {
    	 
    	 var surface = this._getSurface();
    	 if(!surface) return;
    	 
    	 var d1 = surface.children[1].getBoundingBox();
    	 var d2 = dojo.marginBox(this.viewer.domNode);
    	 
    	 var cx = d2.w / 2;
    	 var cy = d2.h / 2;
    	 var d2width = d2.w - 20;  // margin * 2
    	 var d2height = d2.h - 20;  // margin * 2
    	 
    	 var ratioX = d2width / d1.width;
    	 var ratioY = d2height / d1.height;
    	 var scale = (ratioX < ratioY) ? ratioX : ratioY;
    	 // limit scale value
    	 if(scale > 2) { 
    		 scale = 2;
    	 } else if (scale < 0.05) {
    		 scale = 0.05;
    	 }  
    	 
      	 this.viewer.rootLayer.setScale(1);
      	 surface.children[1].setTransform({dx: (cx - (d1.x + d1.width /2))  , dy: (cy - (d1.y + d1.height /2))  });
      	 this.slider.set( 'value', scale ); //$NON-NLS-1$
     },
     
     setSliderValue: function(message) {
    	 if(this._getDiagramEditor() == message.editor){
    		 this.slider.set('value', message.scale); //$NON-NLS-1$
    	 }
     },

     resetZoom: function() {
    	 this.slider.set( 'value', 1 ); //$NON-NLS-1$
    	 this._getDiagramLayer().restore();
    	 if(this.viewer.outline && this.viewer.outline.restore) this.viewer.outline.restore();
     },

     addZoom: function() {
    	 this.slider = new dijit.form.HorizontalSlider({
             name: "slider", //$NON-NLS-1$
             value: 1.0,
             // min/max must be sync with WheelZoom policy which is created in DiagramLayer
             minimum: 0.05,
             maximum: 2.0, 
             intermediateChanges: true,
             style: "width:200px;" //$NON-NLS-1$
         });

    	 dojo.style(this.slider.domNode, {"display": "inline-block", "vertical-align": "middle"}); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
    	 this.addTool( this.slider );
    	 this.percent = this.addButton( "100%", dojo.hitch( this, "resetZoom" ), "resetZoom", true ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
     },

     reWorkLayout: function( id ){
    	this.viewer.getCommandController(this.canvas).execute(id, { containerMapping: this._getDiagramEditor() });
     },

     createMenuItem: function( menu, label, id, call, icon ) {
    	 menu.addChild( new dijit.MenuItem({
        	label: label,
        	id: id,
        	onClick: dojo.hitch( this, "reWorkLayout", call  ), //$NON-NLS-1$
        	iconClass: icon
        }) );
     },

	// Mixin - callback
	_setDirty: function(){},

	saveDiagram: function() {
	},

	deleteDiagram: function() {
	},
	
	editConfiguration: function() {
		if(this.diagramView) {	
			this.diagramView._onEdit();
		}
	},

	reloadView: function() {
		if(this.diagramView) {	
			this.diagramView._onReload();
		}
	},
	
	initializeDiagram: function(){
		this.configuration.canvas = this.canvas;
		this.configuration.slider = this.slider;
		this.configuration.percent = this.percent;

		this.viewer  = new com.ibm.relm.explorer.web.ui.internal.view.DiagramWidget({ configuration: this.configuration, viewUri: this.viewUri, webBaseUri: this.webBaseUri}, this.canvas);

		dojo.marginBox( this.viewer.domNode,  "content-cell" ); //$NON-NLS-1$
		this.viewer.startup();
		
		var diagramLayer = this._getDiagramLayer();
		if(diagramLayer){
			this.connect(diagramLayer, "onConfigLoaded", dojo.hitch(this, function(config){//$NON-NLS-1$
				if(this.printView){
					if(config && config.diagramName){
						window.document.title = config.diagramName;
					}
//					setTimeout(window.print, 1000);
				}
				this._updateToolbarForSelection();
			}));			
		}
		
		var editRoot = this._getDiagramEditRoot();
		if(editRoot) {
			if(editRoot.getSelectionController){
				var selectionController = editRoot.getSelectionController();
				if(selectionController){
					this.connect(selectionController, "onSelectionChanged", "handleSelection" ); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}		

		if(this.initialZoomLevel != 1.0){
			this.viewer.setScale(this.initialZoomLevel, 0, 0);
		}
		
		this.viewer.setOutline( this.outlineState );
	},
	
	removeDynamicPolicyTools: function() {
		if (this._dynamicPolicyTools && this._dynamicPolicyTools.length > 0) {
            var tool;
			for (var i = 0; tool = this._dynamicPolicyTools[i]; i++) {
				this.removeTool(tool);
			}
		}
		this._dynamicPolicyTools = [];
	},

	addDynamicPolicyTools: function() {
		var editRoot = this._getDiagramEditRoot();
		if(editRoot){
			this.selection = editRoot.getSelectionController().getSelection();
		}

        var mapping;
		for(var i = 0; mapping = this.selection[i]; i++) {
			for (var policyID in mapping.policies) {
				if (mapping.policies[policyID].getToolbarWidgets) {
					var widgets = mapping.policies[policyID].getToolbarWidgets();
					for (var j=0, widget; widget=widgets[j]; j++) {
						this._dynamicPolicyTools.push(widget);
						this.addTool(widget);
					}
				}
			}
		}
	},

	handleSelection: function(){
	},

	addTool: function(item){
		if(!this.toolbar){
			console.warn("Layout.toolbar doesn't exist");//$NON-NLS-1$
			return;
		}
		this.toolbar.addChild(item);
	},

	removeTool: function(item){
		item.destroyRecursive();
	},

	layout: function() { },

	toggleOutline: function() {
		this.outlineState = !this.outlineState;
		this.viewer.setOutline( this.outlineState );
	},

	toggleMouseWheel: function() {
		this.wheelState = !this.wheelState;
		var diagramLayer = this._getDiagramLayer();
		if(diagramLayer){
			if(this.wheelState) {
				// use wheel scroll
				diagramLayer.removePolicyForWheelZoom();
				diagramLayer.createPolicyForWheelMove();
				this.wheel_button.set("imgClass", "wheelScrollOn"); //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				diagramLayer.removePolicyForWheelMove();
				diagramLayer.createPolicyForWheelZoom();
				this.wheel_button.set("imgClass", "wheelScrollOff"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}		
	},
	
	toggleHighlight: function() {
		this.highlightState = !this.highlightState;
		if(this.highlightState) {
			this.highlight_button.set("imgClass", "highlightOn"); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			this.highlight_button.set("imgClass", "highlightOff"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		this._getDiagramEditor().setHighlightState(this.highlightState);
	},

	resize: function(s) {
		if(!s) {
			s = dojo.marginBox(this.containerNode.parentNode);
		}
//		console.log(s);

		if(s.w <= 0) s.w = 1; // for IE

		if(this.toolbar){
			this.toolbar.domNode.style.width = "" + s.w + "px"; //$NON-NLS-1$ //$NON-NLS-2$
			s.h = s.h - 30; // height of Toolbar...
		}

		if(this.viewer) {
			this.viewer.resize(s);
		}
	},

	undo: function() {
		this.viewer.getCommandController(this.canvas).commandStack.undo();
	},

	redo: function() {
		this.viewer.getCommandController(this.canvas).commandStack.redo();
	}
});

})();

