/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2016. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.MoveView"); //$NON-NLS-1$

dojo.require("com.ibm.relm.explorer.web.ui.internal.view.ViewClient"); //$NON-NLS-1$
dojo.provide("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

//dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler"); //$NON-NLS-1$

dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.Tree"); //$NON-NLS-1$
dojo.require("jazz.ui.tree.IconItem"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ViewMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { // module container

var ServiceResponseHandler = com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler;
var ViewClient = com.ibm.relm.explorer.web.ui.internal.view.ViewClient;

/**
 * Widget to move a report or folder
 */
var MoveView = dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.MoveView", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/templates/MoveView.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			dojo.mixin(this, args);

			this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ViewMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
				
		postCreate: function() {
			this.inherited(arguments);
			
			this._responseCount = 0;
			this._saving = false;
			this._allFolders = [];
			// A map mapping folder nodes to the underlying folder
			this._folderMap = {};
			
			this._createFolderTree();
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			
			if(this.shared) {
				// check permission - need delete permission to move resources
				this._moveButton.disabled = !this.permissions.deletePermission;
			}
			
			var args = {
//				ownerUUID: session.getAuthenticatedContributor().itemId,
					scope: "personal" //$NON-NLS-1$
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders", //$NON-NLS-1$
					"_remoteError"); //$NON-NLS-1$
			ViewClient.getFolderDescriptors(responseHandler, args);

			args = {
					scope: "shared" //$NON-NLS-1$
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders", //$NON-NLS-1$
					"_remoteError"); //$NON-NLS-1$
			ViewClient.getFolderDescriptors(responseHandler, args);

			this._updateEnablement();
		},
		
		_selectFolder: function(folderUUID)
		{
			//Get the itemID from the folder map based on the folder UUID
			//the select that item in the tree
			for (var key in this._folderMap)
			{
				//If the folder in the map is the same as the supplied folder UUID then get the item
				if (this._folderMap[key].folderUUID == folderUUID)
				{
					//Get the item from the tree
					var folderToSelect = this._folderTreeWidget.getItem(this._folderMap[key].treeItemID);
					//Expand the tree down to the folder to select
					var parentItem = folderToSelect.getParent();
					while (parentItem != null) 
					{
						parentItem.setExpanded(true);
						parentItem = parentItem.getParent();
					}
					//select it
					this._folderTreeWidget.setSelection(folderToSelect);
					this._originalParentNode = folderToSelect;  //save this for comparison with the folder to move/copy to					
					break;
				}
			}
		},
		
		_removeFolderFromTree: function(folderUUID)
		{
			for (var key in this._folderMap)
			{
				//If the folder in the map is the same as the supplied folder UUID then get the item
				if (this._folderMap[key].folderUUID == folderUUID) 
				{
					//Get the item from the tree
					var folderItem = this._folderTreeWidget.getItem(this._folderMap[key].treeItemID);
					var parentItem = folderItem.getParent();
						parentItem.removeChild(folderItem, false);
					break;
				}
			}
		},
				
		_moveOrCopyChanged: function(ev){
			this._updateEnablement();
		},
		
		_handleSave: function() {
			if(this._saving) return;
			
			this._saving = true;
			this.messageBox.hideMessage();
			dojo.removeClass(this._messageBoxContainer, "messageBoxVisible"); //$NON-NLS-1$
			var args = {};
			
	        var currentFolderNode = this._folderTreeWidget.getSelection();
	        var currentFolder = this._folderMap[currentFolderNode];
	        if (!currentFolder || currentFolder == null) {
	        	this._showErrorMessage(this.messages.pleaseSelectFolder);
	        	return;
	        }
			
			//This is the no-op case where the user tries to move the item to it's original folder.
			if (this._originalParentNode == currentFolderNode[0])
				this._saveComplete();
	        
			
			var isAMove = this._moveButton.checked;

			this._saveButton.disabled = true;
			this._movingTypeCount = 0;
			
			this._operationInfo = {isMove: isAMove, targetShared: currentFolder.shared};
			if (this.configURIs && this.configURIs.length > 0) //Moving queries
			{
				this._movingTypeCount++;				
				args.configURIs = this.configURIs;
				//Set the correct folder to move/copy the report to.
				args.parentUUID = currentFolder.folderUUID;

				var responseHandler = new ServiceResponseHandler(this, "_saveComplete", "_remoteError"); //$NON-NLS-1$ //$NON-NLS-2$
				if (isAMove) 
					ViewClient.postMoveConfigDescriptors(responseHandler, args);
				else
					ViewClient.postCopyConfigDescriptors(responseHandler, args);
			}
			if (this.folderUUIDs && this.folderUUIDs.length > 0)  //Moving folders
			{
				this._movingTypeCount++;				
				args.folderUUIDs = this.folderUUIDs;
				//Set the correct folder to move/copy the report to.
				args.parentUUID = currentFolder.folderUUID;

				var responseHandler = new ServiceResponseHandler(this, "_saveComplete", "_remoteError"); //$NON-NLS-1$ //$NON-NLS-2$
				if (isAMove) 
					ViewClient.postMoveFolderDescriptors(responseHandler, args);
				else
					ViewClient.postCopyFolderDescriptors(responseHandler, args);
			}
		},
		
		_handleCancel: function() 
		{
			if (this.cancelCallback != null)
				this.cancelCallback();
		},
		
		_saveComplete: function(args) 
		{
			this._movingTypeCount--;				
			if (this.okCallback != null && this._movingTypeCount <= 0)
				this.okCallback(this._operationInfo);
			this._saving = false;
		},
		
		_remoteError: function(error) 
		{
			this._movingTypeCount--;				

			// Check if the content is still displaying
			if (!dojo.isDescendant(this._saveButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this._showErrorMessage(this.messages.invokeServiceError + error.message);
			this._saving = false;
		},

		_getFoldersFromResult: function(response){

			return dojo.map(response.results.bindings, function(r){
				return { 
					"name": {  //$NON-NLS-1$
						label: r.name.value, 
						type: "uri",  //$NON-NLS-1$
						value: r.uri.value
					}, 
					"description": r.description.value, //$NON-NLS-1$
					"creator": r.creator.value, //$NON-NLS-1$
					"modified": r.modified.value,            			 //$NON-NLS-1$
					"ownedBy": r.ownedBy.value, //$NON-NLS-1$
					"shared" : (r.scope.value == "shared") ? true: false, //$NON-NLS-1$ //$NON-NLS-2$
					"parentUUID": r.parentUuid.value, //$NON-NLS-1$
					"folderUUID": r.folderUUID.value //$NON-NLS-1$
				};
			});
		},

		_addFolders: function(response) {
			var folders = this._getFoldersFromResult(response);
			this._responseCount++;
			if (!folders) folders = [];
			for (var i = 0; i < folders.length; i++) {
				this._allFolders.push(folders[i]);
			}
			if (this._responseCount == 1) {
				return;
			}
			this._addFoldersToTree();
			
			//If there is a parent folder, select it
			if (this.parentUUID != "") //$NON-NLS-1$
				this._selectFolder(this.parentUUID);
			
			//If we are moving folders, remove the folders being moved and their children from the tree
			if (this.folderUUIDs != null && this.folderUUIDs.length > 0) 
			{
				for (var i = 0; i < this.folderUUIDs.length; i++) 
					this._removeFolderFromTree(this.folderUUIDs[i]);
			}
		},
		
		_addFoldersToTree: function() {
			// First, find the two root folders
			var sharedRoot = null;
			var privateRoot = null;
			for (var i = 0; i < this._allFolders.length; i++) {
				var folder = this._allFolders[i];
				if (folder.parentUUID == "") { //$NON-NLS-1$
					if (folder.shared) {
						sharedRoot = folder;
					} else {
						privateRoot = folder;
					}
				}
			}

			if (privateRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: this.messages.myQueries, iconClass: "iconFolder"}); //$NON-NLS-1$
				privateRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderMap[rootItem] = privateRoot;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, privateRoot, this._allFolders, false, false);
			}
			if (sharedRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: this.messages.sharedQueries, iconClass: "iconFolder"}); //$NON-NLS-1$
				sharedRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderMap[rootItem] = sharedRoot;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, sharedRoot, this._allFolders, false, true);
			}
		},
		
		/**
		 * From the list of folders, find any that are children of parentFolder, and
		 * add them to the tree. Then recurse for each of the children.
		 */
		_addChildrenForFolder: function(parentFolderNode, parentFolder, folders, parentIsRoot, parentIsShared) {
			var foldersToAdd = [];
			for (var i = 0; i < folders.length; i++) {
				// Special case where the parent is a root system folder
				if (parentIsRoot) {
					if (!folders[i].parentUUID && (folders[i].shared == parentIsShared) && (folders[i].name.label != parentFolder.name.label)) {
						foldersToAdd.push(folders[i]);
					}
				} else {
					if (folders[i].parentUUID == parentFolder.folderUUID) {
						folders[i].shared = parentFolder.shared;
						foldersToAdd.push(folders[i]);
					}
				}
			}
			foldersToAdd = foldersToAdd.sort(function(a, b) {return a.name.label > b.name.label;});
			for (var i = 0; i < foldersToAdd.length; i++) {
				var childNode = new jazz.ui.tree.IconItem({label: com.ibm.relm.explorer.web.ui.internal.util.escapeHtml(foldersToAdd[i].name.label), iconClass: "iconFolder"}); //$NON-NLS-1$
				foldersToAdd[i].treeItemID = childNode.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderMap[childNode] = foldersToAdd[i];
				parentFolderNode.addChild(childNode);
				this._addChildrenForFolder(childNode, foldersToAdd[i], folders, false, parentIsShared);
			}
		},

		_createFolderTree: function() {
			this._folderTreeWidget = new jazz.ui.Tree();
			this._folderTree.appendChild(this._folderTreeWidget.domNode);
			this._folderTreeWidget.onSelectionChanged = dojo.hitch(this,this._handleFolderSelectionChange);			
		},
		
		_handleFolderSelectionChange: function(selection)
		{
			var selectedFolder = this._folderMap[selection];
			if(this._noCreatePermission(selectedFolder)) {
				alert(this.messages.noPermissionToCreateView);
			}
			this._updateEnablement();			
		},

		_updateEnablement: function() {
			this._saveButton.disabled = (!this._moveButton.checked && !this._copyButton.checked) || 
										this._folderTreeWidget.getSelection().length == 0 ||
										this._noCreatePermission(this._folderMap[this._folderTreeWidget.getSelection()[0]]);
		},

		_noCreatePermission: function(selectedFolder) {
			if(selectedFolder.shared) {
				return !this.permissions.createPermission;
			} else {
				return false;
			}
		},
		
		_showErrorMessage: function(message) {
			var status = {
				type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary : message,
				closable : false
			};
			dojo.addClass(this._messageBoxContainer, "messageBoxVisible"); //$NON-NLS-1$
			this.messageBox.showMessage(status);
		}
	}
);

MoveView.CSH_NEW_VIEW_EDITOR = "com.ibm.jazz.reports.web.cshelp.new_view_editor"; //$NON-NLS-1$

})();