/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.ResourceList"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
dojo.require("jazz.ui.FilterBox"); //$NON-NLS-1$
dojo.require("jazz.ui.PaginationControl"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("dojo.date.stamp"); //$NON-NLS-1$
dojo.require("dojo.date.locale"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ResourceLinkBatchLoader'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ResourceLinkScrollLoader'); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ResourceListMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var DATE_XML_NAMESPACE = "http://www.w3.org/2001/XMLSchema#dateTime"; //$NON-NLS-1$
var getAuthenticatedContributorUri	= com.ibm.team.repository.web.client.session.getAuthenticatedContributorUri;
var DEFAULT_PAGE_SIZE = 25;


/**
 * Gets the complete text content of a node, no matter the browser.
 */
function scrapeText(node) {
	return node.textContent || node.innerText;
}


dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.ResourceList",  //$NON-NLS-1$
	[dijit._Widget, dijit._Templated],
{
	// variables
	pageSize: DEFAULT_PAGE_SIZE,
	pageIndex: 0,
	sortColumn: null,
	sortDir: -1,
	mergeCell: true,
	_downArrowImageUrl: dojo.moduleUrl("com.ibm.relm.explorer.web","explorer/images/down-arrow.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	_upArrowImageUrl: dojo.moduleUrl("com.ibm.relm.explorer.web","explorer/images/up-arrow.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	_resources: [],
	_totalResult: 0,
	_userSelectedPageSize: DEFAULT_PAGE_SIZE,
	permissions: {},

	constructor: function(args, node) {
		var batchLoader = com.ibm.relm.common.web.ResourceLinkBatchLoader();
		this._scrollLoader = com.ibm.relm.common.web.ResourceLinkScrollLoader(batchLoader);
	},

	_refreshContent: function(pageUrl, reload) {
		this._scrollLoader.stopAndReset();
		// sub class should override and call this.inherited(arguments);
	},
	
	// template
    widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web","ui/internal/view/templates/ResourceList.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._msgs = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ResourceListMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function(){
		this.inherited(arguments);
		this.paging.onNextClick = dojo.hitch(this, this._paging);
		this.paging.onPreviousClick = dojo.hitch(this, this._paging);
	},
	
	getTitleString: function(){
		return this.titleText.innerHTML;
	},
	
	showEditButton: function(show){
		if(this.editButton){
			this.editButton.domNode.style.display = show ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
		}
	},

	// tool-bar actions
	onReload: function() {
    	this._refreshContent(null, true);
    },
    
    onEditQuery: function() {
    },

    _getPageSize: function() {
    	return this._userSelectedPageSize;
    },
    
	_pageSizeChanged: function() {
		this._userSelectedPageSize = this.pageSize = this.pageSizeCombo.options[this.pageSizeCombo.selectedIndex].value - 0;
		this.pageIndex = 0;
		// disable pagination immediately after page size change
		this.paging.startPage = 0;
		this.paging.setTotal(0);
		this._refreshContent(null, false);
	},
	
	_paging: function(widget, start, end){
		this.pageIndex = Math.floor(start / this.pageSize);
		this._refreshContent();
	},

	_updatePaginationBar: function(res, totalResults, noPaging){
		// Page size combo
		var max = totalResults || res.length || 0; 

		this.pageSizeComboContainer.style.display = noPaging ? "none" : ""; //$NON-NLS-1$ //$NON-NLS-2$
		if(!noPaging){
			var combo = this.pageSizeCombo;
			combo.options.length = 0;
			var size=0;
			var pageSizeArray = [10, 25, 50, 100, 250];
			for(var i=0; i<pageSizeArray.length; i++){
				var s = pageSizeArray[i];
		        if(max && max < s) break;
		        var option = combo.options[size++] = new Option(s, s);
		        if(this.pageSize == s){
		        	option.selected = true;
		        }
			}
	        var option = combo.options[size++] = new Option(this._msgs.ShowAll, -1);
	        if(this.pageSize < 0 || max < this.pageSize) {
	        	option.selected = true;     
	        }
		}

        // paging bar
        if(this.pageSize < 0) this.pageSize = max;
        this.paging.startPage = this.pageIndex;
        this.paging.increment = noPaging ? max : this.pageSize;
		this.paging.setTotal(max);
	},
	
	_removeContent: function(){
		dojo.empty(this.tableHeader);
		dojo.empty(this.tableBody);
	},
	
	_displayList: function(info){
		this._scrollLoader.stopAndReset();
		var resources = info.resources;
		var showIndex = info.showIndex;
		var startIndex = info.startIndex || 1; 
		var showActions = info.showActions;
		var columns = info.columns;
		var thead = info.thead || this.tableHeader;
		var tbody = info.tbody || this.tableBody;
		
		// remove list first
		dojo.empty(thead);
		dojo.empty(tbody);
		
		var self = this;
		var format = function(value, td, resource, dontCreateLink, isPredicate){
			if(!value) return null;

			var displayValue = dojo.isString(value) ? value : value.value; // value might be object of string
			displayValue = jazz.util.html.removeScript(displayValue);
			var sortValue = value.value;
			if(value.type == DATE_XML_NAMESPACE || value.datatype == DATE_XML_NAMESPACE){
				sortValue = dojo.date.stamp.fromISOString(value.value); // convert created to Date type
				if(sortValue){
					displayValue = dojo.date.locale.format(sortValue, {formatLength: "medium"}); //$NON-NLS-1$
				}
			} else if (value.type == "uri"){ //$NON-NLS-1$

				var hadIcon = false;
				if (value.iconSrc) {
					// Use given icon if available
					dojo.create('img', { //$NON-NLS-1$
						src: value.iconSrc,
						className: 'resourceIcon' //$NON-NLS-1$
					}, td);
					hadIcon = true;
				}
				
				if(displayValue) {
					displayValue = value.label || decodeURIComponent(displayValue.substring(displayValue.lastIndexOf('/') + 1)); //$NON-NLS-1$
		            if(!resource._explorerTemp_._uri_){
		            	// used in onDelete
		            	// assumes the first uri is the resource uri
		            	resource._explorerTemp_._uri_ = value.value;
		            	resource._explorerTemp_._name_ = displayValue;
		            }
		            sortValue = displayValue; // sort by displayed string
		            if(!dontCreateLink){
		            	var href = (isPredicate) ? value.value : "#edit"; //$NON-NLS-1$
						var link = dojo.create("a", {href: href, innerHTML: displayValue, title: value.value}, td); //$NON-NLS-1$
			            link.resource = resource;
			            link._uri_ = value.value;
			            link._conceptResourceUri_ = value.conceptResourceUri;
			            link._name_ = displayValue;
			            link._isPredicate_ = isPredicate;
			            self.connect(link, "onclick", function(evt){ //$NON-NLS-1$
			            	self.onResourceSelected(evt.target);
			                dojo.stopEvent(evt);
			            });
			            if(!isPredicate){
			            	var previewIconNode = undefined;
			            	if (!hadIcon && value.isLink) {
			            		// No icon, so place-hold with a blank. We'll get the OSLC preview
			            		// icon if possible to replace. We use a 16x16 blank so that the text
			            		// doesn't shift when reloading.
			            		previewIconNode = dojo.create('img', { //$NON-NLS-1$
			            			src: self._blankGif,
			            			className: 'resourceIcon' //$NON-NLS-1$
			            		}, td, 'first'); //$NON-NLS-1$
			            	}

							var jazzLink = new jazz.ui.ResourceLink( { 
								uri: value.value, 
								lazyFetch: true, 
								retainLinkText: value.label !== undefined,
								iconNode: previewIconNode, 
								useAbbreviation: false}, link);

							self._scrollLoader.watchResourceLink(jazzLink);
							jazzLink.connect(jazzLink, '_applyTitle', dojo.hitch(self, self._setNeedsFilterChanged)); //$NON-NLS-1$
			            }
		            	displayValue = null;
		            }
				}
			}
			
			if (td && displayValue){
				td.innerHTML = displayValue;
				td.title = value.datatype || value.type;
			}

			return sortValue;
		};
		
		if(!columns){
			columns = [];
			var resource = resources[0];
			for(var n in resource){
				if(n != "_explorerTemp_"){ //$NON-NLS-1$
					columns.push({id: n, label: n, sort: "_by_" + n}); //$NON-NLS-1$
				}
			}
		}

		var tr = dojo.create("tr"); //$NON-NLS-1$
		var td;
		if(showIndex) dojo.create("th", {innerHTML: "#", className: "columnheader",scope:'col'}, tr); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		for(var i=0; i<columns.length; i++){
			var c = columns[i];
			td = dojo.create("th", {innerHTML: c.label + " ", className: "columnheader",scope:'col'}, tr); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			if(c.sort){
				td.sortColumn = c.sort;
				this.connect(td, "onclick", function(evt){ //$NON-NLS-1$
					this._sortResources(evt.target.sortColumn, info);
				});
				this["sort" + c.sort] = dojo.create("span", null, td); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		if(this.sortColumn){
			this["sort" + this.sortColumn].innerHTML =  //$NON-NLS-1$
				this.sortDir > 0 
					? "<img src='" + this._downArrowImageUrl + "'/>"  //$NON-NLS-1$ //$NON-NLS-2$
					: "<img src='" + this._upArrowImageUrl + "'/>"; //$NON-NLS-1$ //$NON-NLS-2$
		}

		if(showActions){
			td = dojo.create("th", {innerHTML: this._msgs.Actions, className: "columnheader actioncolumn",scope:'col'}, tr); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		thead.appendChild(tr);

		// display contents
		var aboveRes = null;
		for(var i=0; i<resources.length; i++){
			// we only support creator and not support ownedBy right now. 
			var res = resources[i];
			var loginUser = getAuthenticatedContributorUri();
			var isOwner = res.creator && (loginUser == res.creator.value);
			
			res._explorerTemp_ = {}; // used by sort
			tr = dojo.create("tr"); //$NON-NLS-1$
			if(showIndex) td = dojo.create("td", {innerHTML: i+startIndex}, tr); // index //$NON-NLS-1$
			var canBeMerged = this.mergeCell;
			for(var j=0; j<columns.length; j++){
				var col = columns[j];
				var cell = dojo.create("td", null, tr); //$NON-NLS-1$
				var cellValue = res[col.id];
				var dontCreateLink = false;
				if(canBeMerged){
					if(cellValue && /* cellValue.type == "uri" && */ aboveRes && aboveRes[col.id] 
						&& ((cellValue.value !== undefined && cellValue.value === aboveRes[col.id].value) || cellValue === aboveRes[col.id])){
						// if cell content is same as above cell's resource, then merge this cell to above cell
						dojo.addClass(cell, "merged"); //$NON-NLS-1$
						dontCreateLink = true;
					}else{
						canBeMerged = false;
					}
				}
				var sortVal = format(cellValue, cell, res, dontCreateLink, col.isPredicate); 
				res._explorerTemp_[col.sort] = dojo.isString(sortVal) ? sortVal.toLowerCase() : sortVal;					
			}
			if(showActions){
				// actions
				td = dojo.create("td", null, tr); //$NON-NLS-1$
				if( (this.scope == "personal" || isOwner || this.permissions.ModifyPermission ) && dojo.isFunction(this.onEdit)) { //$NON-NLS-1$
					// Modify
					link = dojo.create("button", {title: this._msgs.EditAction}, td); //$NON-NLS-1$
					dojo.addClass(link, "edit"); //$NON-NLS-1$
					link.resource = res;
					this.connect(link, "onclick", function(evt){ //$NON-NLS-1$
						self.onEdit(evt.target.resource);
						dojo.stopEvent(evt);
					});					
				}
				if((this.scope == "personal" || isOwner || this.permissions.deletePermission) && dojo.isFunction(this.onDelete)) { //$NON-NLS-1$
					// Delete
					link = dojo.create("button", {title: this._msgs.DeleteAction}, td); //$NON-NLS-1$
					dojo.addClass(link, "delete"); //$NON-NLS-1$
					link.resource = res;
					this.connect(link, "onclick", function(evt){ //$NON-NLS-1$
						self.onDelete(evt.target.resource);
						dojo.stopEvent(evt);
					});
				}
			}
			
			// add row
			tbody.appendChild(tr);
			aboveRes = res;
		}
		
		this._lastDisplayedInfo = info;
		this._scrollLoader.start();
	},

	/**
	 * Queues up a refresh of the filter text results. This is needed
	 * if OSLC previews come in later and update the titles. We then
	 * need to re-run the filter. This won't filter yet, but will wait
	 * til the next run loop so that multiple calls in the same loop
	 * only cause one filter at the end.
	 */
	_setNeedsFilterChanged: function() {
		clearTimeout(this._filterTimeout);

		var self = this;
		this._filterTimeout = setTimeout(function() {
			self._filterChanged();
		}, 0);
	},
	
	_sortResources: function(column, info){
		var info2 = dojo.mixin({}, info);
		var sortDir = this.sortDir;
		info2.resources = info.resources.sort(function(a, b){
			var aval = a._explorerTemp_[column];
			var bval = b._explorerTemp_[column];
			var ret = 0;
			if(sortDir > 0) {
				ret = (aval > bval) ? 1 : -1;
			} else {
				ret = (aval < bval) ? 1 : -1;
			}
			return ret;
		});
		this.sortDir = sortDir * -1;
		if(this.sortColumn){
			this["sort" + this.sortColumn].innerHTML = ""; //$NON-NLS-1$ //$NON-NLS-2$
		}
		this.sortColumn = column;
		this._displayList(info2);
	},
	
    errorHandler: function(response, ioargs){
    	var message = com.ibm.relm.explorer.web.ui.internal.util.getXhrErrorMessage(response, ioargs);
    	this.messageArea.showMessage(message);
        if(response.responseText){
       		response.responseText = null; // don't show the message in error dialog
        }
	},
	
	/**
	 * Called when the filter text changes of the list changes in some way
	 * and the filter needs to be reapplied.
	 */
	_filterChanged: function(){
		var tbody = this._lastDisplayedInfo.tbody || this.tableBody;
		var searchString = this.filterBox.get('value'); //$NON-NLS-1$
		this._filterResults(searchString, tbody);
		this._scrollLoader.positionsDidChange();
	},

	/**
	 * Simply hides rows that don't match a filter string.
	 * @param  {string} searchString text to search for.
	 * @param  {TBODY} tbody        the body of the table of results to filter.
	 */
	_filterResults: function(searchString, tbody) {
		var terms = null;
		if (searchString) {
			searchString = dojo.trim(searchString).toLowerCase();
			terms = searchString.split(/\s+/);
		}

		var rows = tbody.rows;

		for (var i = 0; i < rows.length; i++) {
			var row = rows[i];

			if (!searchString) {
				row.style.display = ''; //$NON-NLS-1$
				continue;
			}

			var textContent = scrapeText(row).toLowerCase();

			var matches = dojo.every(terms, function(term) {
				return textContent.search(term) !== -1;
			});

			if (matches) {
				row.style.display = ''; //$NON-NLS-1$
			}
			else {
				row.style.display = 'none'; //$NON-NLS-1$
			}
		}
	}
	
});
})();
