/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.SetParametersTable"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("jazz.ui.HoverPopup"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils"); //$NON-NLS-1$


dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "CreateNewDiagramWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.relm.explorer.web.explorer.queryparams.paramprompt", "DataPromptMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { 

var util = com.ibm.relm.explorer.web.ui.internal.util;
var enumUtils = com.ibm.relm.explorer.web.ui.internal.view.common.EnumerationUtils;
var HoverPopup = jazz.ui.HoverPopup;
var CLASS_INLINE = "dijitInline";  //$NON-NLS-1$

dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.common.SetParametersTable",  //$NON-NLS-1$
			[dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: "<div class='com-ibm-relm-explorer-setparametertable'><div dojoType='com.ibm.relm.explorer.web.ui.internal.view.common.Table' dojoAttachPoint='_paramTable' selectable='true' style='table-layout: fixed; width:100%;'></div></div>",//$NON-NLS-1$
	
	setParameters: function(params){
		enumUtils.loadEnum(dojo.hitch(this, function(enums) {
			this.enums = enums;
			var isEnum = false;
			dojo.some(params, function(param) {
				var type = param.type;
				if (enumUtils.isEnumType(type, this.enums)) {
						this._setParameters(params, this.enums);
					isEnum = true;
					return true;
				}
			}, this);
			
			// If there is no enum type in params, call immediately
			if (!isEnum) {
				this._setParameters(params);
			}
		}));
	},
	
	_setParameters: function(params, enums){
		this.parameters = params;

		var propTable = this._paramTable;
		propTable.clearItems();
		
		var self = this;

		if(this.parameters) {
			dojo.forEach(this.parameters, function(param) {
				var type =  param.type;
				var options = null;
				if (enums && enumUtils.isEnumType(type, enums)) {
					var category = enumUtils.getEnumCategory(type, enums);
					options = enumUtils.getEnumValueOptions(enums, category, self.datamessages.selectValue); 
				} else {
					options = null;
				}
				
				propTable.addItem({name: param.name, 
									value: param.value,
									description: param.description, 
									param: param,
									editButton: self._getEditButton(type, param.value),//$NON-NLS-1$
									buttonTarget: "value"//$NON-NLS-1$
									}, {value: options}, 
									function(data, id, newValue, rowWidget) {
										if (id == "value") {//$NON-NLS-1$
											// push newValue back to data
											data.param[id] = newValue;
										}
									}, this);
			});
		}
	},
	
	onUpdateParamValue: function(){
		
	},

	postMixInProperties: function() {
		this.inherited(arguments);
		this._popups = [];		
		this.messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "CreateNewDiagramWizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.datamessages = dojo.i18n.getLocalization('com.ibm.relm.explorer.web.explorer.queryparams.paramprompt', 'DataPromptMessages'); //$NON-NLS-1$ //$NON-NLS-2$
		this._infoIconSrc = dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/images/info_ovr.gif"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function(){
		this.inherited(arguments);
		dojo.style(this.domNode, {"table-layout": "fixed"}); //$NON-NLS-1$//$NON-NLS-2$ 
		this._initTable();
	},

	destroy: function(){
		this.inherited(arguments);		
		dojo.forEach(this._popups, function(popup){popup.destroy();});
	},
	
	_getEditButton: function(type, value){
		var button = null;
		if(util.hasPicker(type)){
			button = dojo.create("a", { //$NON-NLS-1$
				href: "#", //$NON-NLS-1$
				title: this.messages.openSelectDialog,
				className: "dijitInline actionIcon" //$NON-NLS-1$
			}); 
			this.connect(button, "onclick", function(ev){//$NON-NLS-1$
				var initialValue = button.targetColumn ? button.targetColumn.get("value") : value;  //$NON-NLS-1$
				util.openPicker(type, initialValue, this.messages.paramSelectParameterValue, function(value){
					if(value.uri){
						value = value.uri; // artifact picker and product picker
					}
					button.targetColumn.set("value", value); //$NON-NLS-1$
					this.close();
				});
				dojo.stopEvent(ev);
			});
			dojo.create("span", { //$NON-NLS-1$
				className: "dijitInline selectButton" //$NON-NLS-1$
			}, button);

		}
		return button;
	},

	_initTable: function() {
		
		var self = this;
		var propTable = this._paramTable;
		
		propTable.setupColumns([
		    {
		    	id: "name",  //$NON-NLS-1$
		    	name: this.messages.parameterName, 
		    	style: "width: 30%", //$NON-NLS-1$
		    	getCellContent: function(cellData) {
		    		var rowWidget = this;
		    		var div = dojo.create("span"); //$NON-NLS-1$
		    		dojo.create("label", {innerHTML : util.escapeHtml(cellData)}, div); //$NON-NLS-1$
		    		var desc = rowWidget.data.description;
		    		if(desc && desc.length > 0) {
		    			var image = dojo.create("img", {src : self._infoIconSrc, alt : "Info"}, div); //$NON-NLS-1$//$NON-NLS-2$

		    			self._popups.push(new HoverPopup({
		    				around: image,
		    				html: util.escapeHtml(desc),
		    				openDelay: 300
		    			}));
		    		}
		    		return div;
		    	}
		    	
		    },
		    {
		    	id: "value", //$NON-NLS-1$
		    	name: this.messages.parameterValue, 
		    	style: "width: 70%", //$NON-NLS-1$
		    	getCellContent: function(cellData) {
		    		var rowWidget = this;
		    		var editBox = new dijit.InlineEditBox({
		    			title : cellData,
		    			style: {width : "100%"}, //$NON-NLS-1$
		    			noValueIndicator: "<span class='noValueIndicator'>" + self.messages.noValueIndicator + "</span>", //$NON-NLS-1$ //$NON-NLS-2$
		    			onChange : function(value) {
		    				var paramName = rowWidget.data.name;
		    				// set paramName, paramValue to model
		    				self._updateParamData(paramName, value);
		    			}
		    		}, dojo.create("div", { //$NON-NLS-1$
		    			className : CLASS_INLINE,
		    			innerHTML : cellData
		    		}));
		    		return editBox;
		    	}
		    },
  		    {
  		    	id: "assist", //$NON-NLS-1$
  		    	name: this.messages.inputAssist,
		    	style: "width: 60px", //$NON-NLS-1$
  		    	isAction: true
  		    }
		]);
	},
	
	_updateParamData: function(name, value) {
		if(this.parameters) {
			dojo.forEach(this.parameters, function(param){
				if(param.name == name) {
					param.value = value;
				}
			});
		}
		this.onUpdateParamValue();
	}
});

})();