/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit._KeyNavContainer"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.util"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "CommonMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var CLASS_ROW_HOVER = "rowHover"; //$NON-NLS-1$
var CLASS_SELECTED = "selected"; //$NON-NLS-1$
var CLASS_TABLE_ROW = "com-ibm-relm-explorer-common-Row"; //$NON-NLS-1$
var CLASS_ACTION_CELL = "actionCell"; //$NON-NLS-1$
var CLASS_ACTION_HEADER_CELL = "actionHeaderCell"; //$NON-NLS-1$

var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;

var messages = {};

var escapeHtml = com.ibm.relm.common.web.ui.util.escapeHtml;
var unescapeHtml = com.ibm.relm.common.web.ui.util.unescapeHtml;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.common.Table", //$NON-NLS-1$
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/common/templates/Table.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	hasHeader: true,
	columns: null, // an array of objects : [ { id:"col1", name:"Column 1" }, { id:"col2", name:"Column 2" },...]

	selectable: false,
	_selected: null,
	
	postCreate: function(){
		messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "CommonMessages"); //$NON-NLS-1$ //$NON-NLS-2$

		this.inherited(arguments);
		
		// setup columns
		this.setupColumns(this.columns || []);
	},
	
	setupColumns: function(columns){
		this.columns = columns;
		
		if(this.hasHeader){
			if(!this.headerRow){
				var thead = dojo.create("thead"); //$NON-NLS-1$
				this.headerRow = dojo.create("tr", { //$NON-NLS-1$
					className: "headerRow" //$NON-NLS-1$
				}, thead);
				dojo.place(thead, this.domNode, "first"); //$NON-NLS-1$
			}
			var headerRow = this.headerRow;
			dojo.forEach(columns, function(column){
				var prop = {};
				if(column.name){
					prop.innerHTML = escapeHtml(column.name);
					prop.title = column.name;
				}
				if(column.id){
					prop.id = column.id;
				}
				if(column.className){
					prop.className = column.className;
				}
				if(column.style){
					prop.style = column.style;
				}
				if(column.isAction){
					if(prop.className){
						prop.className += " " + CLASS_ACTION_HEADER_CELL; //$NON-NLS-1$
					}else{
						prop.className = CLASS_ACTION_HEADER_CELL;
					}
				}
				prop.scope='col'; //$NON-NLS-1$
				dojo.create("th", prop, headerRow); //$NON-NLS-1$
			});
		}
	},
	
	addItem: function(data, options, callback, scope){
		var item = new com.ibm.relm.explorer.web.ui.internal.view.common.Row({
			columns: this.columns || [],
			data: data,
			options: options,
			callback: (callback ? dojo.hitch(scope, callback) : null)
		});
		this.addChild(item);

		if(this.selectable === true){
			var self = this;
			this.connect(item, "onClick", function(){ //$NON-NLS-1$
				self.selectItem(item);
			});
			// ondijitclick somehow absorbes events inside cells (hover actions, inline edit box), so use onkeydown instead
			this.connect(item.domNode, "onkeydown", function(e){ //$NON-NLS-1$
				if(e.keyCode == dojo.keys.ENTER){
					self.selectItem(item);
				}
			});
		}
		
		return item;
	},
	
	getSelectedItem: function(){
		return this._selected;
	},

	selectItem: function(item){
		if(this._selected){
			this._selected.setSelected(false);
		}
		this._selected = item;
		if(item){
			item.setSelected(true);
		}
		this.onSelected(item);
	},

	selectItemByIndex: function(index) {
		var TRs = this.getChildren();
		if(TRs && (0 <= index) && (index < TRs.length) ) {
			this.selectItem(TRs[index]);
		}
	},
	
	selectItemById: function(id) {
		var TRs = this.getChildren();
		for(var i = 0; i < TRs.length; i++) {
			if(TRs[i].data.id == id) {
				this.selectItem(TRs[i]);
				break;
			}
		}
	},
	
	onSelected: function(selected){},
	
	clearItems: function(){
		this._selected = null;

		dojo.forEach(this.getChildren(), dojo.hitch(this, function(child){
			this.removeChild(child);
			child.destroy();
		}));
	},
	
	destroy: function(preserveDom){
		this.clearItems();
		this.inherited(arguments);
	}
});

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.common.Row", //$NON-NLS-1$
	[dijit._Widget, dijit._Contained],
{
	columns: null, // an array of objects : [ { id:"col1", name:"Column 1" }, { id:"col2", name:"Column 2" },...]
	data: null, // an object containing cell data : { col1:"Data 1", col2:"Data 2",...  }
	options: null, // an object containing select options : {col1:[{label: "Label1", value:"Vaue1"}, {label:"Label2", value:"Value2"}] , col2:[]
	_cells: null,
	callback: null,
	
	buildRendering: function(){
		this.domNode = dojo.create("tr", { //$NON-NLS-1$
			tabIndex: 0,
			className: CLASS_TABLE_ROW
		});
		
		// descendant widget won't get destroyed by destroyRecursive without containerNode
		this.containerNode = this.domNode;
		this.focusNode = this.domNode;
	},
	
	postCreate: function(){
		this._cells = [];
		this.inherited(arguments);
		
		// set hover CSS class
		var domNode = this.domNode;
		this.connect(domNode, "onmouseover", function(){ //$NON-NLS-1$
			dojo.toggleClass(domNode, CLASS_ROW_HOVER, true);
		});
		this.connect(domNode, "onmouseout", function(){ //$NON-NLS-1$
			dojo.toggleClass(domNode, CLASS_ROW_HOVER, false);
		});
		
		// populate cells
		dojo.forEach(this.columns, function(column){
			var cellData = this.data[column.id];
			if(column.id == "name" && this.data["showNameWith"] ) { //$NON-NLS-1$ //$NON-NLS-2$
				cellData = this.data[this.data["showNameWith"]]; //$NON-NLS-1$
			}
			var td;
			var className = column.className;
			if(column.getCell){
				td = dojo.hitch(this, column.getCell)(cellData);
			}else{
				td = this.getCell(cellData,column.id);
			}
			if(td){
				if(column.isAction === true){
					if(!className){
						className = CLASS_ACTION_CELL;
					}else{
						className += " "+CLASS_ACTION_CELL; //$NON-NLS-1$
					}
					this.setActions(cellData, this.data.editButton, td);
				}else{
					var cellContent;
					if(this.data.readonly){
						cellContent = this.getCellContent(cellData);						
					}else if (column.id == "value" && this.data.readonlyParams){ //$NON-NLS-1$
						cellContent = this.getParametersAsTable(cellData);
					}else if(this.options && this.options[column.id]){
						var callback = this.callback;
						var rowWidget = this;
						var editBox = tableUtils._createInlineEditBox(cellData, function(newValue) {
							// call user's call back 
							if (callback) {
								callback(rowWidget.data, column.id, newValue, rowWidget);
							}
						}, column.preventEmpty ? rowWidget.data[column.id]: null, this.options[column.id], this);
						cellContent = editBox;
						if(column.width){
							cellContent.style = {width: column.width};
						}

					}else if(column.getCellContent){
						cellContent = dojo.hitch(this, column.getCellContent)(cellData);
					}else{
						cellContent = this.getCellContent(cellData);
					}
					if(cellContent){
						if(this.data.editButton && this.data.buttonTarget == column.id){
							//create Table
							var table = dojo.create("table", {style: {tableLayout: "fixed", width: "100%"}, cellSpacing: "0"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
							var cellStyle = {
									padding: "0px 2px 0px 0px", //$NON-NLS-1$
									border: "none" //$NON-NLS-1$
								};
							var buttonCellStyle = dojo.mixin({width: "16px"}, cellStyle); //$NON-NLS-1$
							var tr = dojo.create("tr", {}, table); //$NON-NLS-1$
							var tdValue = dojo.create("td", {className: "innerTableCell", style: cellStyle}, tr); //$NON-NLS-1$ //$NON-NLS-2$
							tdValue.appendChild(cellContent.domNode || cellContent);
							var tdButton = dojo.create("td", {className: "innerTableCell", style: buttonCellStyle}, tr); //$NON-NLS-1$ //$NON-NLS-2$

							this.data.editButton.targetColumn = cellContent;
							if(this.data.editButton.editInPlace) {
								if(this.data.editButton.domNode) {
									tdButton.appendChild(this.data.editButton.domNode);
								} else {
									tdButton.appendChild(this.data.editButton);
								}
							}
							td.appendChild(table);
						}else{
							td.appendChild(cellContent.domNode || cellContent);
							td.title = cellData;
						}
						if(cellContent.startup){
							cellContent.startup();
						}
					}
				}
				if(className){
					dojo.addClass(td, className);
				}
				this.domNode.appendChild(td);
				this._cells.push(td);
			}
		}, this);
	},
	
	onFocus: function(){
		dojo.addClass(this.domNode, CLASS_ROW_HOVER);
	},

	onBlur: function(){
		dojo.removeClass(this.domNode, CLASS_ROW_HOVER);
	},
	
	getCell: function(cellData,labelledby){
		// return null for not adding null
		return dojo.create("td",{'aria-labelledby': labelledby}); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	getCellContent: function(cellData){
		if(cellData && cellData.length>0){
			return dojo.doc.createTextNode(unescapeHtml(cellData));
		}
		return null;
	},
	
	getParametersAsTable: function(cellData){
		if(cellData && cellData.length>0){
			var table = dojo.create("table", {style: {tableLayout: "fixed", width: "100%"}}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			var params = dojo.fromJson(cellData);
			for(name in params){
				var tr = dojo.create("tr", {}, table); //$NON-NLS-1$
				var tdName = dojo.create("td", {className: "innerTableCell", style: {width: "40%"}}, tr); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				dojo.create("span", { //$NON-NLS-1$
					innerHTML: escapeHtml(name),
					title: name
				}, tdName);
				var tdValue = dojo.create("td", {className: "innerTableCell", style: {width: "60%"}}, tr); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				dojo.create("span", { //$NON-NLS-1$
					innerHTML: escapeHtml(params[name].value),
					title: params[name].value
				}, tdValue);
			}
			return table;
		}
		return null;
	},
	
	getCellByColumnId: function(colId){
		for(var i=0, len=this.columns.length; i<len; i++){
			if(this.columns[i].id == colId){
				return this._cells[i];
			}
		}
		return null;
	},

	setSelected: function(selected){
		dojo.toggleClass(this.domNode, CLASS_SELECTED, selected);
	},
	
	setActions: function(actionArray, editButton, container){
		if(!actionArray && !editButton){
			container.appendChild(dojo.doc.createTextNode(messages.NA));
		}else{
			if(editButton && !editButton.editInPlace ){
				if(editButton.domNode) {
					editButton = editButton.domNode;
				}
				container.appendChild(editButton);
			}
			dojo.forEach(actionArray, function(action){
				container.appendChild(this.createActionIcon({
					iconClass: action.iconClass,
					alwaysVisible: action.alwaysVisible,
					onClick: dojo.hitch(this, action.onClick),
					title: action.title,
					owner: this
				}));
			}, this);
		}
	},
	
	createActionIcon: function(/*Object*/ arg){
		// summary:
		//		creates action button which:
		//		- Changes background on mouse hover
		//		- Invokes an action when clicked
		// arg:
		//		an object with 'iconClass' or 'iconUrl', 'onClick', 'title' properties
		//			/*String*/ iconClass : CSS class name string for icon
		//			/*String*/ iconUrl : URL for icon image
		//			/*Function*/ onClick : function that gets called when this icon is clicked
		//			/*String*/ title : tooltip text
		//			/*boolean*/ alwaysVisible : show icon even the row is not selected
		//			/*_Widget*/ owner : pass owner widget so that 'onclick' handler gets disconnected when that widget gets destroyed
		var className = (arg.alwaysVisible) ? "dijitInline alwaysVisibleIcon" : "dijitInline actionIcon"; //$NON-NLS-1$ //$NON-NLS-2$
		var anchor = dojo.create("a", { //$NON-NLS-1$
			href: "#", //$NON-NLS-1$
			title: arg.title,
			className: className
		});
		arg.owner.connect(anchor, "onclick", function(e){ //$NON-NLS-1$
			dojo.stopEvent(e);
			arg.onClick(anchor);
		});
		if(arg.iconClass){
			dojo.create("span", { //$NON-NLS-1$
				className: "dijitInline "+arg.iconClass //$NON-NLS-1$
			}, anchor);
		}else{
			dojo.create("img", { //$NON-NLS-1$
				src: arg.iconUrl
			}, anchor);
		}
		return anchor;
	},

	destroy: function(preserveDom){
		this._cells = null;
		this.destroyDescendants(preserveDom);
		this.inherited(arguments);
	}
});

})();