/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$

(function(){
		
dojo.require("dijit.InlineEditBox"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Select"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.util"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var escapeHtml = com.ibm.relm.common.web.ui.util.escapeHtml;
var messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "ConfigFormEditorMessages"); //$NON-NLS-1$  //$NON-NLS-2$
	
com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils = {};

var tableUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
dojo.mixin(tableUtils, 
{
	/*
	 * Convenience method to create a table column description which contains name, value, description, type, mandatory, actions columns
	 */
	createParamTableColumnsEx: function(nameChangeHandler, valueChangeHandler, disableNameEdit, minimumWidth, scope){
		var columns = [
           {id: "name", name: messages.configEditorParameterName, editable: !disableNameEdit, callback: nameChangeHandler, preventEmpty: true, width: !minimumWidth ? "200px": undefined}, //$NON-NLS-1$ //$NON-NLS-2$
           {id: "value", name: messages.configEditorParameterValue, editable: true, callback: valueChangeHandler, preventEmpty: false, width: !minimumWidth ? "200px": undefined} //$NON-NLS-1$ //$NON-NLS-2$
		];
		if(!minimumWidth){
			columns.push(
	           {id: "type",  //$NON-NLS-1$
	        	   name: messages.configEditorParameterType, 
	        	   editable: true, 
	        	   options: [
	        	             {label: messages.configEditorParameterType_Normal, value: ""}, //$NON-NLS-1$ 
	        	             {label: messages.configEditorParameterType_Product, value: "product"}, //$NON-NLS-1$ 
	        	             {label: messages.configEditorParameterType_Artifact, value: "artifact"} //$NON-NLS-1$
	        	             ],
	        	  callback: valueChangeHandler, 
	        	  preventEmpty: false, 
	        	  width: !minimumWidth ? "150px": undefined}); //$NON-NLS-1$
			columns.push(
	           {id: "mandatory",  //$NON-NLS-1$
	        	   name: messages.configEditorParameterMandatory, 
	        	   editable: true, 
	        	   options: [
	        	             {label: messages.configEditorParameterMandatory_True, value: true},  
	        	             {label: messages.configEditorParameterMandatory_False, value: false} 
	        	             ],
	        	  callback: valueChangeHandler, 
	        	  preventEmpty: true, 
	        	  width: !minimumWidth ? "100px": undefined}); //$NON-NLS-1$
			columns.push({id: "description", name: messages.configEditorParameterDescription, editable: !disableNameEdit, callback: valueChangeHandler, preventEmpty: false});//$NON-NLS-1$
		}
		columns.push({id: "actions", name: messages.configEditorAction, isAction: true}); //$NON-NLS-1$
		return tableUtils.createTableColumns(columns, scope || this);
	},
	
	/*
	 * Convenience method to create a table column description which contains name, value, description, actions columns
	 */
	createNameValueTableColumns: function(nameChangeHandler, valueChangeHandler, disableNameEdit, valueWidth, noDescription, noAction, scope){
		var columns = [
           {id: "name", name: messages.configEditorParameterName, editable: !disableNameEdit, callback: nameChangeHandler, preventEmpty: true, width: "200px"}, //$NON-NLS-1$ //$NON-NLS-2$
           {id: "value", name: messages.configEditorParameterValue, editable: true, callback: valueChangeHandler, preventEmpty: true, width: valueWidth || "200px"} //$NON-NLS-1$ //$NON-NLS-2$
		];
		if(!noDescription){
			columns.push({id: "description", name: messages.configEditorParameterDescription, editable: !disableNameEdit, callback: valueChangeHandler, preventEmpty: false});//$NON-NLS-1$
		}
		if(!noAction){
			columns.push({id: "actions", name: messages.configEditorAction, isAction: true}); //$NON-NLS-1$
		}
		return tableUtils.createTableColumns(columns, scope || this);
	},
	
	/*
	 * columnDefArray: [ {id, name, editable, callback, preventEmpty, options, width, isAction}, ... ]
	 */
	createTableColumns: function(columnDefArray, scope){
		scope = scope || this;
		return dojo.map(columnDefArray, function(columnDef) { 
			return tableUtils.createTableColumnDescription(columnDef, scope); 
		}, scope);
	},
	
	createTableColumnDescription: function(columnDef, scope){
		var col = {
			id: columnDef.id,
			name: columnDef.name,
			isAction: columnDef.isAction
		};
		if(columnDef.editable){
			scope = scope || this;
			var self = tableUtils;
			var callback = dojo.hitch(scope, columnDef.callback);
			col.getCellContent = function(cellData) {
				var rowWidget = this;
				var cb = this.callback || callback;
      		   	var editBox = self._createInlineEditBox(cellData, function(newValue) {
      		   		// call user's call back 
      		   		cb(rowWidget.data, col.id, newValue, rowWidget);
      		   	}, columnDef.preventEmpty ? rowWidget.data[col.id]: null, columnDef.options, scope);
      		   	return editBox;
			};
		}
		if(columnDef.width){
			col.style = {width: columnDef.width};
		}
		return col;
	},
	
	_getOption: function(options, value) {
		var option = null;
		dojo.some(options, function(opt) {
			if (typeof opt.value == "boolean") { //$NON-NLS-1$
				if (opt.value == (value == true.toString())) {
					option = opt;
					return true;
				}
			} else if (opt.value == value) {
				option = opt;
				return true;
			}
		});
		return option;
	},
	
	_createInlineEditBox: function(cellData, onChangeCallback, valueForEmpty, options, scope){
		var title = cellData;
		if(options){
			if(dojo.isFunction(options)){
				options = options();
			}else{
				// need copy to set selected 
				options = dojo.clone(options); 
			}
			
			var option = this._getOption(options, cellData);
			if (option) {
				option.selected = true;
				title = option.label;
			}
		}
		
		onChangeCallback = dojo.hitch(scope || this, onChangeCallback);
		
		var self = this;
		var noValueString = (valueForEmpty) ? "<span class='noValueIndicator'>" + messages.noValueIndicator + "</span>" : "_"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		var editBox = new dijit.InlineEditBox({
			editor: options ? "com.ibm.relm.explorer.web.ui.internal.view.common.Select" : "dijit.form.TextBox",//$NON-NLS-1$ //$NON-NLS-2$
			editorParams: {
				optionsArray : options
			},
			title : title,
			noValueIndicator: noValueString,
			onChange : function(value){
				if (!value || value.length === 0) {
					if(valueForEmpty){
						alert(messages.configEditorEmptyValue);
						this.set("value", valueForEmpty); //$NON-NLS-1$ 
						return;
					}else{
						value = "";//$NON-NLS-1$
					}
				}

				// replace value into label. 
				// com.ibm....view.common.Select widget shows value instead of label unexpectedly once option is selected.
				// Setting displayedValue in Select widget can resolve this label problem, but if you do so, InlineEditBox cannot distinguish
				// labels which have the same name. As a result, onChange event won't come up if you change option to one which has the same name as current one. 
				if (options) {
					var option = self._getOption(options, value);
					if (option) {
						this.domNode.innerHTML = escapeHtml(option.label);
					}
				}
				onChangeCallback(value);
			}
		}, dojo.create("div", { //$NON-NLS-1$
			innerHTML : escapeHtml(title)
		}));
		return editBox;
	}
});

})();