/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007,
 * 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.AttributeContent"); //$NON-NLS-1$

dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.layout.ContentPane"); //$NON-NLS-1$
dojo.require("dijit.form.Select");  //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.Table"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.ParamsEditDialog"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.AttributeUtils"); //$NON-NLS-1$

dojo.requireLocalization(
		"com.ibm.relm.explorer.web.ui.internal", "PropertiesPaneMessages"); //$NON-NLS-1$ //$NON-NLS-2$
// message mixin
dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "ReportMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() { // module container
	var tUtils = com.ibm.relm.explorer.web.ui.internal.view.common.TableUtils;
	var attrUtils = com.ibm.relm.explorer.web.ui.internal.view.common.AttributeUtils;
	
	/**
	 * Widget to show property pane
	 */
	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.AttributeContent", [ dijit._Widget, dijit._Templated ], { //$NON-NLS-1$
		templatePath : dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/edit/templates/AttributeContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		
		widget: null,

		constructor : function(args){
			this.messages = dojo.i18n.getLocalization(
					"com.ibm.relm.explorer.web.ui.internal", "PropertiesPaneMessages"); //$NON-NLS-1$ //$NON-NLS-2$
			var repoMessages = dojo.i18n.getLocalization(
					"com.ibm.relm.explorer.web.ui.internal", "ReportMessages"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.mixin(this.messages, repoMessages);			
		},
		
		postCreate : function() {
			this.inherited(arguments);
			
			this._initialize();
	
			this.attributeTable.setupColumns(tUtils.createTableColumns(
				[{
					id: "name", //$NON-NLS-1$
					name: this.messages.attrName,
					editable: false, 
					preventEmpty: true,
					width: "40%" //$NON-NLS-1$
				},
				{
					id: "value", //$NON-NLS-1$
					name: this.messages.attrValue,
					editable: true, 
					callback: dojo.hitch(this, this.attrValueChanged),
					preventEmpty: false,
					width: "60%" //$NON-NLS-1$
				},
				{
					id: "actions", //$NON-NLS-1$
					isAction: true,
					width: "16px" //$NON-NLS-1$
				}]
			));

			this.subscribe("btp:PropertyChanged", "propertyChanged");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_initialize : function() {
		},
		
		setWidget : function(widget) {
			this.widget = widget;
			var attributes = null;
			if(widget && widget.hlp_getAttributes){
				attributes = widget.hlp_getAttributes();
			}
			// attribute
			this.setAttributes(attributes);
		},
		
		setAttributes : function(attributes) {
			var propTable = this.attributeTable;
			propTable.clearItems();
			if(!attributes)
				return;
			var self = this;
			attributes = self.reorderAttributes(attributes);

			for(var i = 0; i < attributes.length; i++) {
				var attr = attributes[i];
				if(!attr) continue;
				var name = attr["name"]; //$NON-NLS-1$
				var value =  self.widget.hlp_getPropertyValue(name, true) || ""; //$NON-NLS-1$
				if(name == "queryParams" && (value == "" || value == null)) { //$NON-NLS-1$ //$NON-NLS-2$
					// skip to show queryParams without value
					continue;
				}
				if((name == "baseQueryId" ) && !self.widget._viewer.config.getQueryTemplate(value)) { //$NON-NLS-1$
					value = ""; //$NON-NLS-1$
				}
				if(!dojo.isString(value)) value = dojo.toJson(value);
				if(name == "font") { //$NON-NLS-1$
					this.addFontItems(name, value); 
				} else if(name != "layout") { // tentative except layout attribute //$NON-NLS-1$
					
					var button =  self._getEditButton(name, value);
					
					propTable.addItem({name : name, 
									label: attrUtils.getAttributeLabel(name),
									showNameWith: "label", //$NON-NLS-1$
									value: value,
									editButton: button,
									readonlyParams: (name == "parameters"), //$NON-NLS-1$
									actions:[]
									},
									{value: self.getOptions(name) },
									self.attrValueChanged,
									self);
				}
			}
		},
		
		showUnorderedAttributes: true, // in default, show additional attributes even attributes are ordered 
		
		reorderAttributes: function(attributes){
			if(!dojo.isFunction(this.getAttributeOrder)){
				return attributes;
			}
			var order = this.getAttributeOrder();
			if(!order) {
				return attributes;
			}
			var attrRet = [];
			var attrOpt = [];
			for(var i = 0; i < attributes.length; i++) {
				var index = dojo.indexOf(order, attributes[i].name);
				if(index < 0) {
					if(this.showUnorderedAttributes)
						attrOpt.push(attributes[i]);
				} else {
					attrRet[index] = {name: attributes[i].name, value: attributes[i].value};
				}
			}
			return attrRet.concat(attrOpt);
		},
		
		addFontItems: function(fontName, fontValue) {
			var self = this;
			var propTable = this.attributeTable;
			var fontProps = dojo.fromJson(fontValue);
			
			for(var name in fontProps) {
				var value = fontProps[name];
				
				var button =  self._getEditButton(name, value);
				propTable.addItem({name : name, 
					label: attrUtils.getAttributeLabel(name),
					showNameWith: "label", //$NON-NLS-1$
					value: value,
					editButton: button,
					actions:[]
					},
					{value: self.getOptions(name) },
					self.attrFontValueChanged,
					self);
			}
			
		},
		
		propertyChanged: function(message){
			var hint = message.hint || {};
			if(message.src == this.widget || hint.widget == this.widget){
				this.setWidget(this.widget);
			}
		},
		
		_getEditButton: function(name, value) {
			return null;
		},
		
		getOptions : function(attrName) {
			return null; 
		},
		
		attrValueChanged : function(rowData, attrName, newValue, dirtyKey) {
			var widget = this.widget;
			if(!widget){
				// widget is not selected
				return;
			}
			var oldValue = widget.hlp_getPropertyValue(rowData.name, true);
			if(rowData.name == "parameters") { //$NON-NLS-1$
				if(dojo.isString(newValue)){
					try {
						newValue = dojo.fromJson(newValue);
					}
					catch (e) {
						return;
					}
				}
			}else if(rowData.name == "href") { //$NON-NLS-1$
				// sanitize link
				newValue = com.ibm.relm.explorer.web.ui.internal.util.sanitizeUrl(newValue);
			}
			if( oldValue != newValue) {
				var propName = rowData.name;
				var noReload = false;
				if(propName=="id" && newValue !="") { //$NON-NLS-1$ //$NON-NLS-2$
					// container id
					var allContainers = this.diagramView._getDiagramEditor().getContainerSet();
					var isUnique = allContainers.every(function(container){
						return newValue != container.hlp_getPropertyValue("id"); //$NON-NLS-1$
					});
					if(!isUnique) {
						alert(dojo.string.substitute(this.messages.duplicatedId, [newValue]));
						newValue = oldValue || ""; //$NON-NLS-1$
					};
				}
				if(propName=="showGrid" || propName=="pack" || propName=="allowOverwrap") { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					newValue = newValue == "true"; // boolean //$NON-NLS-1$
				}
				if(propName=="fill" || //$NON-NLS-1$
						propName=="stroke" || //$NON-NLS-1$
						propName=="left" || //$NON-NLS-1$
						propName=="top" || //$NON-NLS-1$ 
						propName=="r"){ //$NON-NLS-1$ 
					noReload = true;
				}
				if(propName=="width" || propName=="height") { //$NON-NLS-1$ //$NON-NLS-2$
					if(widget.hlp_getPropertyValue("pack")) { //$NON-NLS-1$
						alert(this.messages.packInfoMessage);
					}
				}
				var prop = {};
				prop[rowData.name] = newValue;
				this._updateWidget(prop, noReload);
			}
		},
		
		attrFontValueChanged : function(rowData, attrName, newValue, dirtyKey) {
			var widget = this.widget;
			if(!widget){
				// widget is not selected
				return;
			}
			var json = widget.hlp_getPropertyValue("font"); //$NON-NLS-1$
			var fontDef = dojo.fromJson(json);
			if( fontDef[rowData.name] != newValue) {
				fontDef[rowData.name] = newValue;
				var prop = {"font": dojo.toJson(fontDef)}; //$NON-NLS-1$
				this._updateWidget(prop);
			}
		},
		
		_updateWidget: function(prop, noReload){
			var widget = this.widget;
			if(!widget){
				// widget is not selected
				return;
			}
			var cc = new com.ibm.relm.explorer.web.base.internal.canvas.command.CompoundCommand();
			var ec = com.ibm.relm.explorer.web.base.internal.canvas.command.EditCommand;
			for(var name in prop){
				var propName = name;
				var propValue = prop[name];
				cc.append(new ec.SetPropertyValueCommand(widget, propName, propValue));
			}
			if(!noReload){
				cc.setLastCommand(new com.ibm.relm.explorer.web.ui.internal.view.edit.EditCommand.UpdateCommand(widget));
			}
			var editor = widget.hlp_getEditor();
			editor.executeCommand(cc);
		},
		
		_findAttribute: function(attrs, name){
			if(attrs){
				for(var i = attrs.length - 1; i >= 0; i--){
					var attr = attrs[i];
					if(attr.name === name) return attr;
				}
			}
			return null;
		}
	});
	
})();