/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.extensions.xmlGenerator"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$

(function(){
	
	var util = com.ibm.relm.explorer.web.ui.internal.util;

var getIndentSpace = function(indent){
	var indentStr = "";//$NON-NLS-1$
	for(var i=0; i<indent; i++){
		indentStr += "   ";//$NON-NLS-1$
	}
	return indentStr;
};

var generateWidget = function(widget, indent){
	var ml = ""; //$NON-NLS-1$
	
	if(widget.preventSave) return ml;
	
	var widgetName = null;
	var params = null;
	if(dojo.isFunction(widget.getOutputTagName)){
	
		widgetName = widget.getOutputTagName();
		if(widgetName){
			ml = getIndentSpace(indent) + '<' + widgetName; //$NON-NLS-1$
			var componentInfo = widget.hlp_getComponentInfo();
			
			var attrs = widget.hlp_getAttributes() || [];
			for(i = 0, len = attrs.length; i < len; i++){
				var attr = attrs[i];
				if(attr.readonly === true){ continue; }
				var name = attr.name;
				var value = widget.hlp_getPropertyValue(name, true);
				if(value && value.length > 0) {
					value = util.escapeHtml(value);
				}
				
				var writeValue = false;
				if(widgetName == "container" && name == "parameters") { //$NON-NLS-1$ //$NON-NLS-2$
					if(value) {
						params = value;
					}
				} else if (componentInfo && typeof value == "boolean") {//$NON-NLS-1$
					// In case specified value is boolean type and is not default value, write it.
					dojo.some(componentInfo.attributes, function(attribute) {
						if (attribute.name == name) {
							writeValue = attribute.value === undefined ? value : attribute.value != value;
							return true;
						}
					});
				} else if(value){
					writeValue = true;
				}
				
				if (writeValue) {
					ml +=  " " + name + "='" + value + "'"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}
			}
			ml += ">\n";//$NON-NLS-1$
		}
	}
	
	if(params) {
		// param tags under <container> tag
		for(var name in params) {
			var value = params[name].value;
			var description = params[name].description;
			if(!value) value = ""; //$NON-NLS-1$
			if(!description) description = ""; //$NON-NLS-1$
			ml += getIndentSpace(indent + 1) + "<param name='" + name + "' value='" + value + "' description='" + description + "'/>\n"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
	}
	
	var children = widget.getChildren ? widget.getChildren() : null;
	if(children){
		for(var i = 0, len = children.length; i < len; i++){
			ml += generateWidget(children[i], indent + 1);
		}
	}
	
	if(widgetName){
		ml += getIndentSpace(indent) + '</' + widgetName + '>\n'; //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	return ml;
};


com.ibm.relm.explorer.web.ui.internal.view.extensions.xmlGenerator = {
	generate: function(widget){
		widget = widget || this;
		var ml = generateWidget(widget, 0);
		return "<layout xmlns='http://relm.ibm.com/view/0.9/'>\n" + ml + "</layout>";  //$NON-NLS-1$ //$NON-NLS-2$ 
	},
	
	_findRootContainer: function(widget){
		if(widget.componentInfoId === "container") return widget;//$NON-NLS-1$
		var children = widget.getChildren ? widget.getChildren() : null;
		var container = null;
		if(children){
			for(var i = 0, len = children.length; i < len && !container; i++){
				container = this._findRootContainer(children[i]);
			}			
		}
		return container;
	}	
};

})();