/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.loader.DefaultLoader"); //$NON-NLS-1$

dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("jazz.app.context"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.loader.AbstractLoader"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.loader.Config"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.layout.AbstractLayout"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.layout.DefaultLayout"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.layout.GridLayout"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.layout.TreeLayout"); //$NON-NLS-1$
// pre-load default components
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.ResourceNode"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.Image"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.Text"); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.ui.internal.util'); //$NON-NLS-1$


(function(){
	
dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.loader.DefaultLoader", //$NON-NLS-1$
	com.ibm.relm.explorer.web.ui.internal.view.loader.AbstractLoader, 
{
	loadDiagram: function(configSource){
		var viewer = this.viewer;
		
		//
		var params = {};
		var hash = dojo.doc.location.hash;
		if(hash && hash.length > 1){
			params = dojo.queryToObject(hash.substring(1));
			// remove Jazz UI params
			delete params.action; 
			delete params.id; 
			// remove RELM own params
			delete params.viewUri; 
			delete params.viewName; 
		}
		// covert { paramName: paramValue, ...} to { paramName: {value: paramValue}, ...} 
		for(var p in params){
			params[p] = {value: params[p]};
		}

		// load configuration
		var config = new com.ibm.relm.explorer.web.ui.internal.view.loader.Config();
		config.viewUri = this.viewUri;
		config.load(configSource, params);

		// unescape diagram name and description
		config.diagramName = com.ibm.relm.explorer.web.ui.internal.util.unescapeHtml(config.diagramName);
		config.description = com.ibm.relm.explorer.web.ui.internal.util.unescapeHtml(config.description);

		// set the configuration to the viewer
		viewer.config = config;
		viewer.registerComponentInfo(config.componentInfo);

		this.reloadLayout(configSource);
	},
	
	reloadLayout: function(configSource){
		var viewer = this.viewer;
		var props = {};
		// find <layout>
		var layout = dojox.xml.parser.parse(configSource.layout).documentElement;
		var hasAttribute = false;
		if(layout){
			dojo.forEach(layout.attributes, function(a){
				// ignore if the property (or method) already exists
				if(a.name != "layout" && a.name != "xmlns"){  //$NON-NLS-1$ //$NON-NLS-2$
					props[a.name] = a.value;
					hasAttribute = true;
				}
			});			
		}
		dojo.mixin(props, {
			_viewer: viewer,
			_domNode: layout
		});
		
		// create initial container 
		// if attribute exists in <layout> (1.0 style), it should be saved in the first <container> 
		var container = hasAttribute 
			? viewer.createComponent("container", props) //$NON-NLS-1$
			: new com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeContainer(dojo.mixin(props, {width: 0})); 
		// 
		viewer.removeFocus(); // cannot drag & drop after refresh without this!!
		viewer.removeHalfFocus(); // same as above
		viewer.destroyDescendants();
		
		viewer.addChild(container);

		container.setup();
		container.layout();

		// create widget tree
		viewer.setupWidgetsRecursively();
		dojo.forEach(viewer.getChildren(), function(child){
			if(dojo.isFunction(child.startup)){
				child.startup();
			}
		});
		if(viewer.adjustCanvasSize){
			setTimeout(function(){
				viewer.adjustCanvasSize();
			}, 0);
		}
		viewer.fireInputChanged();		
	}	
}
);

})();