/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.Pinning'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui', 'PinningMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.rtc.StreamManagement'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$

(function() {

var messages = dojo.mixin(
	dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'PinningMessages'), //$NON-NLS-1$ //$NON-NLS-2$
	dojo.i18n.getLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages')); //$NON-NLS-1$ //$NON-NLS-2$

	var productapi = com.ibm.relm.ple.web.ui.productapi;
	var StreamManagement = com.ibm.relm.ple.web.rtc.StreamManagement;
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	
	function togglePinned(parentProduct, product, nowPinned, statusMessage) {
		var partUri = product['pd:part']; //$NON-NLS-1$

		var message = com.ibm.relm.common.web.StatusMessage.show(statusMessage);
		var deferred = new dojo.Deferred();

		jazz.client.xhrPost({
			handleAs: 'json', //$NON-NLS-1$
			url: net.jazz.ajax._contextRoot + '/ple/pin', //$NON-NLS-1$
			content: {
				parentVersionUri: parentProduct.uri,
				partUri: partUri,
				pinned: nowPinned
			},

			load: function (newItem) {
				com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh().addBoth(function() {
					dojo.publish('ple.detail_view.refresh', [newItem]); //$NON-NLS-1$
					deferred.callback();
				});
			},

			error: function(_, args) {
				deferred.errback(args.xhr);
			},

			handle: function() {
				message.hide();
			}
		});
		return deferred;
	}

	/**
	 * Unpins the current item version, which causes the latest version for the branch to
	 * bind in and the tree to refresh to show the new version.
	 * @param item store item for item version.
	 */
	function _doUnpin(parentProduct, product) {
		return togglePinned(parentProduct, product, false, messages.unpinning);
	}

	/**
	 * Shows a progress dialog while it attemps to to pin
	 * a product to its parent. Will close the dialog and show an error
	 * above the tree if there is one.
	 * 
	 * @param  {[type]} parentProduct [description]
	 * @param  {[type]} product       [description]
	 * @return {[type]}               [description]
	 */
	function pinProduct(parentProduct, product) {
		var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
			primaryTitle: messages.pinDialogTitle, 
			width: '400px', //$NON-NLS-1$
			explanation: '', //$NON-NLS-1$
			canCancel: false,
			mirrorStatusMessage: true,
			iconClass: 'ple-icon-pinned' //$NON-NLS-1$
		});
		progressDialog.startup();

		function onError(errorObject) {
			dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
			progressDialog.close();
		}

		togglePinned(parentProduct, product, true, messages.pinning)
		.then(function() {
			progressDialog.close();
		})
		.otherwise(function(xhr) {
			dojo.publish('ple.product_tree_error', [dojo.fromJson(xhr.responseText)]); //$NON-NLS-1$
			progressDialog.close();
		});
	}

	/**
	 * Confirms the unpin action with a dialog showing some help text
	 * @param  {Object} product - Required
	 * @param  {Array} productPath - Required
	 * 					An array of products with the last one in the array being the current product
	 * @param  {HTML String} - Optional
	 * 					This is an optional HTML string that can contain text in additional to the 
	 * 					standard explanation for unpinning. This is intended for custom actions that
	 * 					take place with unpinning (ex: update RTC component). Should be wrapped in an
	 * 					html tag to show some seperation and should seem like it is just string concatenation
	 * @param  {Function} callback - Optional
	 * 					The callback to carryout the actual unpinning. If nothing is passed in, then the 
	 * 					standard unpin operation occurs. Intended to be used for custom actions that take 
	 * 					place with unpinning (ex: update RTC component)
	 */
	function confirmUnpin(productPath, product, additionalInfo, callback) {
		var name = TreeSelection.store.getValue(product, 'name'); //$NON-NLS-1$

		var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
			dialogTitle: messages.unpinTitle,
			prompt: dojo.replace(messages.unpinPrompt, [com.ibm.relm.common.web.ui.util.escapeHtml(name)]),
			details: (additionalInfo) ? messages.unpinDetailMessage + additionalInfo : messages.unpinDetailMessage,
			iconPath: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/unpin_co.gif') //$NON-NLS-1$ //$NON-NLS-2$
		});

		d.addButton(messages.unpinAction, dojo.hitch(this, callback || _doUnpin, productPath, product), true);

		d.startup();
	}

 /**
  * Utility to contain all pinning related logic. Performs the standard pinning operation for products,
  * but is also aware of special actions that need to happen at pinning, such as RTC component updates.
  */
dojo.mixin(dojo.getObject('com.ibm.relm.ple.web.ui.Pinning'), { //$NON-NLS-1$

	unpinProduct: function(parentProduct, product) {
		confirmUnpin(parentProduct, product);
	},

	pinProduct: pinProduct
});

})();
