/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui', 'productapi_messages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.branch.FreezeBranch'); //$NON-NLS-1$

(function() {

var messages = dojo.mixin(
	dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'productapi_messages'), //$NON-NLS-1$ //$NON-NLS-2$
	dojo.i18n.getLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages')); //$NON-NLS-1$ //$NON-NLS-2$

var ICONS = {
	doorsBaseline: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/doorsBaseline.png'), //$NON-NLS-1$ //$NON-NLS-2$
	link: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/hyperlink_co.gif'), //$NON-NLS-1$ //$NON-NLS-2$
	productImmutable: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/productInSnapshot_co.gif'), //$NON-NLS-1$ //$NON-NLS-2$
	productMutable: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/product_co.gif'), //$NON-NLS-1$ //$NON-NLS-2$
	productFrozen: dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/product_stopped.gif'), //$NON-NLS-2$ //$NON-NLS-1$
	productStream: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/product_stream.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	productStreamCheckedIn: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/product_stream_checked_in.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	productComponent: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/product_component.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	productComponentCheckedIn: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/product_component_checked_in.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	productConfiguration: dojo.moduleUrl('com.ibm.relm.ple.web', "ui/images/configuration.gif"), //$NON-NLS-1$ //$NON-NLS-2$
	blankGif: dojo.moduleUrl('dojo','resources/blank.gif') //$NON-NLS-1$ //$NON-NLS-2$
};

var htmlEscape = jazz.util.html.escape;
var FROZEN_TIP = "ple:isFrozenTip"; //$NON-NLS-1$


/**
 * Functions to format labels for products, that is, icons and text decorators to show their
 * state, snapshot, and branch.
 */
dojo.mixin(dojo.getObject('com.ibm.relm.ple.web.ui.formatting'), { //$NON-NLS-1$


	/**
	 * Formats a creator's username as HTML. This mainly catches the case where there
	 * is no user name (empty string or undefined), which happens to products that are
	 * implicitly checked out to modify a child product.
	 * @param  {string} creatorUserName username of creator, or empty string or undefined if none.
	 * @return {string} HTML fragment for the username. Safe to insert with innerHTML (already escaped).
	 */
	formatCreator: function(creatorUserName) {
		if (creatorUserName) {
			return htmlEscape(creatorUserName);
		}
		else {
			return '<span class="ple-noCreator" title="' + htmlEscape(messages.noCreatorToooltip) + '">' + htmlEscape(messages.noCreatorName) + '</span>'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
	},

	/**
	 * Formats an indirect creator's username as HTML. This behaves similarly to formatCreator(), but
	 * formats the creator name differently because the creator didn't explicitly create the item.
	 * @param  {string} indirectCreatorUserName username of indirect creator, or empty string or undefined if none.
	 * @return {string} HTML fragment for the username. Safe to insert with innerHTML (already escaped).
	 */
	formatIndirectCreator: function(indirectCreatorUserName) {
		if (indirectCreatorUserName) {
			return '<span class="ple-noCreator" title="' + htmlEscape(messages.indirectCreatorTooltip) + '">' + htmlEscape(indirectCreatorUserName) + '</span>'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		else {
			return '<span class="ple-noCreator" title="' + htmlEscape(messages.noCreatorTooltip) + '">' + htmlEscape(messages.noCreatorName) + '</span>'; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
	},

	/**
	* Has conditional logic to return the icon's SRC for inclusion where needed.
	*/
	getIconSrcAndAltTextForItem: function(item) {
		if (!item || !item.type) {
			return ICONS.productImmutable; 
		}
		var type = item.type;

		//Cannot add class for disabled hierarchy here since the artifacts with icons don't come through this
		if (type === 'link') //$NON-NLS-1$
		{
			return [ICONS.link, messages.artifactAlt]; 
		}
		else
		{
			if (item.rtcStream) {
				if (item['pd:immutable']){ //$NON-NLS-1$
					return [ICONS.productStreamCheckedIn, messages.stream];
				} else {
					return [ICONS.productStream, messages.stream];
				}
			}

			if (item.rtcComponent) {
				if (item['pd:immutable']){ //$NON-NLS-1$
					return [ICONS.productComponentCheckedIn, messages.component];
				} else {
					return [ICONS.productComponent, messages.component];
				}
			}

			if (item.conditionVariables !== undefined) {
				return [ICONS.productConfiguration, messages.configuration];
			}

			var immutable = item["pd:immutable"]; //$NON-NLS-1$
			var frozenTip = item[FROZEN_TIP];
			if (frozenTip) {
				return [ICONS.productFrozen, messages.frozenProductAlt]; 
			}
			return immutable ?
				[ICONS.productImmutable, messages.productInSnapshotAlt] : 
				[ICONS.productMutable, messages.productAlt]; 
		}
	},

	//XXX: Logic copied from here to getIconSrcAndAltTextForItem
	getIconClassAndAltTextForItem: function(item){
		if (!item || !item.type) {
			return 'ple-icon-productInSnapshot'; //$NON-NLS-1$
		}
		var type = item.type;

		// Cannot add class for disabled hierarchy here since the artifacts with icons don't come through this
		if (type === 'link') //$NON-NLS-1$
		{
			return ['ple-icon-artifactLink', messages.artifactAlt]; //$NON-NLS-1$
		}
		else
		{
			if (item.rtcStream) {
				if (item['pd:immutable']){ //$NON-NLS-1$
					return ['ple-icon-productStreamCheckedIn', messages.stream]; //$NON-NLS-1$
				} else {
					return ['ple-icon-productStream', messages.stream]; //$NON-NLS-1$
				}
			}
			if (item.rtcComponent) {
				if (item['pd:immutable']){ //$NON-NLS-1$
					return ['ple-icon-productComponentCheckedIn', messages.component]; //$NON-NLS-1$
				} else {
				return ['ple-icon-productComponent', messages.component]; //$NON-NLS-1$
				}
			}

			if (item.conditionVariables !== undefined) {
				return ['ple-icon-rtcConfiguration', messages.configuration]; //$NON-NLS-1$
			}

			var immutable = item["pd:immutable"]; //$NON-NLS-1$
			var frozenTip = item[FROZEN_TIP];
			if (frozenTip) {
				return ['ple-icon-productInFrozenBranch', messages.frozenProductAlt]; //$NON-NLS-1$
			}
			return immutable ?
				['ple-icon-productInSnapshot', messages.productInSnapshotAlt] : //$NON-NLS-1$
				['ple-icon-product', messages.productAlt]; //$NON-NLS-1$
		}
	},

	getIconClassForItem: function(item){
		return this.getIconClassAndAltTextForItem(item)[0];
	},

	/**
	*	Has conditional logic to return an actual img if its a link or else a img tag with an icon class for spriting
	*/
	getIconImageForItem: function(item) {
		if (item.type === 'link') { //$NON-NLS-1$
			return dojo.create('img', { //$NON-NLS-1$
				'src': ICONS.link, //$NON-NLS-1$
				'class': 'ple-icon', //$NON-NLS-1$ //$NON-NLS-2$
				alt: messages.artifactAlt
			});
		} 
		var iconClassAndAltText = this.getIconClassAndAltTextForItem(item);

		return dojo.create('img', { //$NON-NLS-1$
			src: ICONS.blankGif,
			className: iconClassAndAltText[0], 
			alt: iconClassAndAltText[1]
		});
	},
	
	/**
	 * Gets an IMG tag for an Item, not a particular version.
	 */
	getProductItemIconImageForItem: function() {
		return dojo.create('img', { //$NON-NLS-1$
			src: ICONS.blankGif,
			className: 'ple-icon-product', //$NON-NLS-1$
			alt: messages.productAlt
		});
	},

	formatProductIconAndTitleForItem: function(item) {
		if (item.link) {
			// We'll prefer icons for links still since OSLC previews return icon src links
			// for new icons. Once those are applied we'd need to remove the ple-icon-artifactLink class,
			// but I haven't tracked down all the places to do that. jazz.ui.ResourceLink does not do that for us.
			var imgSrc = item.isDoorsBaseline ? ICONS.doorsBaseline : ICONS.link;
			return '<img class="ple-icon" alt="' + messages.artifactAlt + '" src="' + imgSrc + '">' + this.formatProductTitleForItem(item); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		else {
			var classAndAlt = this.getIconClassAndAltTextForItem(item);
			return '<img class="ple-icon ' + classAndAlt[0] + '" alt="' + classAndAlt[1] + '" src="' + ICONS.blankGif + '">' + this.formatProductTitleForItem(item); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
	},

	/**
	 * Get HTML string for an IMG of an Item, not a particular version.
	 * 
	 * @param  {object} item an item version to format as an item.
	 * @return {[type]}      [description]
	 */
	formatProductItemIconAndTitleForItem: function(item) {
		return '<img class="ple-icon ple-icon-product" alt="' +  messages.productAlt + '" src="' + ICONS.blankGif + '">' + this.formatProductItemTitleForItem(item); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},

	/**
	*	This is meant for use by the product audit log only. The icon it uses is always the default one for a mutable product. 
	*/
	formatDefaultProductIconAndTitle: function(type, name, branchName, snapshot, immutable) {
		var imgSrc = ICONS.productMutable;
		return '<img class="ple-icon" src="' + imgSrc + '" alt="' + messages.productAlt + '">' + this.formatProductTitle(name, branchName, snapshot); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},

	formatDimensionIconAndTitle: function(dimension) {
		if (dimension.shared) {
			return '<img class="ple-icon-dimensionShared ple-icon" src="' + ICONS.blankGif + '" title="' + 'Shared Dimension' + '"> ' + htmlEscape(dimension.name); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
		else {
			return '<img class="ple-icon-dimension ple-icon" src="' + ICONS.blankGif + '" title="' + 'Dimension' + '"> ' + htmlEscape(dimension.name); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			
		}
	},

	formatDecoratorForItem: function(item) {
		var args = {
			snapshot: htmlEscape(item._snapshot),
			branch: htmlEscape(item._branch_name)
		};

		if (item._snapshot && item._branch_name) {
			return dojo.replace(messages.branchXSnapshotX_html, args);	
		}
		else if (item._snapshot) {
			return dojo.replace(messages.snapshotX_html, args);		
		}
		else if (item._branch_name) {
			return dojo.replace(messages.branchX_html, args);
		}
		else {
			return ''; //$NON-NLS-1$
		}
	},
	
	formatSnapshotTitle: function(snapshot) {
		var args = {
				snapshot: snapshot._snapshot || '' //$NON-NLS-1$
		};
		return dojo.replace(messages.snapshotTitleX_html, args);
	},
	
	formatBranchTitle: function(branch) {
		var args = {
				branch: branch._branch_name
		};		
		if (!args.branch) {
			args.label = messages.noDimensionsLabel;
			return dojo.replace(messages.grayLabel_html, args);
		}
		else{
			return dojo.replace(messages.branchTitleX_html, args);
		}
	},

	_formatProductTitle: function(params) {
		var template;

		if (params.branch && params.snapshot) {
			template = messages.productTitleXBranchXSnapshotX_html;
		}
		else if (params.branch) {
			template = messages.productTitleXBranchX_html;
		}
		else if (params.snapshot) {
			template = messages.productTitleXSnapshotX_html;
		}
		else {
			template = messages.productTitleX_html;
		}

		if (!params.title) {
			// Strip leading snapshot dash if no title.
			template = template.replace(/^ - /, ''); //$NON-NLS-1$
		}

		var title = dojo.replace(template, function(wholeMatch, name) {
			return jazz.util.html.escape(params[name]);
		});

		if (params.forRtc) {
			return '<span class="ple-rtcConnected">' + title + '</span>'; //$NON-NLS-1$ //$NON-NLS-2$
		}
		else {
			return title;
		}

	},

	formatProductTitle: function(title, branch, snapshot, forRtc) {
		var params = {
			title: title,
			branch: branch,
			snapshot: snapshot,
			forRtc: forRtc
		};

		return this._formatProductTitle(params);
	},

	formatProductItemTitleForItem: function(item) {
		return this._formatProductTitle({
			title: item.name,
			forRtc: item.rtcStream || item.rtcComponent
		});
	},

	formatProductTitleForItem: function(item) {

		return this._formatProductTitle({
			title: item.name,
			branch: item._branch_name,
			snapshot: item._snapshot,
			forRtc: item.rtcStream || item.rtcComponent
		});
	},

	/**
	 * Returns a new function that can be used with array.sort to sort product
	 * items by their labels (including branch and version). This returns a
	 * new function every time since the function will cache the formatted
	 * displayname, which could take some time to compute for large lists of
	 * products.
	 * 
	 * @return {function} new compare function to sort products with.
	 */
	newProductItemCompareFunction: function() {
		var titleCache = {};
		
		var self = this; 
		
		function getTitle(product) {
			var uri = product.uri;
			var title = titleCache[uri];
			if (!title) {
				title = self.formatProductTitleForItem(product);
				titleCache[uri] = title;
			}
			return title;
		}

		return function sortProduct(left, right) {
			left = getTitle(left);
			right = getTitle(right);

			if (left < right) {
				return -1;
			}
			else if (left === right) {
				return 0;
			}
			else {
				return 1;
			}
		};
	}
});

})();
