/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.ui.pages.RemoveDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.DialogHelpHeader'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.pages", "RemoveDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$


(function(){
	var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'RemoveDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$
	var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;
	var contextRoot = net.jazz.ajax._contextRoot;

	dojo.declare(
		"com.ibm.relm.ple.web.ui.pages.RemoveDialog", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/pages/templates/RemoveDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			dialogTitle: messages.removeDialogTitle,
			prompt: "", //$NON-NLS-1$
			details: "", //$NON-NLS-1$
			detailsList: "", //$NON-NLS-1$
			width: '500px', //$NON-NLS-1$

			/**
			 * When removing child products and links, this must be their
			 * common parent product. If deleting top level products, can be null.
			 * @type {string}
			 */
			parentProductUri: null,

			/**
			 * When removing child products and links, this must be their
			 * common parent view. If deleting top level products, can be null.
			 * @type {string}
			 */
			parentViewUri: null,

			/**
			 * All the products to be deleted. Array of {child: <product URI>, part: <part URI>}.
			 * If deleting top level products the 'part' field can be null (in that case there aren't
			 * 'child' products really, but the same name is used). Can be empty.
			 * @type {array}
			 */
			partAndChildUris: [],

			/**
			 * All links to be removed. Can be empty.
			 * @type {Array}
			 */
			linkUris: [],

			/**
			 * If any of the products being deleted have children, must be true.
			 * @type {Boolean}
			 */
			hasChildren: false,

			/**
			 * If any products are being removed are immutable.
			 * @type {Boolean}
			 */
			immutable: false,

			constructor: function(args) {
				this.messages = messages;
				this.iconSrc = dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/question.gif'); //$NON-NLS-1$ //$NON-NLS-2$
				this._deleteCheckboxId = dijit.getUniqueId('ple-RemoveDialog-checkbox'); //$NON-NLS-1$
				this.parentProductUri = args.parentProductUri;
				this.name = args.name;
			},

			postMixInProperties: function() {
				this.inherited(arguments);

				if (!this.parentProductUri) {
					// No parent. Only top level products.
					this.prompt = this.partAndChildUris.length == 1 ?
						dojo.replace(messages.deleteXPrompt, [com.ibm.relm.common.web.ui.util.escapeHtml(this.name)]) :
						dojo.replace(messages.deleteXProductsPrompt, [this.partAndChildUris.length]);

					this.details = messages.deleteProductDetailHead + messages.deleteProductDetailsList;
					this.detailsList = messages.deleteProductDetailsList;
				}
				else if (this.linkUris.length && !this.partAndChildUris.length) {
					// Only links
					this.prompt = this.linkUris.length > 1 ?
						dojo.replace(messages.removeXLinksPrompt, [this.linkUris.length]) :
						dojo.replace(messages.removeXPrompt, [com.ibm.relm.common.web.ui.util.escapeHtml(this.name)]);

					this.details = messages.removeLinkDetails;
				}
				else if (!this.linkUris.length && this.partAndChildUris.length) {
					// Only child products
					this.prompt = this.partAndChildUris.length > 1 ?
						dojo.replace(messages.removeXProductsPrompt, [this.partAndChildUris.length]) :
						dojo.replace(messages.removeXPrompt, [com.ibm.relm.common.web.ui.util.escapeHtml(this.name)]);

					this.details = messages.removeProductDetails;
				}
				else {
					// Mix of links and child products
					this.prompt = dojo.replace(messages.removeXItems, [this.linkUris.length + this.partAndChildUris.length]);
					this.details = messages.removeItemsDetails;
				}
			},

			cancel: function() {
				this._dialog.close();
			},

			_enableDelete: function() {
				this.checkbox.disabled = false;
				dojo.removeClass(this.checkboxContainer, 'relm-RemoveDialog-checkbox-disabled'); //$NON-NLS-1$
				dojo.query('*[title]', this.checkboxContainer).forEach(function(node) { //$NON-NLS-1$
					dojo.attr(node, 'title', this.messages.deleteEnabledTooltip); //$NON-NLS-1$
				}, this);
			},

			postCreate: function() {
				this.inherited(arguments);

				if (this.linkUris.length) {
					this.checkboxContainer.style.display = 'none'; //$NON-NLS-1$
				}

				//If parentProductUri is undefined, then that means we have a
				//top level product and the dialog displayed will be different
				if (this.parentProductUri) {
					this.removeButton = this.addButton(
						this.messages.removeProductButton,
						dojo.hitch(this, this.onRemove),
						true);

					if (!this.hasChildren && !this.immutable) {
						this._enableDelete();
					}

					//TODO: do this check only if delete option is selected
					//ask troy if I should disable check box before hand
//					this.checkPrereqs(this.removeButton);
				}else{
					this.deleteButton = this.addButton(
						this.messages.deleteProductButton,
						dojo.hitch(this, this.onDelete),
						true);

					this.checkboxContainer.style.display = 'none'; //$NON-NLS-1$
					//This can be added once the DialogHelpHeader is working
//					this.checkPrereqs(this.deleteButton);
				}

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.dialogTitle,
					width: this.width,
					destroyContents: true,
					contentNode: this.domNode,
					closable: false
				});
			},

			_onClickDeleteCheckbox: function(e) {
				com.ibm.relm.common.web.ui.util.setText(this.removeButton, this.checkbox.checked ?
					this.messages.removeAndDeleteProductButton :
					this.messages.removeProductButton);
			},

			addButton: function(title, callback, isDefaultButton) {
				var button = dojo.create('button'); //$NON-NLS-1$
				button.appendChild(document.createTextNode(title));

				this.connect(button, 'onclick', callback); //$NON-NLS-1$

				if (isDefaultButton) {
					this._defaultButton = button;
				}

				dojo.place(button, this._customButtonsNode, 'last'); //$NON-NLS-1$

				//Don't know if this is safe memory leak wise
				return button;
			},

			onDelete: function() {
				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.deletingStatus);

				// Deleting top level products, so ignore "parts" that should be null. Just grab URIs.
				var args = {
					itemVersionUris: dojo.map(this.partAndChildUris, function(partAndChild) {
						return partAndChild.child;
					})
				};

				jazz.client.xhrDelete({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/item_version', //$NON-NLS-1$
					content: args,

					handle: function() {
						message.hide();
					},

					load: dojo.hitch(this, function(newItem) {
						dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$
						var dialog = this._dialog;

						com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh().addBoth(function() {
							dojo.publish('ple.select_nothing'); //$NON-NLS-1$
							dialog.close();
						});
					}),

					failOk: true,
					error: dojo.hitch(this, function(error) {
						console.log("Delete failed"); //$NON-NLS-1$
						//TODO: show error as an info box and not as a warning
						// TODO: show message in THIS dialog box, don't close. User must cancel if they can't resolve the problem
						this._dialog.close();

						dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$
						dojo.publish('ple.product_tree_xhrError',[error, true]); //$NON-NLS-1$
					})
				});
			},

			onRemove: function() {
				var self = this;

				var deleteOption = this.checkbox.checked;
				var editChoice = (deleteOption)? "unuse_and_delete" : "unuse"; //$NON-NLS-1$ //$NON-NLS-2$

				if (this.removeButton) this.removeButton.disabled = true;
				if (this.deleteButton) this.deleteButton.disabled = true;
				this.cancelButton.disabled = true;

				var deleteCheckboxDisabled = this.checkbox.disabled;
				this.checkbox.disabled = true;

				var message = com.ibm.relm.common.web.StatusMessage.show(this.messages.removingStatus);

				var dialog = this._dialog;

				var load = function() {
					dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$

					com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh().addBoth(function() {
						dojo.publish('ple.select_nothing'); //$NON-NLS-1$
						dialog.close();
					});
				};

				var error = function(error) {
					self.checkbox.disabled = deleteCheckboxDisabled;
					if (self.removeButton) self.removeButton.disabled = false;
					if (self.deleteButton) self.deleteButton.disabled = false;
					self.cancelButton.disabled = false;

					dojo.publish('ple.refresh'); //$NON-NLS-1$

					dojo.publish('ple.product_tree_hide_message'); //$NON-NLS-1$
					dojo.publish('ple.product_tree_xhrError', [error, true]); //$NON-NLS-1$

					// TODO: show error in THIS dialog.
					self._dialog.close();
				};

				var partAndChildUris = this.partAndChildUris || [
					{
						part: this.itemPartUri,
						child: this.itemVersionUri
					}
				];

				var content = {
					partAndChildUris: dojo.toJson(partAndChildUris),
					linkUris: dojo.toJson(this.linkUris || []),
					edit_choice: editChoice,
					parentVersionUri: this.parentProductUri,
					parentViewUri: this.parentViewUri
				};

				var serviceUrl = contextRoot + '/ple/product_version'; //$NON-NLS-1$

				jazz.client.xhrPost({
					handleAs: 'json', //$NON-NLS-1$
					url: serviceUrl,
					content: content,
					load: load,
					error: error,
					handle: function() {
						message.hide();
					}
				});
			},

			checkPrereqs: function(button) {
				var hasChildren = this.hasChildren;
				var immutable = this.immutable;

				if (hasChildren || immutable) {
					var message = null;
					if(hasChildren && immutable) message = this.messages.removeProductDoubleWarning;
					else if(hasChildren) message = this.messages.removeProductChildWarning;
					else if(immutable) message = this.messages.removeProductImmutableWarning;
					button.disabled = true;
					this.messageBox.showMessage({
						type: STATUS_TYPES.WARNING,
						summary: message
					});
				}
			},

			startup: function() {
				if (this._defaultButton) {
					var self = this;
					this.connect(window.document, 'onkeydown', function(e) { //$NON-NLS-1$
						if (e.keyCode == dojo.keys.ENTER) {
							self._defaultButton.click();
						}
					});
				}
			}

		});

	})();