/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.ple.web.ui.picker.AddUrlDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.picker", "AddUrlDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('jazz.util.html'); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.RequiredMarker'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require("jazz.app.ua.TextHoverHelp"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$

(function(){

	var LAST_URI_COMPONENT_RE = /(\/|#)([^\/#]+)$/;

	/**
	 * Dialog to select which picker to use for a product.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.AddUrlDialog', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ], 
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/AddUrlDialog.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			/**
			 * Callback function when the dialog is submitted. Will be passed
			 * a JSON object of all the form elements (name, pinning state, etc.).
			 */
			onOk: null,
			

			constructor: function(args, node) 
			{
				this.messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.picker", "AddUrlDialogMessages"); //$NON-NLS-1$ //$NON-NLS-2$
				this._uriId = dijit.getUniqueId('ple-AddUrlDialog'); //$NON-NLS-1$
				this._nameId = dijit.getUniqueId('ple-AddUrlDialog'); //$NON-NLS-1$

				this._shouldGuessName = true;
			},

			postCreate: function() {
				this.inherited(arguments);
				
				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.messages.dialogTitle,
					width: '500px', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: 'ple-icon-addHyperlink' //$NON-NLS-1$
				});

				var uaKeyName = "com.ibm.relm.ple.web.text_HyperlinkName001"; //$NON-NLS-1$
				this._nameHelp = new jazz.app.ua.TextHoverHelp({uaKey: uaKeyName, contentNode:this._labelTitle});


				this.connect(this._urlField, 'onchange', this.guessLabel); //$NON-NLS-1$
				this.connect(this._urlField, 'onkeyup', this.guessLabel); //$NON-NLS-1$

				this.connect(this._labelField, 'onchange', function() { //$NON-NLS-1$
					this._shouldGuessName = false;
				});
			},
			
			/**
			 * Takes the URL and guesses a pretty label for it if the user hasn't
			 * entered it yet. Basically removes the ?query portion, then takes the last
			 * path component, whichis often a file name. Also, decodes the URL escaping.
			 */
			guessLabel: function(e) {
				if (!this._shouldGuessName) return;
				var guess = e.target.value;
				var questionIndex = guess.indexOf('?'); //$NON-NLS-1$
				if (questionIndex !== -1) guess = guess.substring(0, questionIndex);

				var m = LAST_URI_COMPONENT_RE.exec(guess);

				if (m) {
					guess = decodeURIComponent(m[2]);
					this._labelField.value = guess;
				}
			},

			startup: function() {
				this._urlField.select();
			},

			_validate: function() {
				var url = dojo.trim(this._urlField.value);
				var name = this._labelField.value;
				if (url.length === 0 || name.length === 0) {
					this._addButton.disabled = true;
				}
				else {
					this._addButton.disabled = false;
				}
			},

			_onSubmit: function(e) {
				dojo.stopEvent(e);

				this._messageBox.hideMessage(true);

				var url = dojo.trim(this._urlField.value);
				var label = dojo.trim(this._labelField.value || this._labelField.placeholder || ""); //$NON-NLS-1$
				label = jazz.util.html.escape(label);

				this._urlField.disabled = this._labelField.disabled = true;
				this._addButton.disabled = this._cancelButton.disabled = true;

				if (this.onOk) {
					this.onOk(url, label);
				}
				else {
					// Shouldn't get here. Should always have callback
					this.close();
				}
			},

			/**
			 * Closes the outer Jazz dialog, which implicitly destroys this.
			 */
			close: function() {
				this._dialog.close();
				delete this._dialog;
			},
			
			showXhrError: function(xhr) {
				this._messageBox.showXhrError(xhr);

				// After showing error, reenable controls to allow user to try again
				this._urlField.disabled = this._labelField.disabled = false;
				this._cancelButton.disabled = false;
				this._validate();
			},

			/**
			 * Cancels the create and closes the dialog.
			 */
			cancel: function() {
				// this callbacks to destroy outer dialog too
				this.close();
			},
			
			uninitialize: function() {
				this._nameHelp && this._nameHelp.destroyRecursive();
			}
		});	
	})();
