/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.PickerResult'); //$NON-NLS-1$

// dojo.require('dojo.i18n');
dojo.requireLocalization('com.ibm.relm.ple.web.ui.picker', 'PickerResultMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dojo.date.locale'); //$NON-NLS-1$
dojo.require('jazz.util.html'); //$NON-NLS-1$

(function(){
	
	var DEFAULT_ICON = dojo.moduleUrl('com.ibm.relm.ple.web', 'ui/images/document.gif'); //$NON-NLS-1$ //$NON-NLS-2$

	var util = com.ibm.relm.common.web.ui.util;
	var html = jazz.util.html;

	var dateStyle = {
		selector: 'date', //$NON-NLS-1$
		formatLength: 'medium' //$NON-NLS-1$
	};

	function formatDate(timeMillis) {
		return dojo.date.locale.format(
			new Date(timeMillis),
			dateStyle);
	}

	var dateTimeStyle = {
		formatLength: 'medium' //$NON-NLS-1$
	};

	function formatDateTime(timeMillis) {
		return dojo.date.locale.format(
			new Date(timeMillis),
			dateTimeStyle);
	}

	/**
	 * A single "row" in the LQE artifact picker search results.
	 * It's complex enough to farm out to its own widget.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.PickerResult', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.ple.web','ui/picker/templates/PickerResult.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: false,
			messages: dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'PickerResultMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * A single JSON item from search results. This class
			 * has intimate knownledge of the properties on the items.
			 * @type {object}
			 */
			item: null,

			postCreate: function() {
				var item = this.item;

				this.inherited(arguments);

				if (item.iconClass) {
					dojo.addClass(this.iconNode, item.iconClass);
				}
				else {
					this.iconNode.src = item.iconUrl || DEFAULT_ICON;
				}
				
				if (item.artifactType) this.iconNode.title = item.artifactType;

				this.titleNode.innerHTML = item.text;

				if (item.description) {
					// RTC workitem possibly contains <div>, <b>, <i> and <a> and user inputted html tags, e.g. <img>, are already sanitized.
					// So scripts are just removed for description instead of escaping all html tag.
					this.descriptionNode.innerHTML = html.removeScript(item.description);
				}
				else {
					dojo.destroy(this.descriptionNode);
				}

				if (item.modifiedTime) {
					dojo.attr(this.modifiedBlock, 'title', formatDateTime(item.modifiedTime)); //$NON-NLS-1$
					var modifiedString = formatDate(item.modifiedTime);
					this.modifiedNode.innerHTML = html.escape(modifiedString);
				}
				else {
					dojo.destroy(this.modifiedBlock);
				}

				if (item.createdTime) {
					dojo.attr(this.createdBlock, 'title', formatDateTime(item.createdTime)); //$NON-NLS-1$
					var createdString = formatDate(item.createdTime);
					this.createdNode.innerHTML = html.escape(createdString);
				}
				else {
					dojo.destroy(this.createdBlock);
				}

				if (item.contributorName) {
					this.ownerNode.innerHTML = html.escape(item.contributorName);
				}
				else {
					dojo.destroy(this.ownerNode);
				}

				if (item.cmStatus) {
					this.statusBlock.style.display = ''; //$NON-NLS-1$
					this.statusNode.innerHTML = html.escape(item.cmStatus) ;
				}

				if (item.depth) {
					var depthPadding = 24 + (24 * item.depth) + 'px'; //$NON-NLS-1$
					this.domNode.style.paddingLeft = depthPadding;

					// dojo.create('div', { //$NON-NLS-1$
					// 	className: 'relm-PickerResult-indent' //$NON-NLS-1$
					// }, this.domNode);
				}

				if (item.doorsProjectTitle) {
					dojo.create('div', { //$NON-NLS-1$
						innerHTML:  html.escape(item.doorsProjectTitle) + ' >', //$NON-NLS-1$ //$NON-NLS-2$
						className: 'j-small ple-PickerResult-project', //$NON-NLS-1$
						title: this.messages.project
					}, this.titleBlock, 'before'); //$NON-NLS-1$
				}

				// No longer needed, so GC
				delete this.item;
			},

			/**
			 * Adds "next column" style link to the result. Used only by DOORS 9 modules
			 * to reveal baselines for now. The caller must connect an event to this to
			 * do anything.
			 * @param {string} linkName name of the link.
			 */
			addDetailsLink: function(linkName) {
				var link = dojo.create('a', { //$NON-NLS-1$
					href: '#', //$NON-NLS-1$
					className: 'j-small ple-icon-goDeeper ple-LqeArtifactPicker-accessory', //$NON-NLS-1$
					innerHTML: html.escape(linkName)
				}, this.sidebarBlock);

				return link;
			},

			/**
			 * Used as the callback after an OSLC preview is loaded.
			 * @param  {object} result OSLC or legacy preview results.
			 */
			updateFromOlscPreview: function(result) {
				// no inline oauth login here
				if (result.oauthRequired) return;

				var title;
				var icon;
				var preview;

				if (result.oslcCompact) {
					title = result.oslcCompact.title || result.oslcCompact.shortTitle;
					icon = result.oslcCompact.icon;
				}
				else if (result.compactRendering) {
					title = result.compactRendering.title || result.compactRendering.shortTitle;
					icon = result.compactRendering.icon;
				}
				else {
					console.debug('Do not know how to render this preview style:', result); //$NON-NLS-1$
					return;
				}

				if (icon) {
					dojo.attr(this.iconNode, 'src', icon); //$NON-NLS-1$
				}

				if (title) {
					// I expect this to be escaped HTML already.
					this.titleNode.innerHTML = title;
				}
			}
		}
	);

})();