/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.QueryPicker'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.picker', 'QueryPickerMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.branch.HeaderItem'); //$NON-NLS-1$
dojo.require('jazz.ui.Tree'); //$NON-NLS-1$

dojo.require('com.ibm.relm.explorer.web.explorer.page.ExplorerClient'); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler'); //$NON-NLS-1$
dojo.require('jazz.ui.tree.IconItem'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.BasicTreeItem'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$

(function(){
	var ServiceResponseHandler = com.ibm.relm.explorer.web.explorer.page.ServiceResponseHandler;
	var ExplorerClient = com.ibm.relm.explorer.web.explorer.page.ExplorerClient;

	var EXPANDED_PREFIX = 'relm.QueryPicker.expandFolder.'; //$NON-NLS-1$

	/**
	 * Widget to compactly present all queries. Used in artifact picker dialog in 4.0.6.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.QueryPicker', //$NON-NLS-1$
		[ dijit._Widget ],
		{
			messages:  dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.picker', 'QueryPickerMessages'), //$NON-NLS-1$ //$NON-NLS-2$

			buildRendering: function() {
				this.domNode = dojo.create('div', { //$NON-NLS-1$
					className: 'ple-QueryPicker ple-borderBox' //$NON-NLS-1$
				});

				this.inherited(arguments);
			},

			postCreate: function() {
				this.inherited(arguments);
				this.refresh();
			},

			/**
			 * Make a ServiceResponseHandler look more like a dojo
			 * XHR call.
			 * @param  {object} args requires arg.load and args.error callbacks.
			 */
			_handler: function(args) {
				return new ServiceResponseHandler(this, args.load, args.error);
			},

			/**
			 * Returns a ServiceResponseHandler that simple callbacks
			 * success and error to a given Deferred.
			 * @param  {dojo.Deferred} deferred
			 * @return {ServiceResponseHandler}
			 */
			_deferredResponseHandler: function(deferred) {
				return new ServiceResponseHandler(this, function(response) {
					deferred.callback(response);
				}, function(error) {
					deferred.errback(error);
				});	
			},

			/**
			 * Creates a tree item (node) for the tree appropriate
			 * for a query folder. Automatically adds it to the given parent node.
			 * @param  {AbstractItem} parentItem the parent to add to.
			 * @param  {object} folderInfo details of the folder.
			 */
			_createFolderItem: function(parentItem, folderInfo) {
				var folderItem = new com.ibm.relm.common.web.BasicTreeItem({
					folderUUID: folderInfo.folderUUID.value,
					label: com.ibm.relm.common.web.ui.util.escapeHtml(folderInfo.name.value),
					iconClass: 'ple-icon-nosprite-folder' //$NON-NLS-1$
				});

				parentItem.addChild(folderItem);
				folderItem.setExpanded(this._lastExpandedState(folderInfo));

				this._loadFolder(folderItem, folderInfo);
			},

			/**
			 * Creates a tree item (node) for the tree appropriate
			 * for a query. Automatically adds it to the given parent node.
			 * Will appears as link to click.
			 * @param  {AbstractItem} parentItem the parent to add to.
			 * @param  {object} queryInfo details of the query.
			 */
			_createQueryItem: function(parentItem, queryInfo) {
				var queryItem = new com.ibm.relm.common.web.BasicTreeItem({
					queryInfo: queryInfo,
					label: com.ibm.relm.common.web.ui.util.escapeHtml(queryInfo.title.value),
					iconClass: 'ple-icon-nosprite-query', //$NON-NLS-1$
					href: queryInfo.uri.value
				});

				parentItem.addChild(queryItem);
			},

			/**
			 * Given an existing folder in the tree, loads all the
			 * subfolders and query members from the server.
			 * @param  {BasicTreeItem} folderTreeItem the folder to load. Will
			 * show a loading indicator.
			 * @param  {object} folderInfo     details of the folder to load.
			 */
			_loadFolder: function(folderTreeItem, folderInfo) {
				var deferredQueries = new dojo.Deferred();
				var deferredSubfolders = new dojo.Deferred();
				var deferredList = new dojo.DeferredList([deferredSubfolders, deferredQueries], false, true);

				folderTreeItem.set('loading', true); //$NON-NLS-1$

				ExplorerClient.getFolderDescriptors(this._deferredResponseHandler(deferredSubfolders), {
					parentUUID: folderInfo.folderUUID.value,
					flags: ExplorerClient.ALL,
					sortByName: true
				});

				ExplorerClient.getQueryDescriptors(this._deferredResponseHandler(deferredQueries), {
					parentUUID: folderInfo.folderUUID.value,
					sortByName: true
				});

				var self = this;
				deferredList
				.then(function(responses) {
					var childFolders = responses[0][1].results.bindings;
					var queries = responses[1][1].results.bindings;

					var parentItem = folderTreeItem;
					dojo.forEach(childFolders, function(childFolderInfo) {
						self._createFolderItem(parentItem, childFolderInfo);
					});
					
					dojo.forEach(queries, function(queryInfo) {
						self._createQueryItem(parentItem, queryInfo);
					});
				})
				.otherwise(function(error) {
					console.debug('Error loading folder ' + folderInfo.name.value, error); //$NON-NLS-1$
				})
				.always(function() {
					folderTreeItem.set('loading', false); //$NON-NLS-1$
				});
			},

			/**
			 * Returns the last known expanded state of this item's folder.
			 * @param  {object} folderInfo the RDF-ish data from the server
			 * @return {boolean}      true if the folder should be expanded.
			 */
			_lastExpandedState: function(folderInfo) {
				var uuid = folderInfo.folderUUID.value; //$NON-NLS-1$
				if (!uuid) return false;
				return window.localStorage[EXPANDED_PREFIX + uuid] === 'true'; //$NON-NLS-1$ //$NON-NLS-2$
			},

			/**
			 * Saves the current expanded state of the folder to be able to restore it later.
			 * @param {AbstractItem}  node       tree node item. Assumed to have a `folderUUID` property
			 * already added it it.
			 * @param {Boolean} isExpanded true if expanded, otherwise false.
			 */
			_setLastExpandedState: function(node, isExpanded) {
				var uuid = node.folderUUID; //$NON-NLS-1$
				if (!uuid) return;
				window.localStorage[EXPANDED_PREFIX + uuid] = !!isExpanded + ''; //$NON-NLS-1$ //$NON-NLS-2$
			},

			/**
			 * Callback from the tree when a node is expanded.
			 * @param  {AbstractItem}  item       tree node item.
			 * @param  {Boolean} isExpanded [description]
			 * @return {[type]}             [description]
			 */
			_onSetExpanded: function(node, isExpanded) {
				this._setLastExpandedState(node, isExpanded);
			},

			_treeOnEnter: function() {
				var selectedItem = this._tree.getSelection()[0];
				if (!selectedItem) return;
				var queryUri = selectedItem.get('href'); //$NON-NLS-1$
				if (!queryUri) return;

				this.onPickedQueryUri(queryUri);
			},

			/**
			 * Callback for parent widget to connect to when a query URI is clicked.
			 * @param  {string} queryUri URI of selected query.
			 */
			onPickedQueryUri: function(queryUri) {
				// callback
			},

			/**
			 * Loads or reloads the whole tree. Everything is loaded--no 
			 * lazy loading is done.
			 */
			refresh: function() {
				this._tree && this._tree.destroyRecursive();
				this._tree = new jazz.ui.Tree({
					indent: 8, 
					onEnter: dojo.hitch(this, this._treeOnEnter)
				});
				this.connect(this._tree, '_setExpanded', this._onSetExpanded); //$NON-NLS-1$
				dojo.addClass(this._tree.domNode, 'ple-hasHeaders'); //$NON-NLS-1$
				this._tree.placeAt(this.domNode);
				this.own(this._tree);

				// Top level node for My Queries.
				// Note we fake a folder UUID here to auto track the collapsed state of these
				this._myFolder = new com.ibm.relm.ple.web.ui.branch.HeaderItem({
					label: this.messages.myQueries,
					folderUUID: 'system.MyQueries' //$NON-NLS-1$
				});

				// Top level node for Shared Queries
				this._sharedFolder = new com.ibm.relm.ple.web.ui.branch.HeaderItem({
					label: this.messages.sharedQueries,
					folderUUID: 'system.SharedQueries' //$NON-NLS-1$
				});

				this._tree.addChild(this._myFolder);
				this._tree.addChild(this._sharedFolder);

				this._myFolder.set('loading', true); //$NON-NLS-1$
				this._sharedFolder.set('loading', true); //$NON-NLS-1$

				var self = this;

				ExplorerClient.getSystemRootFolderDescriptors(this._handler({
					load: function(response) {
						var bindings = response.results.bindings;
						var myRootInfo = bindings[0];
						var sharedRootInfo = bindings[1];

						// Set UUIDs on the node to track collapsed state it localStorage.
						self._myFolder.set('folderUUID', myRootInfo.folderUUID.value); //$NON-NLS-1$
						self._myFolder.setExpanded(this._lastExpandedState(myRootInfo));
						
						self._sharedFolder.set('folderUUID', sharedRootInfo.folderUUID.value); //$NON-NLS-1$
						self._sharedFolder.setExpanded(this._lastExpandedState(sharedRootInfo));

						self._loadFolder(self._myFolder, myRootInfo);
						self._loadFolder(self._sharedFolder, sharedRootInfo);
					},
					error: function() {
						self._myFolder.set('loading', false); //$NON-NLS-1$
						self._sharedFolder.set('loading', false); //$NON-NLS-1$		
					}
				}));
			},

			uninitialize: function() {
				this._tree.destroyRecursive();
			}
		}
	);

})();