/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.tree.AltProductTree'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.AltTree'); //$NON-NLS-1$
dojo.require('jazz.ui.Toolbar'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.OnOffButton'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeMenuFactory'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.Pinning'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.MoveReusePlaceState'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.dnd.Avatar'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.dnd.DropBar'); //$NON-NLS-1$
dojo.require('relm.app.Permissions'); //$NON-NLS-1$
dojo.require('com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.doors.DoorsUtils'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.tree', 'ProductTreeMessages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.require('jazz.ui.internal.ResourceLinkAggregator'); //$NON-NLS-1$

(function(){

	var IS_FAVORITE_TIP_PROP = 'ple:isFavoriteTip'; //$NON-NLS-1$
	var IS_FAVORITE_PROP = '_favorite'; //$NON-NLS-1$

	// Sharing messages for now, since they are the same.
	var messages_ = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.tree", "ProductTreeMessages"); //$NON-NLS-1$ //$NON-NLS-2$

	var TreeMenuFactory = com.ibm.relm.ple.web.ui.TreeMenuFactory;
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	var formatting = com.ibm.relm.ple.web.ui.formatting;
	var MoveReusePlaceState = com.ibm.relm.ple.web.ui.MoveReusePlaceState;
	var util = com.ibm.relm.common.web.ui.util;
	var SystemParameterUtil = com.ibm.relm.explorer.web.explorer.queryparams.SystemParameterUtil;
	var DoorsUtils = com.ibm.relm.ple.web.doors.DoorsUtils;

	dojo.declare(
		'com.ibm.relm.ple.web.ui.tree.AltProductTreeNode', //$NON-NLS-1$
		[ com.ibm.relm.ple.web.ui.tree.AltTreeNode ],
		{
			_linkSrc: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/hyperlink_co.gif'), //$NON-NLS-1$ //$NON-NLS-2$
			_productSrc: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/product_co.gif'), //$NON-NLS-1$ //$NON-NLS-2$
			_productInSnapshotSrc: dojo.moduleUrl('com.ibm.relm.ple.web','ui/images/productInSnapshot_co.gif'), //$NON-NLS-1$ //$NON-NLS-2$

			_cancelNativeDrag: function(e) {
				dojo.stopEvent(e);
				return false;
			},

			updateIcon: function() {
				if (!this._hasArtifactIcon) {
					this.inherited(arguments);
				}

				this._requestCompactXmlUpdate();

				var frozenHierarchy =  this.item["ple:isFrozenHierarchy"]; //$NON-NLS-1$
				if (frozenHierarchy) {
					dojo.addClass(this.rowNode, "ple-AltTree-DisabledHierarchy"); //$NON-NLS-1$
				} else{
					dojo.removeClass(this.rowNode, "ple-AltTree-DisabledHierarchy"); //$NON-NLS-1$
				}
			},

			updateLabel: function() {
				if (!this._hasArtifactTitle) {
					this.inherited(arguments);
				}

				this._requestCompactXmlUpdate();
			},

			_requestCompactXmlUpdate: function() {
				if (this.item.type === 'link') { //$NON-NLS-1$
					if (this.item.isDoorsBaseline) {
						//Doors baslines should use a label and icon known by RELM since they do not have 
						//an OSLC preview
						this._iconNode.className = 'ple-AltTree-icon'; //$NON-NLS-1$
						this._hasArtifactIcon = true;
						this.item.liveIcon = DoorsUtils.DOORS_BASELINE_ICON;
						this._iconNode.src = DoorsUtils.DOORS_BASELINE_ICON;

						// this.labelNode.innerHTML = this.item.name;
						this._hasArtifactTitle = true;
						this.item.liveTitle = this.item.name;
					}
					else if (!this._aggregatorId) {
						this._aggregatorId = jazz.ui.internal.resourceLinkAggreagtor().register(this.item.oslcUri, dojo.hitch(this, this._updateIconAndLabelFromCompactXml), function() {});						
					}
				}	
			},

			_updateIconAndLabelFromCompactXml: function(result) {
				// check that we weren't unloaded
				if (!this._iconNode || !this.labelNode) return;

				delete this._aggregatorId;

				var oslcCompact = result.oslcCompact;
				var compactRendering = result.compactRendering;
				
				var icon = null;
				var title = null;
				
				if (oslcCompact) {
					icon = oslcCompact.icon;
					title = oslcCompact.title;
				}
				else if (compactRendering) {
					icon = compactRendering.icon;
					title = compactRendering.title;	
				}

				if (icon) {
					this._iconNode.src= icon;
					// Remove any custom class since we're using an img now
					this._iconNode.className = 'ple-AltTree-icon'; //$NON-NLS-1$
					this._hasArtifactIcon = true;
					this.item.liveIcon = icon;
				}

				if (title) {
					this.labelNode.innerHTML = title;
					this._hasArtifactTitle = true;
					this.item.liveTitle = title;
				}
			},

			getIconClassAndAltText: function() {
				return com.ibm.relm.ple.web.ui.formatting.getIconClassAndAltTextForItem(this.item);			
			},
				
			getIconSrc: function()
			{
				if (!this.item || !this.item.type) {
					return this._blankGif;
				}

				var type = this.item.type;

				if (type === 'link') //$NON-NLS-1$
				{
					return this._linkSrc;
				}
				else if (type === 'product') //$NON-NLS-1$
				{
					var immutable = this.item['pd:immutable']; //$NON-NLS-1$
					return immutable ? this._productInSnapshotSrc : this._productSrc;
				}
				else {
					return this._blankGif;
				}
			},

			toggleSelected: function(e) {
				dojo.stopEvent(e);

				// If not already in a multi-selection we clear the selected paths. This is done
				// to ensure that we don't maintain the old selected (but not checkboxed) product
				// the first time we click a checkbox.
				if (this.tree._selectedTreeNodes.length === 1 &&
					!dojo.hasClass(this.tree.domNode, 'ple-AltTree-multiSelect')) { //$NON-NLS-1$

					this.tree.clearAllSelectedPaths();
				}

				this.tree._treeNodeWasClicked(e, this, true);
			},

			_confirmUnpin: function(e) {
				var parentItem = this.parentTreeNode().item;
				com.ibm.relm.ple.web.ui.Pinning.unpinProduct(parentItem, this.item);
			},

			/**
			 * Event handler when Pin/Unpin button is clicked.
			 * Will pin or unpin based on current pinned state (cached in the tree)
			 * of the product.
			 * @param  {event} e
			 */
			_onPinClicked: function(e) {
				var parentProduct = this.parentTreeNode().item;
				var product = this.item;

				if (this.item['pd:pinned']) { //$NON-NLS-1$
					com.ibm.relm.ple.web.ui.Pinning.unpinProduct(parentProduct, product);
				}
				else {
					com.ibm.relm.ple.web.ui.Pinning.pinProduct(parentProduct, product);
				}
			},

			createLeadingToolbar: function(item) {
				var store = this.tree.store;

				var toolbar = new jazz.ui.Toolbar();

				// This will show the toolbar when tabbed to for accessibility
				this.connect(toolbar, 'onFocus', function(e) { //$NON-NLS-1$
					dojo.addClass(this.rowNode, 'ple-AltTree-focusedToolbar'); //$NON-NLS-1$
				});
				this.connect(toolbar, 'onBlur', function(e) { //$NON-NLS-1$
					dojo.removeClass(this.rowNode, 'ple-AltTree-focusedToolbar'); //$NON-NLS-1$
				});

				var type = store.getValue(this.item, 'type'); //$NON-NLS-1$

				// checkbox for all types
				this._selectButton = new jazz.ui.toolbar.Button({
					imgClass: 'ple-AltProductTree-checkbox', //$NON-NLS-1$
					title: messages_.uncheckedLabel,
					onClick: dojo.hitch(this, this.toggleSelected)
				});
				dojo.addClass(this._selectButton.domNode, 'ple-AltProductTree-checkboxButton'); //$NON-NLS-1$
				toolbar.addChild(this._selectButton);

				if (type === 'product') { //$NON-NLS-1$
					var isPinned = !!store.getValue(this.item, 'pd:pinned'); //$NON-NLS-1$

					this._pinnedButton = new com.ibm.relm.ple.web.ui.OnOffButton({
						onClick: dojo.hitch(this, this._onPinClicked),
						imgClassOn: 'ple-icon-pinned', //$NON-NLS-1$
						imgClassOff: 'ple-icon-unpinned', //$NON-NLS-1$
						titleOn: messages_.unpinAndReplace,
						titleOff: messages_.pinVersionTooltip
					});
					toolbar.addChild(this._pinnedButton);

					var isChild = this.depth !== 0;

					if (!isChild) {
						this._pinnedButton.domNode.style.visibility = 'hidden'; //$NON-NLS-1$
						this._pinnedButton.set('enabled', false); //$NON-NLS-1$
					}
					else {
						this._pinnedButton.toggleOnOff(isPinned);

						var frozenHierarchy = this.item["ple:isFrozenHierarchy"]; //$NON-NLS-1$
						if (!relm.app.Permissions.canProductsModify() || frozenHierarchy) {
							this._pinnedButton.set('enabled', false); //$NON-NLS-1$
						}
					}
					
					this._favoriteButton = new com.ibm.relm.ple.web.ui.OnOffButton({
						onClick: dojo.hitch(this, this.toggleFavorite),
						imgClassOn: 'ple-icon-favorite', //$NON-NLS-1$
						imgClassOff: 'ple-icon-favoriteOff', //$NON-NLS-1$
						titleOn: messages_.favoriteToggleOnTooltip,
						titleOff: messages_.favoriteToggleOffTooltip
					});

					toolbar.addChild(this._favoriteButton);

					var isFavorite = store.getValue(this.item, IS_FAVORITE_PROP) ||
					store.getValue(this.item, IS_FAVORITE_TIP_PROP);

					this._favoriteButton.toggleOnOff(isFavorite);
				}
				else {
					for (var i = 0; i < 2; i++) {
						var spacer = new jazz.ui.toolbar.Button({
							enabled: false
						});
						toolbar.addChild(spacer);
						dojo.addClass(spacer.domNode, 'ple-AltTree-spacerButton'); //$NON-NLS-1$
					}
				}

				this._menuButton = new jazz.ui.toolbar.Button({
					imgClass: 'ple-icon-dropDownMenu', //$NON-NLS-1$
					title: messages_.menuButton,
					onClick: dojo.hitch(this, this.openMenu)
				});
				toolbar.addChild(this._menuButton);

				this.connect(this.domNode, 'oncontextmenu', this.openMenu); //$NON-NLS-1$
				toolbar.startup();
				return toolbar;
			},

			/**
			 * Toggles the favorite state of a product.
			 * This causes a call to the server to change the state. If successful,
			 * the _favorite attribute of the store is updated, which will notify the tree nodes
			 * that they need to toggle the star icon on or off (see _setItemAttr).
			 */
			toggleFavorite: function(e) {
				var target = e.currentTarget;

				var treeNode = com.ibm.relm.ple.web.ui.tree.AltTree.getEnclosingTreeNode(target);
				if (!treeNode) return;

				var item = treeNode.item;
				var store = this.tree.store;
				var content = {};

				var favoriteButton = dijit.getEnclosingWidget(target);

				if (favoriteButton.isToggledOn()) {
					var isFavoriteTip = store.getValue(item, IS_FAVORITE_TIP_PROP);

					if (isFavoriteTip) {
						content.removeUri = store.getValue(item, 'pd:branch'); //$NON-NLS-1$
					}
					else {
						content.removeUri = store.getValue(item, 'uri'); //$NON-NLS-1$
					}
					var nowFavorite = false;
				}
				else {
					content.addItemVersionUri = store.getValue(item, 'uri'); //$NON-NLS-1$
					nowFavorite = true;
				}

				jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + '/ple/favorites', //$NON-NLS-1$
					content: content,
					handleAs: 'json', //$NON-NLS-1$
					load: dojo.hitch(this, function(addResult) {

						this.tree.setValueForAllItemsWithSameUriAndUpdate(item.uri, IS_FAVORITE_PROP, nowFavorite);

						if (addResult) {
							var tip = addResult[IS_FAVORITE_TIP_PROP];

							if (tip) {
								this.tree.setValueForAllItemsWithSameUriAndUpdate(item.uri, IS_FAVORITE_TIP_PROP, true);
							}
						}
						else {
							// Delete that prop
							this.tree.setValueForAllItemsWithSameUriAndUpdate(item.uri, IS_FAVORITE_TIP_PROP, null);
						}

						dojo.publish('ple.delayed_favorites_refresh'); //$NON-NLS-1$
					}),
					error: function(error) {
						console.error(error);
						dojo.publish('ple.product_tree_xhrError', [error, true]); //$NON-NLS-1$
					}
				});
			},

			openMenu: function(e) {
				dojo.stopEvent(e);
	
				var rightClick = e.type == 'contextmenu'; //$NON-NLS-1$

				this.tree.clickTreeNode(this, dojo.isCopyKey(e));
				 
				var currentRow = this.rowNode;
				dojo.addClass(currentRow, 'ple-AltTree-menuOpen'); //$NON-NLS-1$

				var oldFocusedNode = dijit.getFocus().node;
				var menu = TreeMenuFactory.newContextMenuForSelected(rightClick);

				var popupArgs = {
					parent: this,
					popup: menu,

					orient: {BL:'TL', BR: 'TR', TL:'BL', TR: 'BR'}, //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

					onClose: dojo.hitch(this, function() {
						dojo.removeClass(currentRow, 'ple-AltTree-menuOpen'); //$NON-NLS-1$
						menu.destroyRecursive();
						if (oldFocusedNode && oldFocusedNode.focus) {
							oldFocusedNode.focus();
						}
					}),

					onCancel: function() {
						dijit.popup.close(menu);
					},

					onExecute:  function(e) {
						dijit.popup.close(menu);
					}
				};

				if (rightClick) {
					popupArgs.x = e.clientX;
					popupArgs.y = e.clientY;
				}
				else {
					popupArgs.around = this._menuButton.domNode;
				}

				menu.connect(menu, 'onBlur', function() { //$NON-NLS-1$
					dijit.popup.close(menu);
				});

				dijit.popup.open(popupArgs);
				menu.focus();
			},

			updateToolbars: function(item, attribute, oldValue, newValue) {
				var store = this.tree.store;

				if (this._favoriteButton) {
					var isFavorite = store.getValue(this.item, IS_FAVORITE_PROP);
					this._favoriteButton.toggleOnOff(isFavorite);
				}

				if (this._pinnedButton) {
					var isPinned = !!store.getValue(this.item, 'pd:pinned'); //$NON-NLS-1$
					this._pinnedButton.toggleOnOff(isPinned);
					var frozenHierarchy =  this.item["ple:isFrozenHierarchy"]; //$NON-NLS-1$
					this._pinnedButton.set('enabled', relm.app.Permissions.canProductsModify() && !frozenHierarchy); //$NON-NLS-1$
				}
			},

			getLabelHtml: function() {
				var store = this.tree.store;
				var name = store.getValue(this.item, 'name'); //$NON-NLS-1$

				var isDeleted = store.getValue(this.item, '_isDeleted'); //$NON-NLS-1$

				if (isDeleted) {
					// special case for Compare Tree. Should probably move to special subclass for that view only.
					return '<span class="ple-deleted">' + jazz.util.html.escape(name) + '</span>'; //$NON-NLS-1$ //$NON-NLS-2$
				}
				else if (this.item.link) {
					// dcterms:title from most artifact are XML literals (essentially XHTML).
					return this.item.name;
				}
				else {
					return formatting.formatProductTitleForItem(this.item);
				}
			}
		}
	);

	dojo.declare(
		'com.ibm.relm.ple.web.ui.tree.AltProductTree', //$NON-NLS-1$
		[ com.ibm.relm.ple.web.ui.tree.AltTree ],
		{
			createTreeNode: function(item, depth) {
				return new com.ibm.relm.ple.web.ui.tree.AltProductTreeNode({
					identity: this.store.getIdentity(item),
					depth: depth,
					tree: this,
					item: item
				});
			},

			constructor: function() {
				// All the event handles connected during a drag to manually disconnect
				// at the end.
				this._dragEventHandles = [];
			},

			postCreate: function() {
				this.inherited(arguments);
				this.connect(this, 'selectionChanged', this._updateTreeSelection); //$NON-NLS-1$

				this.connect(this.domNode, 'onmousedown', this._onDragStart); //$NON-NLS-1$

				dijit.setWaiState(this.domNode, 'multiselectable', true); //$NON-NLS-1$
				dijit.setWaiState(this.domNode, 'label', 'Product Tree'); //$NON-NLS-1$ //$NON-NLS-2$
			},

			/**
			 * First step of drag-n-drop. Checks if this is a proper node to
			 * start dragging from. Defers some setup until the mouse actually
			 * moves for performances reasons--no need to a lot of setup when
			 * the user just clicked.
			 * @param  {mouse event} e a mousedown that may proceed a drag.
			 */
			_onDragStart: function(e) {
				// We don't want to drag from buttons etc, so just whitelist few nice places.
				var isValidDragNode = dojo.some(['ple-AltTree-label', 'ple-decorator', 'ple-rtcConnected', 'ple-AltTree-icon', 'ple-AltTree-mainCell'], function(expectedClass) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
					return dojo.hasClass(e.target, expectedClass);
				});
				if (!isValidDragNode) return;
				
				// Ignore right click
				if (e.button > 1) return;

				var draggedTreeNode = com.ibm.relm.ple.web.ui.tree.AltTree.getEnclosingTreeNode(e);
				if (!draggedTreeNode) return;

				var allSelected = TreeSelection.getAllSelectedSites();
				var draggedIsInSelection = dojo.some(allSelected, function(selectedSite) {
					// Normally we just drag the exact tree node that was
					// draggeg, whether it's selected or not. But, if it is
					// selected, we want to drag ALL the selected nodes in the
					// selection. This matches most common selection behaviors
					// (Windows Explorer, OS X Finder).
					return (selectedSite.link || selectedSite.product) == draggedTreeNode.item;
				});

				var draggedSite = TreeSelection.newSitesFromItemPaths([draggedTreeNode.getItemPath()])[0];

				var self = this;
				function avatarFactory() {
					var avatarPos = dojo.position(dojo.query('.ple-AltTree-icon', draggedTreeNode.domNode)[0]); //$NON-NLS-1$
					// Fudge the position a bit to accomodate padding. Not really needed.
					avatarPos.x -= 4;
					avatarPos.y -= 4;

					var avatar = new com.ibm.relm.ple.web.dnd.Avatar({
						startX: avatarPos.x,
						startY: avatarPos.y
					});

					var iconNode;
					var labelNode;

					if (draggedIsInSelection && allSelected.length > 1) {
						labelNode = dojo.create('span', { //$NON-NLS-1$
							className: 'ple-AltTree-label' //$NON-NLS-1$
						});
						util.setText(labelNode, dojo.replace(messages_.countOfXSelectedItems, [allSelected.length]));

						var icons = dojo.map(self._selectedTreeNodes, function(treeNode) {
							return treeNode._iconNode;							
						});
						iconNode = com.ibm.relm.ple.web.ui.internal.views.MultiSelectView.generateMultiIcon(icons);
					}
					else {
						iconNode = draggedTreeNode._iconNode.cloneNode(true);
						dojo.addClass(iconNode, 'ple-icon'); //$NON-NLS-1$
						labelNode = draggedTreeNode.labelNode.cloneNode(true);
						dojo.removeAttr(labelNode, 'tabIndex'); //$NON-NLS-1$
					}

					dojo.place(iconNode, avatar.contentNode);
					dojo.place(labelNode, avatar.contentNode);

					return avatar;
				}

				this.startDrag(e, draggedIsInSelection ? allSelected : draggedSite, avatarFactory);
			},

			/**
			 * Begins a drag-n-drop. can be called from other widgets
			 * to drag from them. Defers some setup until the mouse actually
			 * moves for performances reasons--no need to a lot of setup when
			 * the user just clicked.
			 * @param  {mouse event} e a mouse down or click event that may start a drag.
			 * @param {object or array} draggedSite one or more "sites"
			 *     created by TreeSelection that indicates the object to drag.
			 * @param {function} avatarFactory a function that returns a new
			 *     com.ibm.relm.ple.web.dnd.Avatar object. It must have a
			 *     valid absolute X and Y for the document, but does not need
			 *     to be inserted into the DOM yet.
			 */
			startDrag: function(e, draggedSites, avatarFactory) {
				// Ignore right click
				if (e.button > 1) return;

				this._draggedSites = dojo.isArray(draggedSites) ? draggedSites : [draggedSites];
				this._avatarFactory = avatarFactory;

				// These should only be connected while a drag is active.
				this._dragEventHandles.push(dojo.connect(document, 'onmousemove', this, this._onDragMove)); //$NON-NLS-1$
				this._dragEventHandles.push(dojo.connect(document, 'onmouseup', this, this._onDrop)); //$NON-NLS-1$

				// These are just to protected again missing mouseup events (outside the window for example) that
				// that we never get stuck in a drag (just click to cancel).
				this._dragEventHandles.push(dojo.connect(document, 'onclick', this, this._cancelDrag)); //$NON-NLS-1$
				this._dragEventHandles.push(dojo.connect(this, 'onclick', this, this._cancelDrag)); //$NON-NLS-1$
				this._dragEventHandles.push(dojo.connect(document, 'onkeyup', this, function(e) { //$NON-NLS-1$
					if (e.keyCode === dojo.keys.ESCAPE) this._cancelDrag();
				}));

				// Remember where the drag started so we can calculate how far we've moved later.
				this._startX = e.clientX;
				this._startY = e.clientY;

				// webkit insists on selecting text when a drag starts. Disable selection globally to force prevent this.
				// Must do this on the document itself, or moving over other parts of the page
				// can still be seen as a "drag and select" when moving the mouse too fast.
				this._dragEventHandles.push(dojo.connect(document, 'ondragstart', dojo.stopEvent)); //$NON-NLS-1$
				this._dragEventHandles.push(dojo.connect(document, 'onselectstart', dojo.stopEvent)); //$NON-NLS-1$

				// We defer doing any more setup until the user actually moves
				// the mouse and we know this is real drag or not. This could
				// just be an onclick at this point.
			},

			/**
			 * Called each time the mouse moves during a drag. Updates the
			 * avatar position and tracks the drop target we might be over.
			 * @param  {mouse} e
			 */
			_onDragMove: function(e) {
				dojo.stopEvent(e);

				var diffX = e.clientX - this._startX;
				var diffY = e.clientY - this._startY;

				if (!this._avatar &&
					Math.abs(diffX) < 7 &&
					Math.abs(diffY) < 7) {
						// Need to reach minimum drag distance before we consider it a real drag.
						return;
					}

				if (!this._avatar) {
					// If this is the first "move" event, we haven't created
					// the avatar yet or calculated drop regions. Do that now.
					// This is deferred since it's relatively expensive and
					// want wait until the user actually want to DnD (and not
					// just click).
					this._initDrag(e);
				}

				this._avatar.moveBy(diffX, diffY);

				this._highlightActiveDropRegion(e);
			},

			_checkForReuseOnKeyDown: function(e) {
				if (e.keyCode === dojo.keys.CTRL) {
					dojo.stopEvent(e);
					this._preferReuseOnDrag = true;
					this._updateAvatarHint();
				}
			},

			_checkForReuseOnKeyUp: function(e) {
				if (e.keyCode === dojo.keys.CTRL) {
					dojo.stopEvent(e);
					delete this._preferReuseOnDrag;
					this._updateAvatarHint();
				}
			},

			_updateAvatarHint: function() {
				if (!this._avatar) return;

				var isReuse = this._forceReuseOnDrag || this._preferReuseOnDrag;
				var isMoveToTop = this._activeDropRegion && this._activeDropRegion.moveToTop;

				if (isMoveToTop) {
					if (isReuse) {
						this._avatar.set('hintIconClass', 'ple-icon-reuse'); //$NON-NLS-1$ //$NON-NLS-2$
						this._avatar.set('hintText', messages_.dndAddToTop); //$NON-NLS-1$
					}
					else {
						this._avatar.set('hintIconClass', 'ple-icon-move'); //$NON-NLS-1$ //$NON-NLS-2$
						this._avatar.set('hintText', messages_.dndMoveToTop); //$NON-NLS-1$
					}
				}
				else if (isReuse) {
					this._avatar.set('hintIconClass', 'ple-icon-reuse'); //$NON-NLS-1$ //$NON-NLS-2$
					this._avatar.set('hintText'); //$NON-NLS-1$
				}
				else {
					this._avatar.set('hintIconClass'); //$NON-NLS-1$
					this._avatar.set('hintText'); //$NON-NLS-1$
				}
			},

			/**
			 * Called once a drag really starts (mouse was moved). Creates the
			 * drag avatar and figures out all the drop regions.
			 * @param  {mouseevent} e
			 */
			_initDrag: function(e) {
				var commonParentUri = null;

				this._forceReuseOnDrag = dojo.some(this._draggedSites, function(site) {
					// If we're a product (not a link) and we have no parent,
					// then it's a top level. Move doesn't make sense in that
					// case, so force a reuse.
					if (!site.parentProduct && !site.link) return true;
					
					// For multi-object DnD, all items must have the same parent to
					// participate in a Move.
					if (site.parentProduct) {
						var currentParentUri = site.parentProduct.uri;
						if (commonParentUri && commonParentUri !== currentParentUri) return true;
						commonParentUri = currentParentUri;
					}

					//If we are dragging from a frozen hierarchy, then only allow a move and not a reuse
					if (site.product["ple:isFrozenHierarchy"]) return true; //$NON-NLS-1$

					return false;
				});
	
				var summary = this._forceReuseOnDrag ? messages_.dndReuseHint : messages_.dndMoveHint;
				dojo.publish('ple.show_drag_message', [summary]); //$NON-NLS-1$
				
				this._avatar = this._avatarFactory();

				if (this._forceReuseOnDrag) {
					this._avatar.set('hintIconClass', 'ple-icon-reuse'); //$NON-NLS-1$ //$NON-NLS-2$
				}
				else {
					this._avatar.set('hintIconClass'); //$NON-NLS-1$	
					this._dragEventHandles.push(dojo.connect(document, 'onkeydown', this, this._checkForReuseOnKeyDown)); //$NON-NLS-1$
					this._dragEventHandles.push(dojo.connect(document, 'onkeyup', this, this._checkForReuseOnKeyUp)); //$NON-NLS-1$
				}
				
				this._avatar.placeAt(document.body);

				dojo.addClass(document, 'ple-AltTree-inDrag'); //$NON-NLS-1$
				dojo.addClass(document.body, 'ple-AltTree-inDrag'); //$NON-NLS-1$


				// We cache the positions of all the valid nodes below. This
				// is faster than reading the position information on the fly
				// later on. But, if the region scrolls it will shift the
				// position we should be looking at. We need to cache where
				// the original scrolling position is, so we can compare the
				// current position later.
				this._startScrollTop = this.autoHeightBox.box.scrollTop;
				this._visibleRegion = dojo.position(this.autoHeightBox.box);

				this._initDropRegions();
				this._initTopLevelDropRegions();
				// Don't try to highlight while autoscrolling. Will highlight again on mousemove.
				this._dragEventHandles.push(dojo.connect(this.autoHeightBox.box, 'onscroll', this, this._clearActiveDropRegion)); //$NON-NLS-1$

				this.autoHeightBox.enableAutoScroll();
			},

			_initDropRegions: function() {
				this._dropRegions = [];

				var draggedUriSet = {};
				dojo.forEach(this._draggedSites, function(site) {
					var item = site.link || site.product;
					draggedUriSet[item.uri] = true;
				});

				var dropTargetNodes = dojo.filter(this.getVisibleNodes(), function(treeNode) {
					// Only drop on products.
					if (treeNode.item.type !== 'product') return false; //$NON-NLS-1$

					// if (treeNode.frozenHierarchy || treeNode.item['ple:isFrozenTip']) return false; //$NON-NLS-1$

					// Don't drop on the node we're dragging, or anywhere under it.
					var currentNode = treeNode;
					do {
						if (draggedUriSet[currentNode.item.uri]) return false;
						// Disable dropping in a frozen hierarchy or on a frozen product.
						// XXX: Note sure why it's looking at the node and not just the item.
						if (currentNode.frozenHierarchy || currentNode.item['ple:isFrozenTip']) return false; //$NON-NLS-1$
						currentNode = currentNode.parentTreeNode();
					} while (currentNode);

					return true;
				}, this);

				for (var i = 0; i < dropTargetNodes.length; i++) {
					var treeNode = dropTargetNodes[i];
					var pos = dojo.position(treeNode.rowNode);
					this._dropRegions.push({
						pos: pos,
						treeNode: treeNode
					});
				}
			},

			_initTopLevelDropRegions: function() {
				var notValidForTopLevel = dojo.some(this._draggedSites, function(site) {	
					return site.link || !site.parentProduct;
				});

				if (notValidForTopLevel) {
					this._topLevelDropRegions = [];
					return;
				}

				var treePos = dojo.position(this.domNode);

				var aboveRegion = dojo.mixin({}, treePos, {
					h: 50,
					y: treePos.y - 50
				});

				var belowRegion = dojo.mixin({}, treePos, {
					h: 50,
					y: treePos.y + treePos.h
				});

				this._topLevelDropRegions = [{
					pos: aboveRegion,
					moveToTop: true,
					isAboveTree: true
				},
				{
					pos: belowRegion,
					moveToTop: true
				}];
			},

			/**
			 * If we have hovered over and highlighted a drop region, this will clear it.
			 */
			_clearActiveDropRegion: function() {
				if (this._activeDropRegion) {
					if (this._activeDropRegion.treeNode) {
						dojo.removeClass(this._activeDropRegion.treeNode.rowNode, 'ple-AltTree-overDropTarget'); //$NON-NLS-1$
					}
					delete this._activeDropRegion;
				}

				if (this._dropBar) {
					this._dropBar.destroyRecursive();
					delete this._dropBar;
				}

				this._updateAvatarHint();
			},

			/**
			 * Given a mouse event (for `onmousemove`), determines of the event is "over"
			 * a drop region (a valid AltTreeNode). If so, adds a "highlight" class to 
			 * the nodes. Clears the class from old nodes if not over anything.
			 * @param  {mouse event} e a mouse onmousemove event.
			 */
			_highlightActiveDropRegion: function(e) {
				var vOffset = this._startScrollTop - this.autoHeightBox.box.scrollTop;

				// Check top and bottom regions to move products to the top level
				for (var j = 0; j < this._topLevelDropRegions.length; j++) {
					var topLevelRegion = this._topLevelDropRegions[j];

					if (util.isEventWithin(e, topLevelRegion.pos, vOffset)) {
						if (this._activeDropRegion !== topLevelRegion) {
							this._clearActiveDropRegion();
							this._activeDropRegion = topLevelRegion;

							var barTop = topLevelRegion.pos.y + vOffset;
							if (topLevelRegion.isAboveTree) barTop += topLevelRegion.pos.h;

							this._dropBar = com.ibm.relm.ple.web.dnd.DropBar({
								x: topLevelRegion.pos.x,
								y: barTop,
								w: topLevelRegion.pos.w
							});

							this._updateAvatarHint();
						}
						return;
					}
				}

				// Check if we're over a node in the tree. First, check if the
				// node/region s even in the visible, scollable region (i.e.,
				// not scrolled out of view).
				if (util.isEventWithin(e, this._visibleRegion)) {
					for (var i = 0; i < this._dropRegions.length; i++) {
						var region = this._dropRegions[i];
						if (util.isEventWithin(e, region.pos, vOffset)) {
							if (this._activeDropRegion !== region) {
								this._clearActiveDropRegion();
	
								this._activeDropRegion = region;
								dojo.addClass(region.treeNode.rowNode, 'ple-AltTree-overDropTarget'); //$NON-NLS-1$

								this._updateAvatarHint();
							}

							return;
						}
					}
				}

				// Not over any drop regions
				this._clearActiveDropRegion();
			},

			/**
			 * At the end of DnD this checks if we have an active drop target and does the move
			 * or reuse if so.
			 * @param  {[type]} e [description]
			 * @return {[type]}   [description]
			 */
			_onDrop: function(e) {
				if (!this._activeDropRegion) {
					this._cancelDrag();
					return;
				}

				// Do this at the end since it may hide error dialogs, which
				// can move positions and make the avatar position look wrong.
				MoveReusePlaceState.clearSavedSelection();

				var dropTargetTreeNode = this._activeDropRegion.treeNode;
				var doReuse = this._forceReuseOnDrag || this._preferReuseOnDrag;

				var moveToTop = this._activeDropRegion.moveToTop;
				this._cleanUpAfterDrag();

				if (moveToTop) {
					var removeFromParent = !doReuse;
					com.ibm.relm.ple.web.ui.productapi.addToTopOfTree(this._draggedSites, removeFromParent);
					return;
				}

				var toSite = TreeSelection.newSitesFromItemPaths([dropTargetTreeNode.getItemPath()])[0];
				var fromSites = this._draggedSites;

				if (doReuse) {
					MoveReusePlaceState.reuseAnything(fromSites, toSite);
				}
				else {
					MoveReusePlaceState.moveAnything(fromSites, toSite);
				}
			},

			/**
			 * Disconnects events and free resources associated with a drag. Call after
			 * a successful drop or cancel.
			 */
			_cleanUpAfterDrag: function() {
				this.autoHeightBox.disableAutoScroll();

				dojo.forEach(this._dragEventHandles, dojo.disconnect);
				this._dragEventHandles = [];
				this._avatar && this._avatar.destroyRecursive();
				delete this._avatar;
				delete this._avatarFactory;
				delete this._dropRegions;
				delete this._topLevelDropRegions;
				delete this._startX;
				delete this._startY;
				delete this._forceReuseOnDrag;
				delete this._preferReuseOnDrag;
				delete this._visibleRegion;
				delete this._startScrollTop;
				dojo.publish('ple.hide_drag_message'); //$NON-NLS-1$
				
				this._clearActiveDropRegion();

				dojo.removeClass(document, 'ple-AltTree-inDrag'); //$NON-NLS-1$
				dojo.removeClass(document.body, 'ple-AltTree-inDrag'); //$NON-NLS-1$

				dojo.publish('ple.product_tree.drag_did_end'); //$NON-NLS-1$
			},

			_cancelDrag: function() {
				var deadAvatar = this._avatar;
				delete this._avatar;

				var vOffset = this._startScrollTop - this.autoHeightBox.box.scrollTop;
				
				// Protect against calling this twice. Happens after a onclick and onmouseup both fire.
				deadAvatar && deadAvatar.moveToStartPosition(vOffset).then(function() {
					deadAvatar.destroyRecursive();
				});

				this._cleanUpAfterDrag();
			},

			setValueForAllItemsWithSameUriAndUpdate: function(uri, attribute, newValue) {
				this.walkAllTreeNodes(function(treeNode) {
					if (treeNode.item.uri == uri) {
						treeNode.item[attribute] = newValue;
						treeNode.updateDisplay();
					}
				});
			},

			onDblClickTreeNode: function(treeNode) {
				var link = treeNode.item;
				if (link.type !== 'link') return; //$NON-NLS-1$

				// Open links on double click for convenience.
				var w = window.open(link.link);
				w && w.focus();
			},

			/**
			 * Overrides default key handling to support using <spacebar> to toggle
			 * the selection check box. This matches the behavior in Plans.
			 * @param   {event} e 
			 */
			_onKeyDown: function(e) {
				if (e.keyCode === dojo.keys.SPACE) {
					// Only works if the row (label really) is focused.
					var focusedTreeNode = this.getFocusedTreeNode();
					if (focusedTreeNode && focusedTreeNode.labelNode == e.target) {
						dojo.stopEvent(e);
						focusedTreeNode.toggleSelected(e);
						return;
					}
				}

				this.inherited(arguments);
			},

			_updateTreeSelection: function() {
				var paths = this.getAllSelectedItemPaths();
				TreeSelection.store = this.store;

				try {
					TreeSelection.setSelectedSitesFromItemPaths(paths);
				}
				catch (e) {
					console.debug(e);
				}
			},

			refresh: function() {
				var deferred = new dojo.Deferred();
				var tree = this;

				// `parameters` is only used if the filter is a URI to a product grouping.
				var content = {
					filter: this.rootQuery,
					parameters: dojo.toJson(SystemParameterUtil.getSystemParameterValues()),
					loadedPaths: dojo.toJson(tree.getLoadedPaths())
				};

				jazz.client.xhrPost({
					handleAs: 'json', //$NON-NLS-1$
					url: net.jazz.ajax._contextRoot + '/ple/update_tree', //$NON-NLS-1$
					content: content,

					load: function (rootItems) {
						tree.setRootItems(rootItems);
						deferred.callback();
					},

					error: function(error, ioArgs) {
						console.error('Failed to refresh product tree'); //$NON-NLS-1$
						deferred.errback(error, ioArgs);
					}
				});

				return deferred;
			}
		}
	);

})();
